systemConfigurationModule
    .controller('DeclarationFormConfigurationController', ['$scope', '$log', '$timeout', '$translate', '$rootScope', '$interval', '$compile', '$uibModal', '$q', 'SweetAlert', 'templateGroupService',
        'templateService', 'templateFormulaService', 'stdAccountService', 'spreadJsTipService', 'enums', 'KeyValueConfigService', 'modelConfigurationService', 'industryService',
        'formulaService', 'serviceTypeService', 'mentioUtil', 'region',
        function ($scope, $log, $timeout, $translate, $rootScope, $interval, $compile, $uibModal, $q, SweetAlert, templateGroupService, templateService, templateFormulaService,
                  stdAccountService, spreadJsTipService, enums, KeyValueConfigService, modelConfigurationService, industryService, formulaService, serviceTypeService, mentioUtil, region) {
            'use strict';

            var keyValueExp = /@\S*/g;
            var keyValueIllegalExp = /[^a-zA-z0-9\.]+/;
            var blankExp = /\s/g;
            var notDuplicateOperatorExp = /(,[\*\/]+)|([\+\-\*\/]+)/g;
            var missingExpressionExp = /([\+\-\*\/,]+$)|([\+\-\*\/]+[\),])/;
            var missingOperatorExp = /([^\+\-\*\/,\(]+@)|(@[^\+\-\*\/,\)]+)/;

            var _spread = null;

            // 本页面常量
            var thisConstant = {
                deleteClass: 'material-icons delete',
                editClass: 'material-icons edit',
                findAddClass: 'fa fa-search-plus organge-color',
            };


            var serviceTypeMapping = {
                VAT: $translate.instant('VAT'),
                RPT: $translate.instant('RPT'),
                CIT: $translate.instant('CIT'),
                DTA: $translate.instant('DTA'),
                CF: $translate.instant('CashFlow'),
                AssetsManage: $translate.instant('AssetsManage')
            };

            // Config if demo, if current service is only demo, will not validate the formula
            var demoMapping = {
                VAT: false,
                RPT: true,
                CIT: true,
                DTA: true,
                CF: true,
                AssetsManage: true
            };

            var filingTypeList = _.map(constant.payTaxTypeList, function (x) {
                return {
                    value: x.value,
                    name: $translate.instant(x.name)
                };
            });

            var citReportTypeList = _.map(constant.citReportTypeList, function (x) {
                return {
                    value: x.value,
                    name: $translate.instant(x.name)
                };
            });

            var cfReportTypeList = _.map(constant.cfReportTypeList, function (x) {
                return {
                    value: x.value,
                    name: $translate.instant(x.name)
                };
            });

            var hidePanel = function () {
                $('#editReportFormulaPop').hide('normal');
            };

            var togglePanel = function () {
                if ($('#editReportFormulaPop #editReportFormulaContent').css('display') == 'none') {
                    $('#editReportFormulaPop').height(610);
                } else {
                    $('#editReportFormulaPop').height(40);
                }
                $timeout(function () {
                    $('#editReportFormulaPop #editReportFormulaContent').toggle('normal');
                }, 50);
            };

            //显示纳税申报表的单元格信息详情
            var showConfigPage = function () {
                var h = -3 * $(window).height() / 4;
                var w = 1 * $(window).width() / 4
                $('#editReportFormulaPop').css({
                    'position': 'relative', 'z-index': '1', top: h + 'px', left: w + 'px'
                });
                $('#editReportFormulaPop').show('normal');
            };

            //加载'报表模板'源数据
            var loadTemplateGroup = function () {

                if ($scope.curIndustry && $scope.curIndustry.id) {

                    var payTaxType = null;
                    if ($scope.curServiceTypeId !== enums.serviceType.CIT && $scope.curServiceTypeId !== enums.serviceType.Others) {
                        if ($scope.payTaxType && $scope.payTaxType.value) {
                            payTaxType = $scope.payTaxType.value;
                        }
                        else {
                            return;
                        }
                    }

                    templateGroupService.getByIndustry($scope.curServiceTypeId, payTaxType, $scope.curIndustry.id)
                        .success(function (templateGroupData) {
                            templateGroupData = _.sortBy(templateGroupData, function (num) {
                                return -num.isSystemType;
                            });
                            $scope.templateGroups = templateGroupData;

                            var defaultValue = templateGroupData[0];
                            if ($scope.curTemplateGroup && $scope.curTemplateGroup.name) {
                                var find = _.find(templateGroupData, function (row) {
                                    return row.name === $scope.curTemplateGroup.name || row.id === $scope.curTemplateGroup.id;
                                });

                                if (find) {
                                    defaultValue = find;
                                    $scope.templateGroupId = find.id;
                                }
                            }

                            dxControl.renderTemplateGroupDropDown(templateGroupData, defaultValue);

                            if (templateGroupData && templateGroupData.length > 0) {
                                loadReportType();
                            } else {
                                $('#dx-select-template-groups').dxDropDownBox('instance').option('dataSource', []);
                                $('#dx-select-template').dxDropDownBox('instance').option('dataSource', []);
                            }
                        });

                }
            };

            //加载'报表类型'源数据
            var loadReportType = function () {
                var defaultReportTypeOpt = {
                    displayExpr: 'name',
                    valueExpr: "name",
                    itemTemplate: 'reportTypeItem',
                    onSelectionChanged: function (e) {
                        $scope.reportType = e.selectedItem;
                        listTemplates();
                    }
                };

                if ($scope.curTemplateGroup && $scope.curTemplateGroup.id) {
                    if ($scope.curServiceTypeId === enums.serviceType.CIT) {
                        $scope.detailReportTypeList = angular.copy(defaultReportTypeOpt);
                        $scope.detailReportTypeList.dataSource = new DevExpress.data.ArrayStore({
                            data: citReportTypeList
                        });
                        $scope.detailReportTypeList.value = citReportTypeList[0].name;
                        $('#dx-select-report-type').dxSelectBox('instance').option($scope.detailReportTypeList);
                        $scope.reportType = citReportTypeList[0];
                        listTemplates();
                    } else if ($scope.curServiceTypeId === enums.serviceType.CF) {
                        $scope.detailReportTypeList = angular.copy(defaultReportTypeOpt);
                        $scope.detailReportTypeList.dataSource = new DevExpress.data.ArrayStore({
                            data: cfReportTypeList
                        });
                        $scope.detailReportTypeList.value = cfReportTypeList[1].name;
                        $('#dx-select-report-type').dxSelectBox('instance').option($scope.detailReportTypeList);
                        $scope.reportType = cfReportTypeList[1];
                        listTemplates();
                    } else if ($scope.curServiceTypeId === enums.serviceType.Others) {
                        $scope.detailReportTypeList = angular.copy(defaultReportTypeOpt);
                        $scope.detailReportTypeList.dataSource = new DevExpress.data.ArrayStore({
                            data: citReportTypeList
                        });
                        $scope.detailReportTypeList.value = citReportTypeList[0].name;
                        $('#dx-select-report-type').dxSelectBox('instance').option($scope.detailReportTypeList);
                        $scope.reportType = citReportTypeList[0];
                        listTemplates();
                    } else {
                        $('#dx-select-report-type').dxSelectBox('instance').option('dataSource', []);
                        $scope.reportType = null;
                        listTemplates();
                    }
                }
            };

            var loadIndustry = function () {
                industryService.getAllAvailableIndustry().success(function (industryData) {
                    if (industryData && industryData.length > 0) {
                        if (region !== 'zh-CN') {
                            angular.forEach(industryData, function (x) {
                                switch (x.id) {
                                    case '0':
                                        x.name = $translate.instant('GeneralIndustry');
                                        break;
                                    case '1':
                                        x.name = $translate.instant('IndustryIndustry');
                                        break;
                                    case '10':
                                        x.name = $translate.instant('AssetManagement');
                                        break;
                                    case '13':
                                        x.name = $translate.instant('MobileManufacturingIndustry');
                                        break;
                                    case '14':
                                        x.name = $translate.instant('FinancialServiceIndustry');
                                        break;
                                    case '20':
                                        x.name = $translate.instant('ManufacturingIndustry');
                                        break;
                                    case '23':
                                        x.name = $translate.instant('EstateIndustry');
                                        break;
                                    default:
                                        x.name = x.englishName;
                                        break;
                                }
                            });
                        }

                        //配置行业下拉列表
                        $scope.dataSourceIndustryList = {
                            dataSource: new DevExpress.data.ArrayStore({
                                data: industryData
                            }),
                            displayExpr: 'name',
                            valueExpr: "id",
                            value: industryData[0].id,
                            itemTemplate: 'industryItem',
                            onItemClick: function (e) {
                                $scope.curIndustry = e.itemData;
                                loadTemplateGroup();
                            },
                            onInitialized: function (e) {
                                $scope.dataSourceIndustryListInstance = e.component;
                            }
                        }
                        $scope.curIndustry = industryData[0];
                        $('#dx-select-industry').dxSelectBox('instance').option('value', $scope.curIndustry.id);
                        loadTemplateGroup();
                    }
                });
            };

            var loadServiceType = function () {
                serviceTypeService.getServiceTypeList().success(function (data) {
                    if (data && data.length > 0) {
                        //配置服务类型列表
                        $scope.serviceTypeList = _.chain(data).sortBy(function (s) {
                            return Number(s.id);
                        }).map(function (s) {
                            var rtn = null;
                            if (serviceTypeMapping.hasOwnProperty(s.prefix)) {
                                rtn = {
                                    id: s.id,
                                    name: serviceTypeMapping[s.prefix]
                                };

                                if (demoMapping.hasOwnProperty(s.prefix)) {
                                    rtn.isDemo = demoMapping[s.prefix];
                                }
                            }

                            return rtn;
                        }).reject(function (s) {
                            return _.isEmpty(s);
                        }).value();

                        $scope.serviceTypeList.push({
                            id: '99',
                            isDemo: true,
                            name: $translate.instant('Others')
                        });

                        $scope.curServiceTypeId = $scope.serviceTypeList[0].id;
                        $scope.curServiceTypeName = $scope.serviceTypeList[0].name;
                        $scope.isDemo = $scope.serviceTypeList[0].isDemo;

                        $scope.payTaxType = filingTypeList[0];
                        $('#dx-select-pay-tax-type').dxSelectBox('instance').option('value', $scope.payTaxType.name);

                        loadIndustry();
                    }
                });
            };

            var getServiceTypeList = function () {
                var rtn = [];
                if (!_.isEmpty($scope.serviceTypeList)) {
                    rtn = _.reject($scope.serviceTypeList, function (s) {
                        return s.id === $scope.curServiceTypeId;
                    });
                }

                return rtn;
            };

            var toggleServiceType = function (s) {
                $scope.curServiceTypeId = s.id;
                $scope.curServiceTypeName = s.name;
                $scope.isDemo = s.isDemo;
                $scope.payTaxType = filingTypeList[0];
                $('#dx-select-pay-tax-type').dxSelectBox('instance').option('value', $scope.payTaxType.name);

                loadIndustry();
            };

            var hasError = function () {
                return $scope.editReportFormulaForm &&
                    ($scope.editReportFormulaForm.formula && !_.isEmpty($scope.editReportFormulaForm.formula.$error) ||
                        $scope.editReportFormulaForm.voucher && !_.isEmpty($scope.editReportFormulaForm.voucher.$error) ||
                        $scope.editReportFormulaForm.invoice && !_.isEmpty($scope.editReportFormulaForm.invoice.$error) ||
                        $scope.editReportFormulaForm.model && !_.isEmpty($scope.editReportFormulaForm.model.$error) ||
                        $scope.editReportFormulaForm.validation && !_.isEmpty($scope.editReportFormulaForm.validation.$error))
            };

            //加载'报表'源数据
            var listTemplates = function () {
                templateGroupService.getTemplateList($scope.curTemplateGroup.id, $scope.reportType && $scope.reportType.value).success(function (templateData) {

                    templateData = _.filter(templateData, function (row) {
                        return row.isActiveAssociation;
                    });

                    var defaultValue = templateData[0];
                    if ($scope.currentTemplate && ($scope.currentTemplate.name || $scope.currentTemplate.id)) {
                        var find = _.find(templateData, function (row) {
                            return row.name === $scope.currentTemplate.name || row.id === $scope.currentTemplate;
                        });

                        if (find) {
                            defaultValue = find;
                        }
                    }

                    $scope.templates = templateData; //当前组下的所有模板数据

                    dxControl.renderTemplateDropDown(templateData, defaultValue);
                    //配置报表下拉框
                    if (templateData && templateData.length > 0) {
                        $scope.ToggleSaveAs = showSavebutton();
                    }
                });
            };
            //模板不是默认模板才可以编辑和保存 
            var showSavebutton = function () {
                var isShow = false;
                if ($scope.curTemplateGroup && $scope.curTemplateGroup !== undefined) {
                    var isSystemType = $scope.curTemplateGroup.isSystemType;
                    if (!isSystemType) {
                        isShow = true;
                    }
                }

                return isShow;
            };

            var setCellFormula = function (templateID) {
                $log.debug('current templateID:' + templateID + ', getting template formula date...');

                //根据templateId获取公式配置数据
                templateFormulaService.getCellTemplateConfigList(templateID).success(function (getResult) {

                    if (getResult.result === false) {
                        SweetAlert.warning($translate.instant(getResult.resultMsg));
                        return;
                    }

                    if (!getResult.data) {
                        getResult.data = [];
                    }

                    $scope.templateFormulaList = getResult.data.filter(function (item) {
                        return !item.isReadOnly;
                    });

                    refreshSpreadUI();
                });

            };

            // 刷新选中的过滤凭证科目代码
            var refreshVoucherFilter = function () {
                if (!$scope.editModel.accountCodes) {
                    $scope.editModel.accountCodes = [];
                }

                if (!_.isEmpty($scope.editModel.accountCodes)) {
                    var selectedData = _.filter($scope.accountDataSource, function (data) {
                        return $scope.editModel.accountCodes.indexOf(data.code) > -1;
                    });

                    $scope.selectedAccount = selectedData;
                } else {
                    $scope.selectedAccount = [];
                }

                $scope.refreshGrids();
            };

            // 刷新选中的关联模型
            var refreshModelFilter = function () {
                if (!$scope.editModel.modelIDs) {
                    $scope.editModel.modelIDs = [];
                }

                if (!_.isEmpty($scope.editModel.modelIDs)) {
                    var selectedData = _.filter($scope.modelDataSource, function (data) {
                        return $scope.editModel.modelIDs.indexOf(data.id) > -1;
                    });

                    $scope.selectedModel = selectedData;
                } else {
                    $scope.selectedModel = [];
                }

                $scope.refreshGrids();
            };

            // 获取发票设置
            var getInvoiceFilter = function () {
                var items = [{
                    type: 'PleaseSelect',
                    value: 0,
                    categoryOptions: [],
                    rateOptions: [],
                    amountTypeOptions: [
                        {name: 'PleaseSelect', value: 0},
                        {name: 'TotalAmount', value: 1},
                        {name: 'TaxAmount', value: 2},
                        {name: 'CopyNumber', value: 3}
                    ]
                },
                    {
                        type: 'Income',
                        value: 1,
                        categoryOptions: _.chain(enums.invoiceType).pairs().map(function (it) {
                            return {key: $translate.instant(it[0]), value: it[1], ticked: false};
                        }).value(),
                        rateOptions: [
                            {key: '17%', value: '17%', ticked: false},
                            {key: '13%', value: '13%', ticked: false},
                            {key: '11%', value: '11%', ticked: false},
                            {key: '6%', value: '6%', ticked: false},
                            {key: '5%', value: '5%', ticked: false},
                            {key: '4%', value: '4%', ticked: false},
                            {key: '3%', value: '3%', ticked: false},
                            {key: '1.5%', value: '1.5%', ticked: false}
                        ],
                        amountTypeOptions: [
                            {name: 'PleaseSelect', value: 0},
                            {name: 'TotalAmount', value: 1},
                            {name: 'TaxAmount', value: 2},
                            {name: 'CopyNumber', value: 3}
                        ]
                    },
                    {
                        type: 'Output',
                        value: 2,
                        categoryOptions: _.chain(enums.outputInvoiceType).pairs().map(function (it) {
                            return {key: $translate.instant(it[0]), value: it[1], ticked: false};
                        }).value(),
                        rateOptions: [
                            {key: '17%', value: '17%', ticked: false},
                            {key: '13%', value: '13%', ticked: false},
                            {key: '11%', value: '11%', ticked: false},
                            {key: '6%', value: '6%', ticked: false},
                            {key: '5%', value: '5%', ticked: false},
                            {key: '4%', value: '4%', ticked: false},
                            {key: '3%', value: '3%', ticked: false},
                            {key: '1.5%', value: '1.5%', ticked: false}
                        ],
                        amountTypeOptions: [
                            {name: 'PleaseSelect', value: 0},
                            {name: 'TotalAmount', value: 1},
                            {name: 'TaxAmount', value: 2},
                            {name: 'CopyNumber', value: 3}
                        ]
                    },
                    {
                        type: 'CustomsAudit',
                        value: 3,
                        categoryOptions: [],
                        rateOptions: [],
                        amountTypeOptions: [
                            {name: 'PleaseSelect', value: 0},
                            {name: 'TotalAmount', value: 1},
                            {name: 'TaxAmount', value: 2},
                            {name: 'CopyNumber', value: 3}
                        ]
                    }
                ];

                return items;
            };

            // 选中某一发票类型级联更新税率与金额类型列表
            var selectInvoiceOption = function (editModel, invoiceType, isInit) {
                var selectedInvoice = _.where($scope.invoiceOptions, {value: invoiceType});
                var op = selectedInvoice.length === 0 ? $scope.invoiceOptions[0] : selectedInvoice[0];

                // 如果获取到的editModel.invoiceCategory为[ 99 ],代表发票类型全选；
                // 否则只勾选editModel.invoiceCategory数组中有的税率
                var categoryOptions = _.map(op.categoryOptions, function (o) {
                    var rtn = angular.copy(o);
                    if (isInit && editModel.invoiceCategory &&
                        (_.contains(editModel.invoiceCategory, Number(constant.selectAllValue)) || _.contains(editModel.invoiceCategory, rtn.value))) {
                        rtn.ticked = true;
                    } else {
                        rtn.ticked = false;
                    }

                    return rtn;
                });

                // 如果获取到的editModel.taxRate为[ '99' ],代表税率全选；
                // 否则只勾选editModel.taxRate数组中有的税率
                var rateOptions = _.map(op.rateOptions, function (o) {
                    var rtn = angular.copy(o);
                    if (isInit && editModel.taxRate &&
                        (_.contains(editModel.taxRate, constant.selectAllValue) || _.contains(editModel.taxRate, rtn.value))) {
                        rtn.ticked = true;
                    } else {
                        rtn.ticked = false;
                    }

                    return rtn;
                });

                var choseAmoutType = isInit && editModel.invoiceAmountType && editModel.invoiceAmountType > 0 ?
                    _.where(op.amountTypeOptions, {value: editModel.invoiceAmountType})[0] : op.amountTypeOptions[0];

                editModel.chosenInvoiceOption = {
                    type: op.type,
                    value: op.value,
                    categoryOptions: categoryOptions,
                    rateOptions: rateOptions,
                    invoiceAmountType: choseAmoutType
                };
            };

            var resetFormHeight = function () {
                var sumHeight = $(document).height();
                var headerHeight = $('.menu-header').height();
                var topHeight = $('.templates-top').height();

                var bodyHeigh = $('.templates-body').css('height', (sumHeight - headerHeight - topHeight - 50) + 'px');
                $('#report-view').css('height', (sumHeight - headerHeight - topHeight - 50) + 'px');
            };

            var loadSheet = function (template) {
                $scope.currentTemplate = template;

                $('#report-view').html('');

                templateService.renderSpreadExcelSimple('report-view', template.id).then(function (spread) {
                    $(window).on('resize', function () {
                        resetFormHeight();
                        $timeout(function () {
                            if (spread && spread.refresh) {
                                spread.refresh();
                            }
                        }, 500);
                    });

                    _spread = spread;
                    var activeSheet;
                    if(constant.regesterInformation.active){
                        activeSheet = _spread.getActiveSheet();
                    }
                    else{
                        activeSheet = _spread.getSheet(1);
                    }
                    //设置整个sheet不可编辑
                    activeSheet.isProtected = true;
                    activeSheet.options.protectionOption = {
                        allowEditObjects: false
                    };

                    $scope.activeSheet = activeSheet;

                    setCellFormula(template.id);

                    //给单元格增加双击事件
                    activeSheet.bind(GC.Spread.Sheets.Events.CellDoubleClick, function (sender, args) {
                        var sheet = args.sheet;
                        var cell = sheet.getCell(args.row, args.col);
                        showEditReportFormulaPop(template, args.row, args.col, true);
                    });

                    //给单元格增加单击事件
                    activeSheet.bind(GC.Spread.Sheets.Events.CellClick, function (sender, args) {
                        var sheet = args.sheet;
                        var cell = sheet.getCell(args.row, args.col);
                        showEditReportFormulaPop(template, args.row, args.col, false);
                    });
                });
            };

            var processSpreadContextMenu = function (e) {
                // move the context menu to the position of the mouse point
                var sheet = _spread.getActiveSheet(),
                    target = getHitTest(e.pageX, e.pageY, sheet),
                    hitTestType = target.hitTestType,
                    row = target.row,
                    col = target.col,
                    selections = sheet.getSelections();

                var isHideContextMenu = false;
                if ($scope.currentTemplate.isSystemType) {
                    isHideContextMenu = true;
                }
                else if (hitTestType === GC.Spread.Sheets.SheetArea.colHeader) {
                    if (getCellInSelections(selections, row, col) === null) {
                        sheet.setSelection(-1, col, sheet.getRowCount(), 1);
                    }
                    if (row !== undefined && col !== undefined) {
                        $(".context-header").show();
                        $(".context-cell").hide();
                    }
                } else if (hitTestType === GC.Spread.Sheets.SheetArea.rowHeader) {
                    if (getCellInSelections(selections, row, col) === null) {
                        sheet.setSelection(row, -1, 1, sheet.getColumnCount());
                    }
                    if (row !== undefined && col !== undefined) {
                        $(".context-header").show();
                        $(".context-cell").hide();
                    }
                } else if (hitTestType === GC.Spread.Sheets.SheetArea.viewport) {
                    if (getCellInSelections(selections, row, col) === null) {
                        sheet.clearSelection();
                        sheet.setActiveCell(row, col);
                    }
                    if (row !== undefined && col !== undefined) {
                        $(".context-header").hide();
                        $(".context-cell").hide();
                    } else {
                        isHideContextMenu = true;
                    }
                } else if (hitTestType === GC.Spread.Sheets.SheetArea.corner) {
                    sheet.setSelection(-1, -1, sheet.getRowCount(), sheet.getColumnCount());
                    if (row !== undefined && col !== undefined) {
                        $(".context-header").hide();
                        $(".context-cell").show();
                    }
                }

                var $contextMenu = $("#spreadContextMenu");
                $contextMenu.data("sheetArea", hitTestType);
                if (isHideContextMenu) {
                    hideSpreadContextMenu();
                } else {
                    $contextMenu.css({left: e.pageX, top: e.pageY});
                    $contextMenu.show();

                    $(document).on("click.contextmenu", function () {
                        if ($(event.target).parents("#spreadContextMenu").length === 0) {
                            hideSpreadContextMenu();
                        }
                    });
                }
            };

            var processContextMenuClicked = function () {
                var action = $(this).data("action");
                var sheet = _spread.getActiveSheet();
                var sheetArea = $("#spreadContextMenu").data("sheetArea");

                hideSpreadContextMenu();

                switch (action) {
                    case "applyRowColName":
                        setRowColName(sheet);
                        break;
                    default:
                        break;
                }
            };

            var hideSpreadContextMenu = function () {
                $("#spreadContextMenu").hide();
                $(document).off("click.contextmenu");
            };

            var getCellInSelections = function (selections, row, col) {
                var count = selections.length, range;
                for (var i = 0; i < count; i++) {
                    range = selections[i];
                    if (range.contains(row, col)) {
                        return range;
                    }
                }
                return null;
            };

            var getHitTest = function (pageX, pageY, sheet) {
                var offset = $("#report-view").offset(),
                    x = pageX - offset.left,
                    y = pageY - offset.top;
                return sheet.hitTest(x, y);
            };

            var setRowColName = function (sheet) {
                var candidateNames = getSelectedCells();
                candidateNames.forEach(function (c) {
                    var cell = sheet.getCell(c.rowIndex, c.columnIndex);
                    if (cell) {
                        var cellValue = cell.value();
                        if (!_.isEmpty(cellValue)) {
                            cellValue = cellValue.replaceToEnd('\r', '').replaceToEnd('\n', '').trim();
                        }

                        c.value = cellValue;
                    }
                });
                candidateNames = _.reject(candidateNames, function (x) {
                    return _.isEmpty(x.value);
                });
                templateService.setRowColName($scope.currentTemplate.id, candidateNames).then(function (data) {
                    if (data) {
                        SweetAlert.success($translate.instant('CellRowColNameSetSuccess'));
                        //var template = _.findWhere($scope.templates, { id: $scope.currentTemplate.id })
                        loadSheet($scope.currentTemplate);
                    }
                });
            };

            var getSelectedCells = function () {
                var activeSheet = _spread.getActiveSheet();
                var selection = activeSheet.getSelections();
                var spans = activeSheet.getSpans();
                var exceptedCells = [];
                for (var k = 0; k < spans.length; k++) {
                    for (var i = 0; i < spans[k].rowCount; i++) {
                        for (var j = 0; j < spans[k].colCount; j++) {
                            if (i > 0 || j > 0) {
                                exceptedCells.push({rowIndex: spans[k].row + i, columnIndex: spans[k].col + j});
                            }
                        }
                    }
                }

                var validPosition = [];
                for (var k = 0; k < selection.length; k++) {
                    var n = selection[k];
                    var startRow = n.row >= 0 ? n.row : 0;
                    var startCol = n.col >= 0 ? n.col : 0;

                    if (n.rowCount >= 1 && n.colCount >= 1) {
                        for (var i = 0; i < n.rowCount; i++) {
                            for (var j = 0; j < n.colCount; j++) {
                                var row = startRow + i;
                                var col = startCol + j;
                                if (_.every(exceptedCells, function (c) {
                                        return c.rowIndex !== row || c.columnIndex !== col;
                                    })) {
                                    validPosition.push({rowIndex: row, columnIndex: col});
                                }
                            }
                        }
                    }
                }

                return validPosition;
            };

            var confirmWarningWindow = function (title, text) {
                var deferred = $q.defer();
                SweetAlert.swal({
                        title: title,
                        text: text,
                        type: "warning",
                        showCancelButton: true,
                        confirmButtonColor: "#DD6B55",
                        allowOutsideClick: false,
                        confirmButtonText: $translate.instant('Confirm'),
                        cancelButtonText: $translate.instant('Cancel'),
                        closeOnConfirm: true,
                        closeOnCancel: true
                    },
                    function (isConfirm) {
                        deferred.resolve(isConfirm);
                    });

                return deferred.promise;
            };

            var showEditReportFormulaPop = function (template, row, col, isDoubleClick) {

                if ($scope.templateFormulaList && $scope.templateFormulaList.length > 0) {

                    var formulaModel = _.find($scope.templateFormulaList, function (num) {
                        return num.templateID == template.id && num.rowIndex === row && num.columnIndex === col
                    });
                    if (formulaModel) {
                        $scope.editModel = angular.copy(formulaModel);
                        $scope.editModel.cell = PWC.numToExcelChar(row, col);

                        $scope.editModel.isSystemType = template.isSystemType;
                        $scope.selectedAccount = [];
                        $scope.selectedModel = [];

                        changeTab($scope.editModel);

                        refreshVoucherFilter();
                        refreshModelFilter();

                        // Select default invoice option if no related info in editModel
                        selectInvoiceOption($scope.editModel, $scope.editModel.invoiceType, true);

                        // invoiceOption: Used to save invoice tax rate and amount type options temporarily
                        $scope.invoiceOption = _.findWhere($scope.invoiceOptions, {type: $scope.editModel.chosenInvoiceOption.type});

                        $scope.mentionApi.triggerValidator($scope.editModel.formula, $scope.editReportFormulaForm.formula);
                        $scope.mentionApiForValidation.triggerValidator($scope.editModel.validation, $scope.editReportFormulaForm.validation);
                        asyncFormulaValidate();
                        asyncFormulaValidateForValidation();
                        if (isDoubleClick) {
                            var cell = $scope.activeSheet.getCell(row, col);
                            if (cell.value() === '——') {
                                confirmWarningWindow($translate.instant('ReportCellUpdateWarning'), '').then(function (data) {
                                    if (data) {
                                        showConfigPage();
                                    }
                                });
                            } else {
                                showConfigPage();
                            }
                        }
                        //$('#editReportFormulaPop').show('normal');
                        $(".editReportFormulaPop-modal-content").css('top', '0px');
                        $(".editReportFormulaPop-modal-content").css('left', '0px');
                        $(".editReportFormulaPop-modal-content").draggable({
                            cursor: "cursor",
                            scroll: true,
                            cancel: 'input, label, .btn, .form-control, .close, div.label, .has-error'
                        });
                    }
                }
            };

            var changeTab = function (editModel) {
                if (_.isEmpty(editModel)) {
                    // Do nothing
                }
                else if (editModel.hasFormula) {
                    $scope.selectedTab = 0;
                }
                else if (editModel.hasInvoice || editModel.hasVoucher || editModel.hasKeyIn) {
                    $scope.selectedTab = 1;
                }
                else if (editModel.hasModel) {
                    $scope.selectedTab = 2;
                }
                else if (editModel.hasValidation) {
                    $scope.selectedTab = 3;
                }
                else { // all options are not configured
                    $scope.selectedTab = 0;
                }
            };

            var initAccountSelector = function () {
                $scope.selectedAccount = [];

                $scope.voucherGridOptions = {
                    bindingOptions: {
                        dataSource: {
                            deep: false,
                            dataPath: 'selectedAccount'
                        }
                    },
                    columns: [
                        {caption: $translate.instant('SubjectCodeCol'), dataField: 'code'},
                        {caption: $translate.instant('SubjectNameCol'), dataField: 'name'},
                        {caption: $translate.instant('SubjectDirectionCol'), dataField: 'directionName'},
                        {caption: $translate.instant('SubjectTypeCol'), dataField: 'AcctPropName'},
                        {
                            alignment: 'center',
                            width: '40px',
                            cellTemplate: function (container, options) {
                                var compileInlineBtn = $compile('<a class="dx-link dx-link-delete"\
ng-show="hasEditPermission && ToggleSaveAs && editModel.hasVoucher"/>');
                                compileInlineBtn($scope)
                                    .html('<i class="fa fa-trash" aria-hidden="true"></i>')
                                    .on('dxclick', function () {
                                        options.component.removeRow(options.rowIndex);
                                    })
                                    .appendTo(container);
                            }
                        }

                    ],
                    loadPanel: {
                        enabled: false
                    },
                    paging: {
                        enabled: false
                    },
                    scrolling: {
                        mode: "virtual"
                    },
                    sorting: {
                        mode: "none"
                    },
                    height: '100%',
                    showBorders: true,
                    showRowLines: true,
                    showColumnLines: true,
                    onRowRemoving: function (e) {
                        var idx = $scope.selectedAccount.indexOf(e.data);
                        if (idx >= 0) {
                            $scope.selectedAccount.splice(idx, 1);
                        }
                    }
                };
            };

            var initModelSelector = function () {
                $scope.selectedModel = [];

                $scope.modelGridOptions = {
                    bindingOptions: {
                        dataSource: {
                            deep: false,
                            dataPath: 'selectedModel'
                        }
                    },
                    columns: [
                        {caption: $translate.instant('ModelCode'), dataField: 'code', visible: false},
                        {caption: $translate.instant('ModelName'), dataField: 'name'},
                        {caption: $translate.instant('ModelDescription'), dataField: 'description'},
                        {caption: $translate.instant('ModelFeature'), dataField: 'featureName'},
                        {caption: $translate.instant('ModelType'), dataField: 'modelTypeName'},
                        {caption: $translate.instant('ModelOrganization'), dataField: 'organizationName'},
                        {
                            alignment: 'center',
                            width: '40px',
                            cellTemplate: function (container, options) {
                                var compileInlineBtn = $compile('<a class="dx-link dx-link-delete"\
ng-show="hasEditPermission && ToggleSaveAs && editModel.hasModel"/>');
                                compileInlineBtn($scope)
                                    .html('<i class="fa fa-trash" aria-hidden="true"></i>')
                                    .on('dxclick', function () {
                                        options.component.removeRow(options.rowIndex);
                                    })
                                    .appendTo(container);
                            }
                        }
                    ],
                    loadPanel: {
                        enabled: false
                    },
                    paging: {
                        enabled: false
                    },
                    scrolling: {
                        mode: "virtual"
                    },
                    sorting: {
                        mode: "none"
                    },
                    height: '100%',
                    showBorders: true,
                    showRowLines: true,
                    showColumnLines: true,
                    onRowRemoving: function (e) {
                        var idx = $scope.selectedModel.indexOf(e.data);
                        if (idx >= 0) {
                            $scope.selectedModel.splice(idx, 1);
                        }
                    }
                };
            };

            var showSelectAccountPop = function () {
                $scope.isShowAccounts = true;
            };

            var showSelectModelPop = function () {
                $scope.isShowModels = true;
            };

            /*********************** Formula Editor ***************************************/
            var showSelectFormulaPop = function () {
                $scope.operateType = constant.Operation.Add;
            };

            var showSelectKeyValuePop = function (keyCode) {
                if (!_.isEmpty(keyCode)) {
                    $scope.defaultKeyCode = keyCode;
                    $scope.isUpdate = true;
                }
                else {
                    $scope.defaultKeyCode = null;
                    $scope.isUpdate = false;
                }

                $scope.isShowKeyValue = true;
            };

            var editFormulaParamOrKeyValue = function (e) {
                if ($scope.hasEditPermission && $scope.ToggleSaveAs && $scope.editModel.hasFormula) {
                    $scope.isUpdate = true;
                    if (e && e.tagInfo) {
                        if (e.tagType === '0') { // key value selector
                            $scope.focusTag = e.tagInfo;
                            showSelectKeyValuePop(e.name);
                        }
                        else if (e.tagType === '1') { // formula parameter editor
                            $scope.focusTag = e.tagInfo;
                            $scope.editFormula = e.name;
                            $scope.editParams = e.formulaParams;
                            $scope.isShowParam = true;
                        }
                    }
                }
            };

            var selectParam = function (formula) {
                $scope.isUpdate = false;
                $scope.editFormula = formula.code;
                $scope.editParams = ')';
                $scope.isShowParam = true;
            };

            var cancelHideTranslator = function () {
                $scope.cancelHide = true;
            };

            var timerHideTranslator = function () {
                $scope.cancelHide = false;
                $timeout(function () {
                    if (!$scope.cancelHide && $scope.firedType === '1') {
                        $scope.firedName = '';
                        $scope.firedParams = '';
                    }
                }, 500);
            };

            // 公式验证
            var mentionMenuValidator = function (ngModelValue, ngModel) {
                ngModel.$setValidity('requiredValidator', true);
                ngModel.$setValidity('keyValueNotEmptyValidator', true);
                ngModel.$setValidity('keyValueSeparatorValidator', true);
                ngModel.$setValidity('operatorDuplicateValidator', true);
                ngModel.$setValidity('missingExpressionValidator', true);
                ngModel.$setValidity('missingOperatorValidator', true);

                if (!$scope.hasEditPermission || !$scope.ToggleSaveAs || !$scope.editModel.hasFormula) {
                    return true;
                }

                if (PWC.isNullOrEmpty(ngModelValue)) {
                    ngModel.$setValidity('requiredValidator', false);
                    return false;
                }

                var keyValueMatches = ngModelValue.match(keyValueExp);
                if (!_.isEmpty(keyValueMatches)) {
                    for (var i = 0; i < keyValueMatches.length; i++) {
                        var m = keyValueMatches[i];
                        m = m.substring(1); // Remove prefix @

                        if (m.length === 0) { // The key value is empty
                            ngModel.$setValidity('keyValueNotEmptyValidator', false);
                            return false;
                        }

                        // Check if suffix blank exists for each key value
                        if (keyValueIllegalExp.test(m)) {
                            ngModel.$setValidity('keyValueSeparatorValidator', false);
                            return false;
                        }
                    }
                }

                // Check if operators(+-*/) missed between key codes
                // replacedVal is like: 123+@+@-@
                var replacedVal = ngModelValue.replace(keyValueExp, '@').replace(blankExp, '');
                if (missingOperatorExp.test(replacedVal)) {
                    ngModel.$setValidity('missingOperatorValidator', false);
                    return false;
                }

                // Find the operators and make sure they are not duplicated
                var operatorMatches = ngModelValue.match(notDuplicateOperatorExp);
                if (_.any(operatorMatches, function (m) {
                        return m.length > 1;
                    })) {
                    ngModel.$setValidity('operatorDuplicateValidator', false);
                    return false;
                }

                // Check if the expression ends with operators(+-*/,)
                if (missingExpressionExp.test(ngModelValue)) {
                    ngModel.$setValidity('missingExpressionValidator', false);
                    return false;
                }

                return true;
            };

            var asyncFormulaValidate = function () {
                if (!$scope.isDemo && $scope.mentionApi.isValid()) {
                    var validateFormula = $scope.editModel.formula;
                    if (!_.isEmpty(validateFormula)) {
                        validateFormula = validateFormula.replace(keyValueExp, '0').trim();
                    }

                    templateFormulaService.validate(validateFormula).success(function (validation) {
                        if (!validation.result) {
                            $scope.editReportFormulaForm.formula.$setValidity('formulaFormatValidator', false);
                        } else {
                            $scope.editReportFormulaForm.formula.$setValidity('formulaFormatValidator', true);
                        }
                    });
                }
            };

            // 设置Key Value选择器
            $scope.keyValueTitle = $translate.instant('AddKeyValue');
            $scope.keyValueSelectorOptions = {
                top: 120
            };

            $scope.keyValueSelectorApi = {
                onSelectorConfirm: function () {
                    var selection = this.getSelection();
                    if (!_.isEmpty(selection)) {
                        if ($scope.isUpdate) {
                            var node = $scope.focusTag;
                            $(node).attr('data-code', '@' + selection.keyCode);
                            $(node).attr('title', selection.description);
                            $(node).text('@' + selection.name);
                        } else {
                            $scope.editModel.formula = $.trim($scope.editModel.formula) + ' @' + selection.keyCode + ' ';
                        }
                    }
                }
            };

            // 设置公式参数选择器
            $scope.paramSelectorApi = {
                onSelectorConfirm: function (e) {
                    if ($scope.isUpdate) {
                        var childNodes = $scope.focusTag.childNodes;
                        if (childNodes.length > 2) {
                            for (var i = childNodes.length; i > 1; i--) {
                                childNodes[i].remove();
                            }
                        }

                        childNodes[1].nodeValue = e.param;
                    } else {
                        if (!_.isEmpty(e.formula)) {
                            $scope.editModel.formula = $.trim($scope.editModel.formula) + e.formula + '(' + e.param;
                        }
                    }
                }
            };

            /*********************** End Formula Editor ***********************************/

            /*********************** Formula Editor for Validation ************************/
            var showSelectFormulaPopForValidation = function () {
                $scope.operateTypeForValidation = constant.Operation.Add;
            };

            var showSelectKeyValuePopForValidation = function (keyCode) {
                if (!_.isEmpty(keyCode)) {
                    $scope.defaultKeyCodeForValidation = keyCode;
                    $scope.isUpdateForValidation = true;
                }
                else {
                    $scope.defaultKeyCodeForValidation = null;
                    $scope.isUpdateForValidation = false;
                }

                $scope.isShowKeyValueForValidation = true;
            };

            var editFormulaParamOrKeyValueForValidation = function (e) {
                if ($scope.hasEditPermission && $scope.ToggleSaveAs && $scope.editModel.hasValidation) {
                    $scope.isUpdateForValidation = true;
                    if (e && e.tagInfo) {
                        if (e.tagType === '0') { // key value selector
                            $scope.focusTagForValidation = e.tagInfo;
                            showSelectKeyValuePopForValidation(e.name);
                        }
                        else if (e.tagType === '1') { // formula parameter editor
                            $scope.focusTagForValidation = e.tagInfo;
                            $scope.editFormulaForValidation = e.name;
                            $scope.editParamsForValidation = e.formulaParams;
                            $scope.isShowParamForValidation = true;
                        }
                    }
                }
            };

            var selectParamForValidation = function (formula) {
                $scope.isUpdateForValidation = false;
                $scope.editFormulaForValidation = formula.code;
                $scope.editParamsForValidation = ')';
                $scope.isShowParamForValidation = true;
            };

            var cancelHideTranslatorForValidation = function () {
                $scope.cancelHideForValidation = true;
            };

            var timerHideTranslatorForValidation = function () {
                $scope.cancelHideForValidation = false;
                $timeout(function () {
                    if (!$scope.cancelHideForValidation && $scope.firedValidationType === '1') {
                        $scope.firedValidationName = '';
                        $scope.firedValidationParams = '';
                    }
                }, 500);
            };

            // 公式验证
            var mentionMenuValidatorForValidation = function (ngModelValue, ngModel) {
                ngModel.$setValidity('requiredValidator', true);
                ngModel.$setValidity('keyValueNotEmptyValidator', true);
                ngModel.$setValidity('keyValueSeparatorValidator', true);
                ngModel.$setValidity('operatorDuplicateValidator', true);
                ngModel.$setValidity('missingExpressionValidator', true);
                ngModel.$setValidity('missingOperatorValidator', true);

                if (!$scope.hasEditPermission || !$scope.ToggleSaveAs || !$scope.editModel.hasValidation) {
                    return true;
                }

                if (PWC.isNullOrEmpty(ngModelValue)) {
                    ngModel.$setValidity('requiredValidator', false);
                    return false;
                }

                var keyValueMatches = ngModelValue.match(keyValueExp);
                if (!_.isEmpty(keyValueMatches)) {
                    for (var i = 0; i < keyValueMatches.length; i++) {
                        var m = keyValueMatches[i];
                        m = m.substring(1); // Remove prefix @

                        if (m.length === 0) { // The key value is empty
                            ngModel.$setValidity('keyValueNotEmptyValidator', false);
                            return false;
                        }

                        // Check if suffix blank exists for each key value
                        if (keyValueIllegalExp.test(m)) {
                            ngModel.$setValidity('keyValueSeparatorValidator', false);
                            return false;
                        }
                    }
                }

                // Check if operators(+-*/) missed between key codes
                // replacedVal is like: 123+@+@-@
                var replacedVal = ngModelValue.replace(keyValueExp, '@').replace(blankExp, '');
                if (missingOperatorExp.test(replacedVal)) {
                    ngModel.$setValidity('missingOperatorValidator', false);
                    return false;
                }

                // Find the operators and make sure they are not duplicated
                var operatorMatches = ngModelValue.match(notDuplicateOperatorExp);
                if (_.any(operatorMatches, function (m) {
                        return m.length > 1;
                    })) {
                    ngModel.$setValidity('operatorDuplicateValidator', false);
                    return false;
                }

                // Check if the expression ends with operators(+-*/,)
                if (missingExpressionExp.test(ngModelValue)) {
                    ngModel.$setValidity('missingExpressionValidator', false);
                    return false;
                }

                return true;
            };

            var asyncFormulaValidateForValidation = function () {
                if ($scope.mentionApiForValidation.isValid()) {
                    var validateFormula = $scope.editModel.validation;
                    if (!_.isEmpty(validateFormula)) {
                        validateFormula = validateFormula.replace(keyValueExp, '0').trim();
                    }

                    templateFormulaService.validate(validateFormula).success(function (validation) {
                        if (!validation.result) {
                            $scope.editReportFormulaForm.validation.$setValidity('formulaFormatValidator', false);
                        } else {
                            $scope.editReportFormulaForm.validation.$setValidity('formulaFormatValidator', true);
                        }
                    });
                }
            };

            // 设置Key Value选择器
            $scope.keyValueTitleForValidation = $translate.instant('AddKeyValue');
            $scope.keyValueSelectorOptionsForValidation = {
                top: 120
            };

            $scope.keyValueSelectorApiForValidation = {
                onSelectorConfirm: function () {
                    var selection = this.getSelection();
                    if (!_.isEmpty(selection)) {
                        if ($scope.isUpdateForValidation) {
                            var node = $scope.focusTagForValidation;
                            $(node).attr('data-code', '@' + selection.keyCode);
                            $(node).attr('title', selection.description);
                            $(node).text('@' + selection.name);
                        } else {
                            $scope.editModel.validation = $.trim($scope.editModel.validation) + ' @' + selection.keyCode + ' ';
                        }
                    }
                }
            };

            // 设置公式参数选择器
            $scope.paramSelectorApiForValidation = {
                onSelectorConfirm: function (e) {
                    if ($scope.isUpdateForValidation) {
                        var childNodes = $scope.focusTagForValidation.childNodes;
                        if (childNodes.length > 2) {
                            for (var i = childNodes.length; i > 1; i--) {
                                childNodes[i].remove();
                            }
                        }

                        childNodes[1].nodeValue = e.param;
                    } else {
                        if (!_.isEmpty(e.formula)) {
                            $scope.editModel.validation = $.trim($scope.editModel.validation) + e.formula + '(' + e.param;
                        }
                    }
                },
            };

            /*********************** End Formula Editor ***********************************/
            //模板另存为
            var saveTemplateGroup = function () {
                    var groupName = $scope.groupName;
                    if (groupName === '' || groupName === undefined) {
                        SweetAlert.warning($translate.instant('InputTemplateGroupNameWarning'));
                        return;
                    }

                    $scope.newTemplateGroup.Name = groupName;
                    $scope.newTemplateGroup.ServiceTypeID = $scope.curServiceTypeId;
                    $scope.newTemplateGroup.CopyFrom = $scope.curTemplateGroup.id;
                    $scope.newTemplateGroup.groupType = $scope.curTemplateGroup.groupType;
                    $scope.newTemplateGroup.payTaxType = $scope.curTemplateGroup.payTaxType;
                    $scope.newTemplateGroup.industryIDs = $scope.curTemplateGroup.industryIDs;
                    $scope.newTemplateGroup.ChangedFormulas = $scope.formulasArrayTemp;

                    templateGroupService.addTemplateGroup($scope.newTemplateGroup).success(function (or) {
                        if (or && or.result) {
                            SweetAlert.success($translate.instant('TemplateGroupSaveAsSuccess'));
                            loadTemplateGroup();
                            $('#templateName').val('');
                            $('.templates-save-form').css('display', 'none');
                        } else {
                            SweetAlert.warning($translate.instant(or.resultMsg));
                        }
                    });
                };


            var loadTemplateMenu = function () {
                templateGroupService.getTemplateList($scope.curTemplateGroup).success(function (templateData) {
                    $scope.templates = templateData;
                });
            };
            //保存模板
            var saveTemplate = function () {
                templateFormulaService.saveTemplateFormulaList($scope.formulasArrayTemp).success(function (or) {
                    if (or && or.result) {
                        SweetAlert.success($translate.instant('TemplateSaveSuccess'));
                    } else {
                        SweetAlert.warning($translate.instant(or.resultMsg));
                    }
                });
            };


            var saveFormulaToTemp = function () {
                //find exist
                /*
                var a = _.findWhere($scope.formulasArrayTemp, { id: $scope.editModel.cellTemplateID });

                if (a && a != undefined) {
                    //delete old
                    $scope.formulasArrayTemp.forEach(function (f, index) {
                        if (f.cellTemplateID === $scope.editModel.cellTemplateID) {
                            $scope.formulasArrayTemp.splice(index, 1);
                        }
                    });
                    //add new
                    $scope.formulasArrayTemp.push($scope.editModel);
                }
                else {
                    $scope.formulasArrayTemp.push($scope.editModel);
                }
                */

                if (!$scope.editModel.hasFormula) {
                    $scope.editModel.formula = null;
                }

                if ($scope.editModel.hasInvoice) {
                    // 如果发票类型全选，则将editModel.invoiceCategory设置为[ 99 ]；
                    // 否则将选中的发票类型存入editModel.invoiceCategory数组，如[1, 2, 3]
                    if ($scope.editModel.chosenInvoiceOption.categoryOptions.length > 0 &&
                        _.every($scope.editModel.chosenInvoiceOption.categoryOptions, {ticked: true})) {
                        $scope.editModel.invoiceCategory = [Number(constant.selectAllValue)];
                    } else {
                        $scope.editModel.invoiceCategory = _.chain($scope.editModel.chosenInvoiceOption.categoryOptions)
                            .where({ticked: true}).pluck('value').value();
                    }

                    // 如果税率全选，则将editModel.taxRate设置为[ '99' ]；
                    // 否则将选中的税率存入editModel.taxRate数组，如['17%', '11%', '6%']
                    if ($scope.editModel.chosenInvoiceOption.rateOptions.length > 0 &&
                        _.every($scope.editModel.chosenInvoiceOption.rateOptions, {ticked: true})) {
                        $scope.editModel.taxRate = [constant.selectAllValue];
                    } else {
                        $scope.editModel.taxRate = _.chain($scope.editModel.chosenInvoiceOption.rateOptions)
                            .where({ticked: true}).pluck('value').value();
                    }

                    $scope.editModel.invoiceType = $scope.editModel.chosenInvoiceOption.value;
                    $scope.editModel.invoiceAmountType = $scope.editModel.chosenInvoiceOption.invoiceAmountType.value;

                } else {
                    $scope.editModel.invoiceCategory = [];
                    $scope.editModel.taxRate = [];
                    $scope.editModel.invoiceType = null;
                    $scope.editModel.invoiceAmountType = null;
                }

                if (!$scope.editModel.hasVoucher) {
                    $scope.editModel.accountCodes = [];
                    $scope.editModel.voucherKeyword = '';
                }

                if (!$scope.editModel.hasModel) {
                    $scope.editModel.modelIDs = [];
                }

                if (!$scope.editModel.hasValidation) {
                    $scope.editModel.validation = null;
                    $scope.editModel.validationDescription = null;
                }

                templateFormulaService.saveOrUpdateCellTemplateConfig($scope.editModel).success(function (data) {
                    if (data && data.result) {
                        SweetAlert.success($translate.instant('CellTemplateSaveSuccess'));

                        for (var i = 0; i < $scope.templateFormulaList.length; i++) {
                            if ($scope.templateFormulaList[i].cellTemplateID === $scope.editModel.cellTemplateID) {
                                $scope.editModel.formulaDataSource = data.resultMsg;
                                $scope.templateFormulaList[i] = $scope.editModel;
                            }
                        }
                    } else {
                        SweetAlert.warning($translate.instant(data.resultMsg));
                    }

                    //$('#editReportFormulaPop').modal('hide');
                    //$('#editReportFormulaPop').hide('normal');
                    hidePanel();
                    refreshSpreadUI();
                });
            };

            $scope.showOperateLogPop = function () {
                $scope.isShowLog = true;
                // $('#showOperatePop').modal('show');
            };

            var refreshSpreadUI = function () {

                spreadJsTipService.initialize($scope.activeSheet);
                if ($scope.templateFormulaList && $scope.templateFormulaList.length > 0) {
                    $scope.templateFormulaList.forEach(function (row, index) {
                        var cell = $scope.activeSheet.getCell(row.rowIndex, row.columnIndex);
                        if (row.formulaDataSource) {
                            cell.value(row.formulaDataSource);
                            cell.hAlign(0);
                        } else {
                            if (cell.value() !== '——') {
                                cell.value(null);
                            } else {
                                //is ——
                                if (row.hasFormula || row.hasInvoice || row.hasVoucher || row.hasKeyIn) {
                                    cell.value(null);
                                }
                            }
                        }
                        spreadJsTipService.setCellTip(cell, row);

                        if ($scope.cellTemplateId && $scope.cellTemplateId === row.cellTemplateID) {
                            $scope.activeSheet.setActiveCell(row.rowIndex, row.columnIndex);
                            releaseParameter();
                        }


                        //spreadJsTipService.setCellTipString(cell, "测试数据源");
                    });
                }

                //加载修改了但还未保存下来的数据到excel中            
                if ($scope.formulasArrayTemp && $scope.formulasArrayTemp.length > 0) {
                    $scope.formulasArrayTemp.forEach(function (row) {
                        var cell = $scope.activeSheet.getCell(row.rowIndex, row.columnIndex);
                        if (row.formulaDataSource) {
                            cell.value(row.formulaDataSource);
                            cell.hAlign(0);
                        } else {
                            cell.value(null);
                        }
                        spreadJsTipService.setCellTip(cell, row);
                    });
                }

                spreadJsTipService.paintSheet($scope.activeSheet);
            };

            var getFormulaList = function () {
                $q.all([
                    formulaService.getAll(),
                    formulaService.getAllParam(),
                    formulaService.getAllParamMapping()
                ]).then(function (result) {
                    if (!_.isEmpty(result[0]) && !_.isEmpty(result[0].data) && !_.isEmpty(result[0].data.data)) {
                        var formulaList = _.map(result[0].data.data, function (config) {
                            return {
                                id: config.id,
                                code: config.formulaName,
                                name: config.chineseName,
                                description: config.description,
                                englishName: config.englishName,
                                requiredParamNum: config.requiredParamNum
                            }
                        });

                        if (!_.isEmpty(result[1]) && !_.isEmpty(result[1].data) && !_.isEmpty(result[1].data.data) &&
                            !_.isEmpty(result[2]) && !_.isEmpty(result[2].data) && !_.isEmpty(result[2].data.data)) {
                            var groupedParamList = _.groupBy(result[2].data.data, 'formulaID');
                            var paramList = result[1].data.data;
                            _.each(groupedParamList, function (val, key) {
                                var formula = _.findWhere(formulaList, {id: key});
                                if (formula) {
                                    formula.params = _.chain(val)
                                        .map(function (p) {
                                            return {id: p.formulaParamID, index: p.paramIndex};
                                        })
                                        .sortBy(function (p) {
                                            return p.index;
                                        }).value();
                                    _.each(formula.params, function (p, idx) {
                                        var param = _.findWhere(paramList, {id: p.id});
                                        _.extend(p, param);
                                    });
                                }
                            });
                        }

                        $scope.formulaList = formulaList;
                    }
                });
            };

            var getModelList = function () {
                modelConfigurationService.getModelListByIndustry($scope.curServiceTypeId, $scope.curIndustry.id)
                    .success(function (data) {
                        var rtn = [];
                        var modelTypes = _.invert(enums.modelType);
                        _.each(data, function (m) {
                            if (m.type.toString() === modelTypes.SystemCommonModel) {
                                if (!_.some(rtn, {id: m.id})) {
                                    m.orgID = null;
                                    m.organizationName = $translate.instant('All');
                                    m.featureName = $translate.instant(enums.modelFeature[m.feature]);
                                    rtn.push(m);
                                }
                            } else {
                                var existsModel = _.findWhere(rtn, {id: m.id});
                                if (!existsModel) {
                                    m.featureName = $translate.instant(enums.modelFeature[m.feature]);
                                    rtn.push(m);
                                } else {
                                    existsModel.orgID = existsModel.orgID + ',' + m.orgID;
                                    existsModel.organizationName = existsModel.organizationName + ',' + m.organizationName;
                                }
                            }
                        });

                        $scope.modelDataSource = rtn;
                    });
            };

            var checkVoucherValid = function () {
                if (!$scope.editModel.hasVoucher || !PWC.isNullOrEmpty($scope.editModel.voucherKeyword)
                    || !_.isEmpty($scope.editModel.accountCodes)) {
                    if ($scope.editReportFormulaForm && $scope.editReportFormulaForm.voucher
                        && $scope.editReportFormulaForm.voucher.$error.voucherNotSelectValidator) {
                        $scope.editReportFormulaForm.voucher.$setValidity('voucherNotSelectValidator', true);
                    }
                } else if ($scope.editReportFormulaForm && $scope.editReportFormulaForm.voucher
                    && !$scope.editReportFormulaForm.voucher.$error.voucherNotSelectValidator) {
                    $scope.editReportFormulaForm.voucher.$setValidity('voucherNotSelectValidator', false);
                }
            };

            var checkPermission = function () {
                $scope.$root.checkUserPermission(constant.adminPermission.systemConfiguration.declarationFormConfiguration.editCode).success(function (data) {
                    $scope.hasEditPermission = data;
                });
            };

            $scope.$watch('curIndustry.id', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.accountDataSource = [];
                    $scope.modelDataSource = [];
                } else {
                    stdAccountService.getStdAccountByIndustryID(newValue)
                        .success(function (data) {
                            _.each(data, function (account) {
                                account.directionName = $translate.instant(enums.standardAccountDirection[account.direction]);
                                account.AcctPropName = $translate.instant(enums.standardAccountAcctProp[account.acctProp]);
                            });

                            $scope.accountDataSource = data;
                        });
                    getModelList();
                    getFormulaList();
                }
            });

            $scope.$watch('curServiceTypeId', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.modelDataSource = [];
                } else {
                    getModelList();
                }
            });

            // If changed to no formula config, empty the input.
            $scope.$watch('editModel.hasFormula', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.editModel.formula = '';
                    if ($scope.editReportFormulaForm && $scope.editReportFormulaForm.formula
                        && $scope.editReportFormulaForm.formula.$error.requiredValidator) {
                        $scope.editReportFormulaForm.formula.$setValidity('requiredValidator', true);
                    }
                } else if (PWC.isNullOrEmpty($scope.editModel.formula) &&
                    !$scope.editReportFormulaForm.formula.$error.requiredValidator) {
                    $scope.editReportFormulaForm.formula.$setValidity('requiredValidator', false);
                }
            });

            $scope.$watch('editModel.hasValidation', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.editModel.validation = '';
                    if ($scope.editReportFormulaForm && $scope.editReportFormulaForm.validation
                        && $scope.editReportFormulaForm.validation.$error.requiredValidator) {
                        $scope.editReportFormulaForm.validation.$setValidity('requiredValidator', true);
                    }
                } else if (PWC.isNullOrEmpty($scope.editModel.validation)
                    && !$scope.editReportFormulaForm.validation.$error.requiredValidator) {
                    $scope.editReportFormulaForm.validation.$setValidity('requiredValidator', false);
                }
            });

            // If changed to no voucher config, empty the datasource.
            $scope.$watch('editModel.hasVoucher', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.editModel.voucherKeyword = '';
                    $scope.selectedAccount = [];
                    $scope.editModel.accountCodes = [];
                }

                checkVoucherValid();
            });

            // If invoice option is not selected yet, alert user.
            $scope.$watch('editModel.chosenInvoiceOption', function (newValue, oldValue) {
                if ($scope.editModel.hasInvoice &&
                    (_.isEmpty(newValue) ||
                        newValue.value <= 0 ||
                        newValue.invoiceAmountType.value <= 0 ||
                        newValue.value !== 3 &&
                        (_.every(newValue.categoryOptions, {ticked: false}) ||
                            _.every(newValue.rateOptions, {ticked: false})))) {
                    $scope.editReportFormulaForm.invoice.$setValidity('invoiceNotSelectValidator', false);
                } else {
                    $scope.editReportFormulaForm.invoice.$setValidity('invoiceNotSelectValidator', true);
                }
            }, true);

            $scope.$watch('editModel.hasInvoice', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.editReportFormulaForm.invoice.$setValidity('invoiceNotSelectValidator', true);
                } else if (_.isEmpty($scope.editModel.chosenInvoiceOption) ||
                    $scope.editModel.chosenInvoiceOption.value <= 0 ||
                    $scope.editModel.chosenInvoiceOption.invoiceAmountType.value <= 0 ||
                    $scope.editModel.chosenInvoiceOption.value !== 3 &&
                    (_.every($scope.editModel.chosenInvoiceOption.rateOptions, {ticked: false}) ||
                        _.every($scope.editModel.chosenInvoiceOption.categoryOptions, {ticked: false}))) {
                    $scope.editReportFormulaForm.invoice.$setValidity('invoiceNotSelectValidator', false);
                }
            });

            // If changed to no model config, empty the datasource.
            $scope.$watch('editModel.hasModel', function (newValue, oldValue) {
                if (!newValue) {
                    $scope.selectedModel = [];
                    $scope.editModel.modelIDs = [];
                    $scope.editReportFormulaForm.model.$setValidity('modelNotSelectValidator', true);
                } else if ($scope.editModel.modelIDs.length === 0) {
                    $scope.editReportFormulaForm.model.$setValidity('modelNotSelectValidator', false);
                }
            });

            $scope.$watch('voucherFilterType', function (newValue, oldValue) {
                $scope.editModel.voucherKeyword = '';
                $scope.selectedAccount = [];
                $scope.editModel.accountCodes = [];

                if (newValue === 0) {
                    $scope.refreshGrids();
                }

                checkVoucherValid();
            });

            $scope.$watch('editModel.voucherKeyword', function (newValue, oldValue) {
                checkVoucherValid();
            });

            $scope.$watch('selectedAccount.length', function (newValue, oldValue) {
                if (newValue !== oldValue) {
                    $scope.editModel.accountCodes = _.pluck($scope.selectedAccount, 'code');
                }

                checkVoucherValid();
            });

            $scope.$watch('selectedModel.length', function (newValue, oldValue) {
                if (newValue !== 0 || !$scope.editModel.hasModel) {
                    $scope.editReportFormulaForm.model.$setValidity('modelNotSelectValidator', true);
                } else {
                    $scope.editReportFormulaForm.model.$setValidity('modelNotSelectValidator', false);
                }

                if (newValue !== oldValue) {
                    $scope.editModel.modelIDs = _.pluck($scope.selectedModel, 'id');
                }
            });

            var releaseParameter = function () {
                $scope.templateGroupId = null;
                $scope.templateId = null;
                $scope.cellTemplateId = null;
            };

            // 选中报表
            var selectTemplate = function (model) {
                $scope.currentTemplate = model;

                listTemplates();
            };

            $scope.selectTemplate = selectTemplate;

            // 改变模板的的时候，重新选择
            var selectTemplateGroup = function (model) {
                $scope.curTemplateGroup = null;
                $scope.currentTemplate = null;
                if (model) {
                    $scope.curTemplateGroup = model;
                }

                loadTemplateGroup();
            };

            $scope.selectTemplateGroup = selectTemplateGroup;

            // dx 控件
            var dxControl = {
                groupDropDownId: 'dx-select-template-groups',
                groupDropDownContentId: 'dx-template-group-content',
                templateDropDownId: 'dx-select-template',
                templateDropDownContentId: 'dx-template-content',

                // dx drop down 模板绘制
                renderTemplateGroupDropDown: function (data, selectValue) {
                    $scope.curTemplateGroup = selectValue;

                    if (!selectValue) {
                        selectValue = {};
                        selectValue.name = '';
                    }


                    var bindAddEvent = function () {
                        $(".add-template-group").off("click").on("click", function () {
                            // to create order

                            $scope.$apply(function () {
                                $scope.templateGroupModel = {};
                                $scope.templateGroupModel.serviceTypeID = $scope.curServiceTypeId;
                                $scope.templateGroupModel.industryIDs = $scope.curIndustry.id;

                                if ($scope.curServiceTypeId === enums.serviceType.CIT || $scope.curServiceTypeId === enums.serviceType.Others) {
                                    $scope.templateGroupModel.reportType = $scope.reportType.value;
                                    $scope.operateTemplateGroupType = constant.TemplateGroupOperation.AddGroupFirst;
                                }
                                else {
                                    $scope.templateGroupModel.payTaxType = $scope.payTaxType.value;

                                    // Mock data
                                    $scope.curServiceTypeId = enums.serviceType.VAT;

                                    $scope.operateTemplateGroupType = constant.TemplateGroupOperation.AddDirectly;
                                }
                            });


                            $('#' + dxControl.groupDropDownId).dxDropDownBox('close');
                        });

                    };

                    // 鼠标移开，关闭drop down
                    var bindMouseLeave = function () {

                        $("#" + dxControl.groupDropDownContentId).off("mouseleave").on("mouseleave", function () {
                            // to create order
                            $('#' + dxControl.groupDropDownId).dxDropDownBox('close');
                        });
                    };

                    // 报表模板组下拉列表
                    var listTemplateGroupView = function (e, data, value) {

                        var height = null;

                        if (data && data.length >= 8) {
                            height = '300px';
                        }

                        var list = $('<div class="list-view" id="templateGroupListView">').dxList({
                            dataSource: data,
                            height: height,
                            selectionMode: "single",
                            pageLoadMode: 'scrollBottom',
                            itemTemplate: function (itemData, itemIndex, itemElement) {
                                var div = '<span class="left-part" title="' + itemData.name + '"><span>' + itemData.name + '</span></span>';
                                if (!itemData.isSystemType) {
                                    div += '<span class="right-operation"><i class="material-icons edit">create</i>';
                                    div += ' <i class="material-icons delete">delete</i>';
                                }
                                div += '</span>';

                                itemElement.append(div);
                            },
                            onItemClick: function (args) {

                                $scope.curTemplateGroup = args.itemData;
                                // 单击即使选中
                                if (args.jQueryEvent.target.className === thisConstant.deleteClass) {
                                    $scope.$apply(function () {
                                        $scope.templateGroupModel = args.itemData;
                                        $scope.operateTemplateGroupType = constant.Operation.Delete;
                                    });

                                } else if (args.jQueryEvent.target.className === thisConstant.editClass) {

                                    $scope.$apply(function () {
                                        $scope.templateGroupModel = args.itemData;
                                        $scope.operateTemplateGroupType = constant.Operation.Edit;
                                    });
                                } else {
                                    loadReportType();
                                }

                                $('#' + dxControl.groupDropDownId).dxDropDownBox('close');
                                e.component.option('value', $scope.curTemplateGroup.name);
                            }

                        });

                        list.dxList('instance');
                        return list;
                    };

                    $('#' + dxControl.groupDropDownId).dxDropDownBox({
                        value: selectValue.name,
                        valueExpr: "id",
                        displayExpr: "name",
                        showClearButton: false,
                        onContentReady: function (args) {
                            // var value = args.component.option("value");
                            // syncTreeViewSelection(thisData.orgTreeView, thisData.selectOrgList, false);
                        },
                        contentTemplate: function (e) {
                            // thisData.DropDownEvent = e;
                            var value = e.component.option("value");
                            var $treeView = listTemplateGroupView(e, data, value);

                            var addTemplateGroup = '<div class="add-template-group"><span class="left-part">\
                                <span class="organge-color">\
                                    <i class="material-icons add">add_circle_outline</i>' + $translate.instant('AddTemplate') + '</span>\
                            </span></div>';

                            var div = "<div id='" + dxControl.groupDropDownContentId + "'>";
                            var wrapper = $(div).append($treeView).append(addTemplateGroup);

                            $timeout(function () {
                                bindAddEvent();
                                bindMouseLeave();
                            }, 50);
                            return wrapper;
                        }
                    });
                },

                // 报表下拉框
                renderTemplateDropDown: function (data, selectValue) {
                    if (!data) {
                        data = [];
                    }


                    if (selectValue) {
                        loadSheet(selectValue);
                    } else {
                        $('#report-view').html('');
                        selectValue = {};
                        selectValue.name = '';
                    }

                    var bindAddEvent = function () {
                        $(".add-template-btn").off("click").on("click", function () {
                            // to create order

                            var currentTemplateCode = _.pluck($scope.templates, 'code');

                            $scope.$apply(function () {

                                var serviceTypeId = '2';
                                if ($scope.curServiceTypeId === enums.serviceType.CIT || $scope.curServiceTypeId === enums.serviceType.VAT
                                    || $scope.curServiceTypeId === enums.serviceType.Others) {
                                    serviceTypeId = $scope.curServiceTypeId;
                                }

                                $scope.templateModel = {
                                    templateGroupID: $scope.curTemplateGroup.id,
                                    serviceTypeID: serviceTypeId,
                                    payTaxType: serviceTypeId === enums.serviceType.CIT || serviceTypeId === enums.serviceType.Others ? '' : $scope.payTaxType.value,
                                    reportType: serviceTypeId === enums.serviceType.CIT || serviceTypeId === enums.serviceType.Others ? $scope.reportType.value : '',
                                    industryIDs: $scope.curIndustry.id,
                                    currentTemplateCodeList: currentTemplateCode
                                };

                                $scope.operateTemplateType = constant.Operation.Add;
                            });

                            $('#' + dxControl.templateDropDownId).dxDropDownBox('close');
                        });
                    };

                    // 鼠标移开，关闭drop down
                    var bindMouseLeave = function () {

                        $("#" + dxControl.templateDropDownContentId).off("mouseleave").on("mouseleave", function () {
                            // to create order
                            $('#' + dxControl.templateDropDownId).dxDropDownBox('close');
                        });
                    };

                    var getDisplayName = function (value) {

                        if (!value) {
                            return '';
                        }
                        var displayName = value.name;
                        if (value.code) {
                            displayName = value.code + '_' + value.name;
                        }

                        return displayName;
                    };

                    // 报表模板下拉列表
                    var listTemplateView = function (e, data, value) {

                        var height = null;

                        if (data && data.length >= 8) {
                            height = '300px';
                        }

                        var list = $('<div class="list-view" id="templateListView">').dxList({
                            dataSource: data,
                            height: height,
                            selectionMode: "single",
                            pageLoadMode: 'scrollBottom',
                            noDataText: '无数据',
                            itemTemplate: function (itemData, itemIndex, itemElement) {

                                var disPlay = getDisplayName(itemData);

                                var div = '<span class="left-part" title="' + disPlay + '"><span>' + disPlay + '</span></span><span class="right-operation">';

                                if (!itemData.isSystemType) {
                                    div += '<i class="material-icons edit">create</i>';
                                }

                                div += ' <i class="material-icons delete">delete</i>';

                                div += '</span>';

                                itemElement.append(div);
                            },
                            onItemClick: function (args) {

                                $scope.currentTemplate = args.itemData;
                                // 单击即使选中
                                if (args.jQueryEvent.target.className === thisConstant.deleteClass) {

                                    $scope.$apply(function () {
                                        $scope.operateTemplateType = constant.Operation.Delete;
                                        $scope.templateModel = args.itemData;
                                    });

                                } else if (args.jQueryEvent.target.className === thisConstant.editClass) {

                                    $scope.$apply(function () {
                                        $scope.operateTemplateType = constant.Operation.Edit;
                                        $scope.templateModel = args.itemData;
                                    });

                                } else {
                                    loadSheet(args.itemData);
                                }

                                var disPlay = getDisplayName(args.itemData);

                                $('#' + dxControl.templateDropDownId).dxDropDownBox('close');
                                e.component.option('value', disPlay);
                            }

                        });

                        list.dxList('instance');
                        return list;
                    };

                    var selectName = getDisplayName(selectValue);

                    $('#' + dxControl.templateDropDownId).dxDropDownBox({
                        value: selectName,
                        showClearButton: false,
                        onContentReady: function (args) {
                        },
                        contentTemplate: function (e) {
                            // thisData.DropDownEvent = e;
                            var value = e.component.option("value");
                            var $treeView = listTemplateView(e, data, value);

                            var addTemplateGroup = '<div class="add-template-btn"><span class="left-part">\
                                <span class="organge-color">\
                                    <i class="material-icons add">add_circle_outline</i>' + $translate.instant('AddReport') + '</span>\
                            </span></div>';


                            var div = "<div id='" + dxControl.templateDropDownContentId + "'>";

                            var wrapper = $(div).append($treeView).append(addTemplateGroup);

                            $timeout(function () {
                                bindAddEvent();
                                bindMouseLeave();
                            }, 50);
                            return wrapper;
                        }
                    });
                }

            };


            (function initialize() {
                $log.debug('DeclarationFormConfigurationController.ctor()...');

                $log.debug('...cellTemplateId:' + $scope.templateGroupId);

                resetFormHeight();
                $scope.selectedTab = 0;
                $scope.voucherFilterType = 0;

                $scope.selectFormula = null;
                $scope.cancelHide = false;
                $scope.firedName = '';
                $scope.firedType = '0';
                $scope.firedParams = '';
                $scope.editFormula = '';
                $scope.editParams = '';
                $scope.mentionApi = {};

                $scope.selectFormulaForValidation = null;
                $scope.cancelHideForValidation = false;
                $scope.firedValidationName = '';
                $scope.firedValidationType = '0';
                $scope.firedValidationParams = '';
                $scope.editFormulaForValidation = '';
                $scope.editParamsForValidation = '';
                $scope.mentionApiForValidation = {};

                $scope.curTemplateGroup = {};
                $scope.newTemplateGroup = {};
                $scope.templateGroups = [];
                $scope.templates = [];
                $scope.currentTemplate = {};
                $scope.editModel = {};
                $scope.formulasArrayTemp = [];
                $scope.curIndustry = {};
                $scope.payTaxType = filingTypeList[0];
                $scope.formulaList = [];

                $scope.dataSourcePayTaxTypeListInstance = {};

                $scope.dataSourceIndustryListInstance = {};
                //配置报表类型下拉框
                $scope.dataSourcePayTaxTypeList = {
                    dataSource: new DevExpress.data.ArrayStore({
                        data: filingTypeList
                    }),
                    displayExpr: 'name',
                    valueExpr: "name",
                    value: filingTypeList[0].name,
                    itemTemplate: 'filingTypeItem',
                    onInitialized: function (e) {
                        $scope.dataSourcePayTaxTypeListInstance = e.component;
                    },
                    onSelectionChanged: function (e) {
                        $scope.payTaxType = e.selectedItem;
                        loadTemplateGroup();
                    }
                };

                checkPermission();
                loadServiceType();

                KeyValueConfigService.getAll().success(function (data) {
                    $scope.mentionList = _.map(data, function (config) {
                        return {
                            code: '@' + config.keyCode,
                            name: config.name,
                            description: config.description
                        }
                    });
                });

                var tabs = [
                    {id: 0, text: $translate.instant('AutoCalculateTab')},
                    {id: 1, text: $translate.instant('ManualInputTab')},
                    {id: 2, text: $translate.instant('ModelTab')},
                    {id: 3, text: $translate.instant('ValidationTab')}
                ];
                $scope.tabOptions = {
                    dataSource: tabs,
                    bindingOptions: {
                        selectedIndex: 'selectedTab'
                    },
                    onItemClick: function () {
                        $scope.refreshGrids();
                    }
                };

                initAccountSelector();
                initModelSelector();

                $scope.invoiceOptions = getInvoiceFilter();
                //配置发票类型下拉选项
                $scope.dataSourceInvoice = {
                    bindingOptions: {
                        disabled: '!hasEditPermission || !ToggleSaveAs || !editModel.hasInvoice'
                    },
                    dataSource: $scope.invoiceOptions,
                    displayExpr: function (item) {
                        if (item) {
                            return $translate.instant(item.type);
                        }
                        return "";
                    },
                    onItemClick: function (e) {
                        $scope.invoiceOption = e.itemData;
                        selectInvoiceOption($scope.editModel, $scope.invoiceOption.value);
                    }
                };

                //配置发票产生类型下拉选项
                $scope.dataSourceInvoiceAmount = {
                    bindingOptions: {
                        dataSource: 'invoiceOption.amountTypeOptions',
                        disabled: '!hasEditPermission || !ToggleSaveAs || !editModel.hasInvoice'
                    },
                    displayExpr: function (item) {
                        if (item) {
                            return $translate.instant(item.name);
                        }
                        return "";
                    },
                    onItemClick: function (e) {
                        $scope.editModel.chosenInvoiceOption.invoiceAmountType = e.itemData;
                    }
                };

                // 设置科目代码选择器
                $scope.accountTitle = $translate.instant('AddVoucher');
                $scope.accountSelectorOptions = {
                    height: 500,
                    top: 140,
                    width: 730,
                    selectNodesRecursive: true,
                };
                $scope.accountSelectorApi = {
                    onSelectorConfirm: refreshVoucherFilter
                };

                // 设置模型选择器
                $scope.modelTitle = $translate.instant('AddLinkModel');
                $scope.modelSelectorOptions = {
                    height: 500,
                    top: 140,
                    width: 1030
                };
                $scope.modelSelectorApi = {
                    onSelectorConfirm: refreshModelFilter
                };

                // 为发票税率多选下拉菜单进行国际化
                $scope.langSetting = {
                    selectAll: $translate.instant('SelectAll'),
                    selectNone: $translate.instant('SelectNone'),
                    reset: $translate.instant('Reset'),
                    nothingSelected: $translate.instant('PleaseSelect'),
                    search: $translate.instant('Search')
                };

                $scope.$on(enums.adminEvent.layoutChanged, function () {
                    return $timeout(function () {
                        if (_spread && _spread.refresh) {
                            _spread.refresh();
                        }
                    }, 50);
                });

                $scope.loadSheet = loadSheet;

                $scope.saveTemplateGroup = saveTemplateGroup;

                $scope.loadTemplateMenu = loadTemplateMenu;

                $scope.listTemplates = listTemplates;

                $scope.saveFormulaToTemp = saveFormulaToTemp;

                $scope.saveTemplate = saveTemplate;

                $scope.selectInvoiceOption = selectInvoiceOption;

                $scope.showSelectAccountPop = showSelectAccountPop;
                $scope.showSelectModelPop = showSelectModelPop;
                $scope.getServiceTypeList = getServiceTypeList;
                $scope.toggleServiceType = toggleServiceType;
                $scope.hasError = hasError;
                $scope.hidePanel = hidePanel;
                $scope.togglePanel = togglePanel;

                $scope.showSelectFormulaPop = showSelectFormulaPop;
                $scope.showSelectKeyValuePop = showSelectKeyValuePop;
                $scope.editFormulaParamOrKeyValue = editFormulaParamOrKeyValue;
                $scope.selectParam = selectParam;
                $scope.mentionMenuValidator = mentionMenuValidator;
                $scope.asyncFormulaValidate = asyncFormulaValidate;
                $scope.cancelHideTranslator = cancelHideTranslator;
                $scope.timerHideTranslator = timerHideTranslator;

                $scope.showSelectFormulaPopForValidation = showSelectFormulaPopForValidation;
                $scope.showSelectKeyValuePopForValidation = showSelectKeyValuePopForValidation;
                $scope.editFormulaParamOrKeyValueForValidation = editFormulaParamOrKeyValueForValidation;
                $scope.selectParamForValidation = selectParamForValidation;
                $scope.mentionMenuValidatorForValidation = mentionMenuValidatorForValidation;
                $scope.asyncFormulaValidateForValidation = asyncFormulaValidateForValidation;
                $scope.cancelHideTranslatorForValidation = cancelHideTranslatorForValidation;
                $scope.timerHideTranslatorForValidation = timerHideTranslatorForValidation;

                $('.templates-body').off("click").on("click", function () {
                    $('#' + dxControl.groupDropDownId).dxDropDownBox('close');
                    $('#' + dxControl.templateDropDownId).dxDropDownBox('close');
                    $("#dx-select-industry").dxSelectBox('close');
                    $('#dx-select-pay-tax-type').dxSelectBox('close');
                });

                $('#report-view').unbind("contextmenu").bind("contextmenu", processSpreadContextMenu);
                $("#spreadContextMenu a").click(processContextMenuClicked);
                $(document).on("contextmenu", function () {
                    event.preventDefault();
                    return false;
                });
            })();
        }
    ]);
systemConfigurationModule.directive('declarationFormConfiguration', ['$log',
    function ($log) {
        'use strict';
        $log.debug('declarationFormConfiguration.ctor()...');

        return {
            restrict: 'E',
            templateUrl: '/app/admin/systemConfiguration/declarationFormConfiguration/declaration-form-configuration.html' + '?_=' + Math.random(),
            replace: true,
            scope: {
                templateGroupId: '@',
                templateId: '@',
                cellTemplateId: '@'
            },
            controller: 'DeclarationFormConfigurationController',
            link: function (scope, element) {
                $("#editReportFormulaPop").draggable({ cursor: "cursor", scroll: true, cancel: 'input, label, .btn, .form-control, .close, div.label, .has-error' });
                $("#editReportFormulaPop").resizable({
                    minHeight: 610,
                    maxHeight: 610,
                    minWidth: 500
                });
                //$(".editReportFormulaPop-modal-content").draggable({ cursor: "cursor", scroll: true, cancel: 'input, label, .btn, .form-control, .close, div.label, .has-error' });

                $(".left").on("click", function (e) {
                    e.stopPropagation();
                });

                $('#btnCancel').on('click', function () {
                    $('.templates-save-form').css('display', 'none');
                });

                // Work-around for DX grid height issue.
                $('#editReportFormulaPop').on('shown.bs.modal', function () {
                    scope.refreshGrids();
                });

                scope.refreshGrids = function () {
                    $("#voucherGrid").dxDataGrid("instance").refresh();
                    $("#modelGrid").dxDataGrid("instance").refresh();
                };

                scope.toggleSaveAs = function () {
                    $('.templates-save-form').css('display', 'block');
                };

                scope.showLeftMenu = function () {
                    if ($('.left').is(":hidden")) {
                        $(".left").slideDown('slow');
                    } else {
                        $(".left").slideUp('slow');
                    }

                    $(document).one("click", function () {
                        $(".left").slideUp('slow');
                    });
                    event.stopPropagation();
                };

                scope.renderFinish = function () {
                    $('.templates-menus').first().addClass('active');
                };

                scope.refreshCategorySelect = function (data) {
                    // Change the appearance of isteven-multi-select, show the checkboxes of the
                    // options which were hidden, but the checkboxes always toggle twice.
                    // Add this to make checkboxes which were hidden work correctly.
                    var idx = _.findIndex(scope.invoiceOption.categoryOptions, { 'key': data.key });
                    if (idx >= 0) {
                        var chkbox = element.find('.invoice-category-select .multiSelectItem .checkbox:eq(' + idx + ')');
                        if (data.ticked && !chkbox.prop('checked')) {
                            chkbox.prop('checked', true);
                        }
                        else if (!data.ticked && chkbox.prop('checked')) {
                            chkbox.prop('checked', false);
                        }
                    }
                };

                scope.refreshTaxRateSelect = function (data) {
                    // Change the appearance of isteven-multi-select, show the checkboxes of the
                    // options which were hidden, but the checkboxes always toggle twice.
                    // Add this to make checkboxes which were hidden work correctly.
                    var idx = _.findIndex(scope.invoiceOption.rateOptions, { 'key': data.key });
                    if (idx >= 0) {
                        var chkbox = element.find('.tax-rate-select .multiSelectItem .checkbox:eq(' + idx + ')');
                        if (data.ticked && !chkbox.prop('checked')) {
                            chkbox.prop('checked', true);
                        }
                        else if (!data.ticked && chkbox.prop('checked')) {
                            chkbox.prop('checked', false);
                        }
                    }
                };
            }
        };
    }
]).directive('repeatDone', function () {
    return {
        link: function (scope, element, attrs) {
            if (scope.$last) {
                scope.$eval(attrs.repeatDone);

                $('.template-menu').on('click', function () {
                    $(this).parent().parent().parent().children().each(function () {
                        $(this).removeClass('active');
                    });
                    $(this).parent().parent().addClass('active');
                });

                $('.templates-menu').on('click', function () {
                    $(this).parent().parent().children().each(function () {
                        $(this).removeClass('active');
                    });
                    $(this).parent().addClass('active');
                });

                $('.templates-menus').on('click', function () {
                    $(this).parent().children().each(function () {
                        $(this).removeClass('active');
                    });
                    $(this).addClass('active');
                });
            }
        }
    }
});
systemConfigurationModule.controller('keyValueConfigEditingController', ['$log', '$scope', '$translate', 'serviceTypeService', 'orgService',
    'KeyValueConfigService', 'enums',
function ($log, $scope, $translate, serviceTypeService, orgService, KeyValueConfigService, enums) {
    var serviceTypeDic, induestryDic;
   
    serviceTypeService.getServiceTypeList().success(function (serviceTypesFromServer) {
        $scope.serviceTypes = serviceTypesFromServer;
        serviceTypeDic = PWC.toDict($scope.serviceTypes, 'id');
    });

    orgService.getProjectIndustrys().success(function (industriesFromSever) {
        $scope.industries = industriesFromSever;
        induestryDic = PWC.toDict($scope.industries, 'id');
    });

    $scope.industriesSelectOption = {
        displayExpr: "name",
        valueExpr: "id",
        showSelectionControls: true,
        searchEnabled: true,
        bindingOptions: {
            value: 'editingModel.industryIds',
            dataSource: 'industries',
            disabled: 'state === 0'
        },
        placeholder: $translate.instant('SelectIndustryRequired')
    };
    $scope.industriesSelectOptionValid = {
        name: 'industry',
        validationRules: [
            { type: 'required', message: $translate.instant('RequiredInfo') }
        ],
        validationGroup:'keyValueGroup'
    };
    $scope.serviceTypeSelectOptionValid = {
        name: 'serviceType',
        validationRules: [
            { type: 'required', message: $translate.instant('RequiredInfo') }
        ],
        validationGroup: 'keyValueGroup'
    };
    $scope.serviceTypeSelectOption = {
        displayExpr: "name",
        valueExpr: "id",
        showSelectionControls: true,
        searchEnabled: true,
        bindingOptions: {
            value: 'editingModel.serviceTypeIds',
            dataSource: 'serviceTypes',
            disabled: 'state === 0'
        },
        placeholder: $translate.instant('SelectTaxType')
    };

    $scope.saveEditingKvc = function () {
        angular.forEach($scope.kvcForm.$error.required, function (field) {
            field.$setDirty();
        });
        var fieldValidateResult = DevExpress.validationEngine.getGroupConfig("keyValueGroup").validate();
        
        if (!($scope.kvcForm.$valid &&fieldValidateResult.isValid)) {
            return;
        }
        $scope.editingModel.serviceTypeNames = [];
        $scope.editingModel.industryNames = [];

        _.each($scope.editingModel.serviceTypeIds,function (serviceType) {
            $scope.editingModel.serviceTypeNames.push(serviceTypeDic[serviceType].name);
        });
        $scope.editingModel.serviceTypes = $scope.editingModel.serviceTypeNames.join(',');
        _.each($scope.editingModel.industryIds,function (industryId) {
            $scope.editingModel.industryNames.push(induestryDic[industryId].name);
        });
        $scope.editingModel.industrys = $scope.editingModel.industryNames.join(',');
        $scope.editingModel.dataSource = $scope.editingModel.dataSourceKey + '_' + $scope.editingModel.dataSourceName;
        var copiedEditingModel = angular.copy($scope.editingModel);
        if ($scope.state === 2) {
            KeyValueConfigService.addNewKeyValueConifg($scope.editingModel).success(function () {
                $scope.editDone({ editingModel: copiedEditingModel });
                var dataGrid = $("#KVgridContainer").dxDataGrid("instance");
                dataGrid.refresh();
            });
        } else if ($scope.state === 1) {
            KeyValueConfigService.updateNewKeyValueConifg($scope.editingModel).success(function () {
                $scope.editDone({ editingModel: copiedEditingModel });
                var dataGrid = $("#KVgridContainer").dxDataGrid("instance");
                dataGrid.refresh();
            });
        }
        $('#kvc-editing-modal').modal('hide');

    };

    $scope.editKvc = function () {
        $scope.state = 1;
    };

    $('#kvc-editing-modal').on('hidden.bs.modal', function () {
        $scope.editingModel = {};
        $scope.kvcForm.$setPristine();
    });
   
    $scope.keyInValue = null;
    //var dataSourceChange = function (selectedItem) {
    //    if (_.isNull(selectedItem)) return;
    //    if (selectedItem.key === enums.formulaDataSourceType.KeyInSource) {
    //        if ($scope.state === 2) {
    //            $scope.editingModel.dataSourceName = null;
    //        }
    //        $('#keyInValue').show();
    //    } else {
    //        $scope.editingModel.dataSourceName = selectedItem.value;
    //        $('#keyInValue').hide();
    //    }
    //};
    $scope.dataSourceNameOptions = {
        disabled: true
    };
    //$scope.datasourceOptions = {
    //    bindingOptions: {
    //        dataSource: 'dataSourceList',
    //        disabled: 'state === 0'
    //    },
    //    //dataSource: $scope.dataSourceList,
    //    valueExpr: 'key',
    //    displayExpr: 'value',
    //    onSelectionChanged: function (e) {

    //        var selectedItem = e.selectedItem;
    //        dataSourceChange(selectedItem);
    //    }
    //};
    var formatCharacters = ['(', '+', '-', '*', '/'];
    $scope.mappingDataSource = function () {
        var formula = $scope.editingModel.formula
        if (formula) {
            var r = formula.replace(/[\(|\+|\-|\*|\/]/g, "#");
            formula = r.split('#')[0];
            var source = _.findWhere($scope.dataSourceList, {formulaName:formula});
            if (_.isUndefined(source)) {
                source = {
                    dataSourceType: 10,
                    dataSourceName: $translate.instant('KeyInSource')
                }
            }
            $scope.editingModel.dataSourceKey = source.dataSourceType;
            $scope.editingModel.dataSourceName = source.dataSourceName;
            //KeyValueConfigService.mappingDataSource(formula).success(function (res) {
            //    if (res) {
            //       // var obj = { id: res.data.dataSourceID, name: res.data.dataSourceName };
            //        $scope.editingModel.dataSourceKey = res.data.dataSourceID;
            //        $scope.editingModel.dataSourceName = res.data.dataSourceName;
            //       // dataSourceChange(obj);

            //    }
                
            //}).error(function (err) {
            //    console.log(err);
            //});
        }
    }
    //var getAllKeyCode = function () {
    //    KeyValueConfigService.getAll().success(function (data) {
    //        $scope.mentionList = _.map(data, function (config) { return { code: '@' + config.keyCode, name: config.name, description: config.description } });
    //    });
    //};
    var getAllDataSource = function () {
        KeyValueConfigService.getAllDataSource().success(function (res) {
            $scope.dataSourceList = res.data;
        });
    };

    (function initalization() {
        //getAllKeyCode();
      //  $scope.dataSourceList = _.chain(enums.formulaDataSourceType)
        //.pairs().map(function (obj) { return { id: obj[1], name: $translate.instant(obj[0]) }; }).value();
        getAllDataSource();
        //$('#kvc-editing-modal').on('show.bs.modal', function () {
        //    // 执行一些动作...
        //    $('#keyInValue').hide();
        //})
        var titleStr = $scope.state === 0 ? 'ViewKeyValueConfig' : $scope.state === 1 ? 'UpdateKeyValueConfiguration' : $scope.state === 2 ? 'AddKeyValueConfiguration' : '';
        $scope.modalTitle = $translate.instant(titleStr);
    })();
}]);
systemConfigurationModule.directive('keyValueConfigEditing', ['$log',
    function ($log) {
        return {
            restrict: 'E',
            templateUrl: '/app/admin/systemConfiguration/keyValueConfiguration/key-value-configuration-editing.html' + '?_=' + Math.random(),
            replace: true,
            scope: {
                editingModel:'=',
                state: '=',
                editDone: '&',
                actionAuthority:'='
            },
            controller: 'keyValueConfigEditingController',
            link: function (scope, element, attr) {
            }
        }
    }]);
systemConfigurationModule.controller('keyValueConfigController', ['$log', '$scope', '$translate', '$q',
  'KeyValueConfigService', '$uibModal', 'SweetAlert', 'enums',
function ($log, $scope, $translate, $q, KeyValueConfigService, $uibModal, SweetAlert, enums) {
    var prefectDisplay = function (text, len) {
        (text.length > len) ? text.string.substring(0, len) + '...' : text;
    };
    var selectedEditModel;
    var removeKeyValue = function (key) {
        KeyValueConfigService.remove(key).then(function (res) {
            if (res.data === 0) {
                SweetAlert.warning('', $translate.instant('ActionFail'));
                //delete fail
            } else if (res.data === -1) {
                //has reference
                SweetAlert.warning('', $translate.instant('IsUsed'));
            } else {
                // SweetAlert.warning('', $translate.instant('DeleteSuccess'));
                var dataGrid = $("#KVgridContainer").dxDataGrid("instance");
                dataGrid.refresh();
                //return $q.reject($translate.instant("DeleteSuccess"));
            }

        }, function (response) {
            //return $q.reject($translate.instant("ActionFail"));
            SweetAlert.warning('', $translate.instant('ActionFail'));
        });
    }
    var deFineKeyValueGrid = function () {
        var keyValues = new DevExpress.data.CustomStore({
            key: 'id',
            load: function (loadOptions) {
                return KeyValueConfigService.getAll().then(function (res) {
                    return { data: res.data, totalCount: res.data.length };
                }, function (response) {
                    return $q.reject($translate.instant("DataLoadingError"));
                });
            }

        });

        $scope.keyValues = keyValues;
        $scope.keyValueOptions = {
            bindingOptions: {
                dataSource: {
                    deep: false,
                    dataPath: 'keyValues'
                },
            },
            selection: {
                mode: "single"
            },
            //onSelectionChanged: function (selectedItems) {
            //},
            paging: {
                pageSize: 15
            },
            filterRow: {
                visible: true
            },
            allowColumnResizing: true,
            loadPanel: {
                enabled: false
            },
            searchPanel: {
                visible: true,
                width: 240,
                placeholder: $translate.instant('SearchHintText')
            },
            //editing: {
            //    mode: "row",
            //    allowDeleting: true,
            //    texts: {
            //        deleteRow: $translate.instant('Delete')
            //    }
            //},
            "export": {
                enabled: true,
                fileName: $translate.instant('KeyValueConfiguration'),
                allowExportSelectedData: false,
                texts: {
                    exportAll: $translate.instant('ExportAllData'),
                    exportTo: $translate.instant('Export')
                }
            },
            height: '95%',
            scrolling: {
                mode: 'standard'
            },
            onToolbarPreparing: function (e) {
                var toolbarItems = e.toolbarOptions.items;
                toolbarItems.unshift({
                    location: "before",
                    template: "add"
                });
            },
            //onSelectionChanged: function (selectedItems) {
            //    selectedEditModel = selectedItems.selectedRowsData[0];
            //    if (!selectedEditModel) return;
            //    if (selectedEditModel.keyValueType !== 2) {
            //        return;
            //    }
            //    $scope.editingModel = angular.copy(selectedEditModel);
            //    var vals = selectedEditModel.dataSource.split('_');
            //    if (vals.length === 2) {
            //        $scope.editingModel.dataSourceKey = parseInt(vals[0]);
            //        $scope.editingModel.dataSourceName = vals[1];
            //        //if ($scope.editingModel.dataSourceKey === enums.formulaDataSourceType.KeyInSource) {
            //        //    $scope.editingModel.dataSourceName = vals[1];
            //        //}

            //    }

            //    $scope.state = 0;
            //    $('#kvc-editing-modal').modal('show');
            //},
            //onRowClick: function (e) {
            //    //console.log(e);
            //    selectedEditModel = e.data;
            //    if (!selectedEditModel) return;
            //    //if (selectedEditModel.keyValueType !== 2) {
            //    //    return;
            //    //}
            //    $scope.editingModel = angular.copy(selectedEditModel);
            //    $scope.state = 0;
            //    $('#kvc-editing-modal').modal('show');
            //},
            columns: [
                { dataField: 'id', visible: false,allowExporting:false },
                //{ dataField: 'keyCode', caption: $translate.instant('KeyCode'), fixed: true },
                {
                    dataField: 'name', caption: $translate.instant('KeyName'), fixed: true,
                    filterOperations: [], selectedFilterOperation: 'contains', alignment: 'left'
                },
                {
                    dataField: 'formula', caption: $translate.instant('TemplateFormulaDesc'),
                    filterOperations: [], selectedFilterOperation: 'contains'
                },
                {
                    dataField: 'description', caption: $translate.instant('Description'),
                    filterOperations: [], selectedFilterOperation: 'contains'
                },

                {
                    dataField: 'dataSource', caption: $translate.instant('DataSourceName'),
                    filterOperations: [], selectedFilterOperation: 'contains', width: '10%',
                    calculateCellValue: function (rowData) {
                        if (rowData.dataSource) {
                            var s = rowData.dataSource.split('_');
                            return s.length === 2 ? s[1] : s[0];
                        }
                        return rowData.dataSource;
                    },
                    calculateFilterExpression: function (filterValue, selectedFilterOperation) {
                        return [this.calculateCellValue, selectedFilterOperation || 'contains', filterValue];
                    }
                },
                {
                    width: '10%',
                    caption: $translate.instant('ApplicationScope'),
                    filterOperations: [], selectedFilterOperation: 'contains',
                    calculateCellValue: function (rowData) {
                        if (rowData.scopeSummary) {
                            var arry = rowData.scopeSummary.split(',');
                            var tArry = '';
                            if (arry.length > 1) {
                                _.each(arry, function (i) {
                                    tArry = tArry + ',' + $translate.instant(i);
                                });
                                tArry = tArry.substring(1, tArry.length);
                            } else {
                                tArry = $translate.instant(arry[0]);
                            }

                            return tArry;
                        }

                    },
                    calculateFilterExpression: function (filterValue, selectedFilterOperation) {
                        return [this.calculateCellValue, selectedFilterOperation || 'contains', filterValue];
                    }
                },
                {
                    width: '10%',
                    dataField: 'keyValueType', caption: $translate.instant('KeyValueType'),
                    filterOperations: [], selectedFilterOperation: 'contains',
                    calculateCellValue: function (rowData) {
                        return rowData.keyValueType ? constant.keyValueType[rowData.keyValueType - 1].name : null;
                    },
                    calculateFilterExpression: function (filterValue, selectedFilterOperation) {
                        return [this.calculateCellValue, selectedFilterOperation || 'contains', filterValue];
                    }
                },
                {
                    alignment: 'center', allowExporting: false,
                    //caption: $translate.instant('Delete'),
                    width: '8%', cellTemplate: function (container, options) {
                        var data = options.data;
                        //custom key value
                        if ( data.keyValueType === 2 &&
                            $scope.actionPermission) {
                            $('<i class="fa fa-edit delete-icon" aria-hidden="true" style="margin-right:8px;"></i>').click(function (event) {
                                event.stopPropagation();
                               
                                $scope.editingModel = angular.copy(data);
                                if (data&&data.dataSource) {
                                    var vals = data.dataSource.split('_');
                                    if (vals.length === 2) {
                                        $scope.editingModel.dataSourceKey = parseInt(vals[0]);
                                        $scope.editingModel.dataSourceName = vals[1];                                       
                                    }
                                }
                               
                                $scope.state = 0;
                                $('#kvc-editing-modal').modal('show');
                               
                            }).appendTo(container);
                            $('<i class="fa fa-trash delete-icon" aria-hidden="true"></i>').click(function (event) {
                                event.stopPropagation();
                                //show message
                                SweetAlert.swal({
                                    title: $translate.instant('ConfirmDelete'),
                                    text: '',
                                    type: "warning",
                                    showCancelButton: true,
                                    confirmButtonColor: "#DD6B55",
                                    confirmButtonText: $translate.instant('Confirm'),
                                    cancelButtonText: $translate.instant('Cancel'),
                                    closeOnConfirm: true
                                }, function (isConfirm) {
                                    if (isConfirm) {
                                        removeKeyValue(data.id);
                                    }
                                });
                            }).appendTo(container);
                        }
                    }
                }
            ],
            masterDetail: {
                enabled: true,
                template: "detail"
            }
        };
    };
    var deFineDetailTab = function () {
        $scope.detailPanel = function (key, countDic) {            
            return {
                height:400,
                titleTemplate: function (itemData, itemIndex, itemElement) {
                    itemElement.append("<p>" + itemData.title + "</p>");
                },
                dataSource: [
                    {
                        title: $translate.instant('ReferenceFinancialReport') + "-" + (countDic["1"] ? countDic["1"] : 0),
                        template: 'financialReportTemplate',
                        FROptions: {
                            noDataText: '',
                            loadPanel: {
                                enabled: false
                            },
                            paging: {
                                enabled: true,
                                pageSize:50
                            },
                            //scrolling:{
                            //    showScrollbar: 'always'
                            //},
                            height: 300,
                            dataSource: {
                                store: new DevExpress.data.CustomStore({
                                    load: function (loadOptions) {
                                        return KeyValueConfigService.getFinacialReference(key);
                                    }
                                })
                            },
                            columns: [
                                { caption: $translate.instant('TemplateName'), dataField: 'name' },
                                { caption: $translate.instant('IndustryID'), dataField: 'industry' },
                                {
                                    caption: $translate.instant('Cell'), width: '6%',
                                    calculateCellValue: function (rowData) {
                                        return PWC.numToExcelChar(rowData.row, rowData.column);
                                    }
                                },
                                { caption: $translate.instant('RowName'), dataField: 'row', width: '6%' },
                                { caption: $translate.instant('ColumnName'), dataField: 'column', width: '6%' }
                            ]
                        }
                    },
                    {
                        title: $translate.instant('ReferenceTaxReport') + "-" + (countDic["2"] ? countDic["2"] : 0),
                        template: 'taxReportTemplate',
                        TROptions: {
                            noDataText: '',
                            loadPanel: {
                                enabled: false
                            },
                            paging: {
                                enabled: true,
                                pageSize: 50
                            },
                            //scrolling: {
                            //    showScrollbar: 'always'
                            //},
                            height:300,
                            dataSource: {
                                store: new DevExpress.data.CustomStore({
                                    load: function (loadOptions) {
                                        return KeyValueConfigService.getTaxReference(key);
                                    }
                                })
                            },
                            columns: [
                                //{ dataField: "cellTemplateID", visible: false },
                                //{ dataField: "templateID", visible: false },
                                //{ dataField: "templateGroupID", visible: false },
                                { caption: $translate.instant('ReportType'), dataField: 'category' },
                                { caption: $translate.instant('IndustryID'), dataField: 'industry' },
                                {
                                    dataField: 'payTaxType',
                                    caption: $translate.instant('PayTaxType'),
                                    calculateCellValue: function (rowData) {
                                        return rowData.payTaxType ? constant.payTaxTypeList[rowData.payTaxType - 1].name : null;
                                    }
                                },

                                { caption: $translate.instant('TemplateGroupName'), dataField: 'templateGroupName' },
                                {
                                    dataField: 'name',
                                    caption: $translate.instant('TemplateName'),
                                    cellTemplate: 'taxLink'
                                },
                                 {
                                     caption: $translate.instant('Cell'), width: '6%',
                                     calculateCellValue: function (rowData) {
                                         return PWC.numToExcelChar(rowData.row, rowData.column);
                                     }

                                 },
                                { caption: $translate.instant('RowName'), dataField: 'row', width: '6%' },
                                { caption: $translate.instant('ColumnName'), dataField: 'column', width: '6%' }
                            ]                            
                        }
                    },
                    {
                        // 'title': $translate.instant('AnalyticsModel') + "<span class='count-custom'>" + (countDic["3"] ? countDic["3"] : 0)+"</span>",
                        'title': $translate.instant('AnalyticsModel') + "-" + (countDic["3"] ? countDic["3"] : 0),
                        'template': 'modelTemplate',
                        'MTOptions': {
                            //noDataText: $translate.instant('NoDataAvailable'),
                            noDataText: '',
                            loadPanel: {
                                enabled: false
                            },
                            paging: {
                                enabled: true,
                                pageSize: 50
                            },
                            //scrolling: {
                            //    showScrollbar: 'always'
                            //},
                            height: 300,
                            dataSource: {
                                store: new DevExpress.data.CustomStore({
                                    load: function (loadOptions) {
                                        return KeyValueConfigService.getModelReference(key);
                                    }
                                })
                            },
                            columns: [
                            //{ dataField: "modelID", visible: false },
                            { caption: $translate.instant('ModelCode'), dataField: 'code' },
                            { caption: $translate.instant('ModelName'), dataField: 'name' },
                            {
                                caption: $translate.instant('ModelType'), dataField: 'type',
                                calculateCellValue: function (rowData) {
                                    return $translate.instant(enums.modelType[rowData.type]);
                                }
                            },
                            { caption: $translate.instant('IndustryID'), dataField: 'industry' },
                            {
                                dataField: 'entityName',
                                caption: $translate.instant('OrganizationName'),
                                cellTemplate: 'modelLink'
                            }
                            ]
                        }
                    }

                ]
            };
        };

    };

    $scope.addNewKvc = function () {
        $('#kvc-editing-modal').modal('show');
        $scope.state = 2;
        $scope.editingModel = {};
    };

    $scope.editDone = function (editingModel) {
        if ($scope.state === 1) {
            PWC.assignModel(editingModel, selectedEditModel);
        }
    };
    // 关于日志部分
    $scope.showOperateLogPop = function () {
        $scope.isShowLog = true;
    };
    var checkPermissionList = function () {
        var list = [];
        var userManageTemp = constant.adminPermission.systemConfiguration.keyValueConfig;
        list = _.values(userManageTemp);
        $scope.viewPermission = false;
        $scope.actionPermission = false;
        $scope.$root.checkUserPermissionList(list).success(function (data) {
            $scope.viewPermission = data[userManageTemp.viewKey];
            $scope.actionPermission = data[userManageTemp.keyAction];
        });
    };
    (function initialize() {
        checkPermissionList();
        deFineKeyValueGrid();
        deFineDetailTab();
    })();
}]);
systemConfigurationModule.directive('keyValueConfig', ['$log',
    function ($log) {
        return {
            restrict: 'E',
            templateUrl: '/app/admin/systemConfiguration/keyValueConfiguration/key-value-configuration.html' + '?_=' + Math.random(),
            replace: true,
            scope: {},
            controller: 'keyValueConfigController',
            link: function (scope, element,attr) {

            }
        }
    }]);
systemConfigurationModule.controller('ModelConfigurationController', [
    '$scope', '$log', 'uiGridConstants', 'SweetAlert', '$translate', 'stdAccountService', 'enterpriseAccountService', '$interval', 'orgService', '$timeout', '$q', '$uibModal', 'modelConfigurationService', 'enums', "projectService", 'KeyValueConfigService', '$location', "dxDataGridService",
    function ($scope, $log, uiGridConstants, SweetAlert, $translate, stdAccountService, enterpriseAccountService, $interval, orgService, $timeout, $q, $uibModal, modelConfigurationService, enums, projectService, KeyValueConfigService, $location, dxDataGridService) {
        'use strict';


        //模型编辑状态枚举
        $scope.editStatusEnum = {
            view: 1,
            edit: 2,
            add: 3
        };

        //模型类型枚举
        $scope.modelTypeEnum = {
            systemCommon: {
                id: 1,
                name: "系统公共模型",
                typeAbbr: "COM"
            },
            organizationCommon: {
                id: 2,
                name: "机构公共模型",
                typeAbbr: "COM"
            },
            organizationCustomize: {
                id: 3,
                name: "机构自定义模型",
                typeAbbr: "UDF"
            }
        };

        //模型特性枚举
        $scope.modelFeatureEnum = {
            indexAnalysis: {
                id: 1,
                name: $translate.instant('IndexAnalysis'),
                abbr: "IA"
            },
            entriesCheck: {
                id: 2,
                name: $translate.instant('JournalCompare'),
                abbr: "EC"
            }
        };

        $scope.queryParams = {
            //查询模板
            listQueryCondition: [],
            //机构ID
            organizationID: "",
            organizationName: "",
            modelID: "",
            parentId: ""
        };
        $scope.selectNode = {};
        $scope.filterData = {
            modelName: "",
            isSystemModel: false,
            isCommonModel: false,
            isCustomModel: false,
            isEnabledModel: false,
            isDisableModel: false
        };

        $scope.buttonShow = {
            editBtnShow: true,
            saveAsBtnShow: true,
            defaultSetBtnShow: false,
            saveBtnShow: false,
            cancelBtnShow: false
        };

        $scope.buttonDisabled = {
            editBtnShow: true,
            saveAsBtnShow: true,
            defaultSetBtnShow: true,
            saveBtnShow: true,
            cancelBtnShow: true,
            previeBtnShow: true
        };
        $scope.haveModelTreeNode = false;
        $scope.isSaveClick = false;
        $scope.dataList = [];
        //模型树
        $scope.modelTreeList = [];
        //模型信息复制
        $scope.copyModelDetail = {};
        $scope.events = [];

        $scope.validationRules = {
            modelName: [
                {
                    type: 'required',
                    message: '请输入模型名称'
                },
                {
                    type: 'stringLength',
                    min: 4,
                    max: 6,
                    message: '请输入 4 到 50 字的模型名称'
                },
                {
                    type: 'pattern',
                    pattern: '^[a-zA-Z]+$',
                    message: '请输入不包含特殊字符的模型名称'
                }
            ]
        };

        $scope.editDetailBtnDisabled = false;
        $scope.addDetailBtnDisabled = true;
        $scope.deleteDetailBtnDisabled = false;
        $scope.indexBtnShow = false;
        $scope.entriesBtnShow = false;
        $scope.gridColumns = [];
        //指标分析编辑行计算顺序
        $scope.currentRowSequence = -1;
        $scope.currentRowIndexShow = -1;

        //视图/编辑模式下使用的 Model
        $scope.viewEditModel = {
            basicModel: {},
            detailModel: {
                featureId: 1,
                dataList: [],
                formulaCriterion: {}
            },
            editStatus: 1,
            showType: 1,
            validationGroupName: "viewEditGroup",
            serviceList: []
        };

        $scope.serviceList = [];
        //创建模式下使用的 Model
        $scope.createModel = {
            basicModel: {},
            detailModel: {
                featureId: 1,
                dataList: [],
                formulaCriterion: {}
            },
            editStatus: 3,
            showType: 2,
            validationGroupName: "createGroup",
            serviceList: []
        };

        //查看默认设置模式下使用的 Model
        $scope.seeDefaultModel = {
            basicModel: {},
            detailModel: {
                featureId: 1,
                dataList: [],
                formulaCriterion: {}
            },
            editStatus: 1,
            showType: 1,
            validationGroupName: "seeDefaultGroup",
            serviceList: []
        };
        $scope.keyValueDataSource = [];
        $scope.industryInfo = {};
        $scope.currentEditOrAddModelCode = '';
        $scope.dxControlsOptions = {
            name: {
                ControlOption: {
                    bindingOptions: {
                        value: "viewEditModel.basicModel.name"
                    }
                },
                validatorOption: {
                    validationRules: [
                        {
                            type: 'required',
                            message: '请输入模型名称'
                        },
                        {
                            type: 'stringLength',
                            max: 50,
                            message: '请输入不大于 50 字的模型名称'
                        }
                    ]
                }
            }
        };

        $scope.isShowKeyValue = false;
        var showSelectKeyValuePop = function () {

        };
        $scope.keyValueSelectorApi = {
            onSelectorConfirm: function () {
                var selection = this.getSelection();
                if (!_.isEmpty(selection)) {
                    $scope.editModel.formula = $.trim($scope.editModel.formula) + ' @' + selection.keyCode + ' ';
                }
            }
        };
        $scope.keyValueTitle = $translate.instant('AddKeyValue');
        $scope.keyValueSelectorOptions = {
            top: 120
        };

        //设置Button的显示
        var setButtonShow = function (index) {

            if (index == 1 || index == 2) {
                $scope.buttonShow.editBtnShow = false;
                $scope.buttonShow.saveAsBtnShow = false;
                $scope.buttonShow.defaultSetBtnShow = false;
                $scope.buttonShow.saveBtnShow = true;
                $scope.buttonShow.cancelBtnShow = true;
                $scope.editDetailBtnDisabled = true;
                $scope.deleteDetailBtnDisabled = true;
                $scope.addDetailBtnDisabled = false;
            }
            if (index == 4 || index == 5) {
                $scope.buttonShow.editBtnShow = true;
                $scope.buttonShow.saveAsBtnShow = true;
                $scope.buttonShow.defaultSetBtnShow = false;
                $scope.buttonShow.saveBtnShow = false;
                $scope.buttonShow.cancelBtnShow = false;
                $scope.editDetailBtnDisabled = false;
                $scope.deleteDetailBtnDisabled = false;
                $scope.addDetailBtnDisabled = true;
            }
        };

        var setButtonDisabled = function (disabled, modelType) {
            if (modelType === 3) {
                $scope.buttonDisabled.editBtnShow = (!disabled && $scope.hasMaintainCustomPermission) ? false : true;
                $scope.buttonDisabled.defaultSetBtnShow = (!disabled && $scope.hasMaintainCustomPermission) ? false : true;
            } else {
                $scope.buttonDisabled.editBtnShow = (!disabled && $scope.hasEditPermission) ? false : true;
                $scope.buttonDisabled.defaultSetBtnShow = (!disabled && $scope.hasEditPermission) ? false : true;
            }
            $scope.buttonDisabled.saveAsBtnShow = (!disabled && $scope.hasMaintainCustomPermission) ? false : true;
            $scope.buttonDisabled.saveBtnShow = disabled;
            $scope.buttonDisabled.cancelBtnShow = disabled;
            $scope.buttonDisabled.previeBtnShow = disabled;
        };

        //自动生成模型Code
        var getNewModelCode = function (ruleId, feature, modelType) {
            var newCode = "";
            if (ruleId < 10) {
                newCode = "0" + ruleId;
            } else {
                newCode = ruleId + "";
            }
            if (feature == $scope.modelFeatureEnum.indexAnalysis.id) {
                newCode += "." + $scope.modelFeatureEnum.indexAnalysis.abbr;
            }
            if (feature == $scope.modelFeatureEnum.entriesCheck.id) {
                newCode += "." + $scope.modelFeatureEnum.entriesCheck.abbr;
            }
            if (modelType == $scope.modelTypeEnum.systemCommon.id || modelType == $scope.modelTypeEnum.organizationCommon.id) {
                newCode += "." + $scope.modelTypeEnum.systemCommon.typeAbbr + "." + randomWord(true, 5);
            }
            if (modelType == $scope.modelTypeEnum.organizationCustomize.id) {
                newCode += "." + $scope.modelTypeEnum.organizationCustomize.typeAbbr + "." + randomWord(false, 5);
            }
            return newCode;
        };
        //生成随机字符串
        var randomWord = function (allNumber, stringLength) {
            var str = "";
            var arr = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"];
            var arrNum = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"];
            for (var i = 0; i < stringLength; i++) {
                if (allNumber) {
                    var pos = Math.round(Math.random() * (arrNum.length - 1));
                    str += arrNum[pos];
                } else {
                    var pos = Math.round(Math.random() * (arr.length - 1));
                    str += arr[pos];
                }
            }

            return str;
        };

        $scope.doDataFilter = function () {
            $scope.queryParams.listQueryCondition = [];
            if (PWC.isNullOrEmpty($scope.queryParams.organizationID)) {
                return;
            }
            if (!PWC.isNullOrEmpty($scope.filterData.modelName)) {
                var modelNameMto = {};
                modelNameMto.conditionName = "模型名称";
                modelNameMto.searchValue = $scope.filterData.modelName;
                $scope.queryParams.listQueryCondition.push(modelNameMto);
            }
            if (!$scope.filterData.isSystemModel || !$scope.filterData.isCommonModel || !$scope.filterData.isCustomModel) {
                if ($scope.filterData.isSystemModel) {
                    var systemMto = {};
                    systemMto.conditionName = "模型类型";
                    systemMto.searchValue = 1;
                    $scope.queryParams.listQueryCondition.push(systemMto);
                }
                if ($scope.filterData.isCommonModel) {
                    var commonMto = {};
                    commonMto.conditionName = "模型类型";
                    commonMto.searchValue = 2;
                    $scope.queryParams.listQueryCondition.push(commonMto);
                }
                if ($scope.filterData.isCustomModel) {
                    var customMto = {};
                    customMto.conditionName = "模型类型";
                    customMto.searchValue = 3;
                    $scope.queryParams.listQueryCondition.push(customMto);
                }
            }
            if (!$scope.filterData.isEnabledModel || !$scope.filterData.isDisableModel) {
                if ($scope.filterData.isEnabledModel) {
                    var enabledMto = {};
                    enabledMto.conditionName = "使用状态";
                    enabledMto.searchValue = 1;
                    $scope.queryParams.listQueryCondition.push(enabledMto);
                }
                if ($scope.filterData.isDisableModel) {
                    var disableMto = {};
                    disableMto.conditionName = "使用状态";
                    disableMto.searchValue = 0;
                    $scope.queryParams.listQueryCondition.push(disableMto);
                }
            }
            loadModelConfigurationTree();
            $('.filter-button').popover("hide");
        };

        $scope.doDataFilterReset = function () {
            $scope.queryParams.listQueryCondition = [];
            $scope.filterData = {
                modelName: "",
                isSystemModel: false,
                isCommonModel: false,
                isCustomModel: false,
                isEnabledModel: false,
                isDisableModel: false
            };
            loadModelConfigurationTree();
            $('.filter-button').popover("hide");
        };

        var getFirstMoelInTheTree = function (node) {
            if (!node)
                return null;

            if (!node.isCategory)
                return node;

            if (node.isCategory) {

                for (var i = 0; i < node.nodes.length; i++) {
                    var subNode = node.nodes[i];
                    var mNode = getFirstMoelInTheTree(subNode);
                    if (mNode)
                        return mNode;
                }
            }

            return null;
        };

        var getModelInTheTreeByModelCode = function (modelCode, list) {
            if (PWC.isNullOrEmpty(modelCode) || !list) {
                return null;
            }
            for (var i = 0; i < list.length; i++) {
                var node = list[i]
                if (!node.isCategory && node.modelCode === modelCode) {
                    return node;
                }
                if (node.nodes.length > 0) {
                    var temp = getModelInTheTreeByModelCode(modelCode, node.nodes);
                    if (temp) {
                        return temp;
                    }
                }
            }
            return null;
        };

        var loadModelConfigurationTree = function () {
            if (!PWC.isNullOrEmpty($scope.queryParams.organizationID)) {
                modelConfigurationService.getModelList($scope.queryParams.organizationID, $scope.queryParams.listQueryCondition).success(function (data) {

                    var dic = {};
                    for (var ix = 0; ix < data.length; ix++) {
                        var item = data[ix];
                        dic[item.modelID] = {
                            id: item.modelID,
                            title: item.modelName,
                            modelCode: item.modelCode,
                            isCategory: item.isCategory,
                            quantity: 0,
                            parentId: item.parentId,
                            modelType: item.modelType,
                            parentNode: null,
                            sort: item.sort,
                            data: item,
                            nodes: [],
                            isEdit: false,
                            isAdd: false
                        };
                    }

                    //查找每个节点是否存在父节点,并维护父节点与本节点的关系
                    for (var key in dic) {
                        if (!dic.hasOwnProperty(key))
                            continue;
                        var item = dic[key];
                        var parentId = item.parentId;
                        if (!parentId || !dic.hasOwnProperty(parentId))
                            continue;

                        var p = dic[parentId];
                        if (!p && p === item)
                            continue;

                        item.parentNode = p;
                        p.nodes.push(item);
                    }

                    var srcData = [];
                    for (var key in dic) {
                        if (!dic.hasOwnProperty(key))
                            continue;

                        var item = dic[key];
                        if (!item || item.parentNode)
                            continue;

                        srcData.push(item);
                    }
                    $scope.modelTreeList = srcData;
                    refreshModelTreeList();


                    var mNode = null;
                    if (!PWC.isNullOrEmpty($scope.currentEditOrAddModelCode)) {
                        mNode = getModelInTheTreeByModelCode($scope.currentEditOrAddModelCode, $scope.modelTreeList);
                        $scope.currentEditOrAddModelCode = '';
                    }
                    if (mNode) {
                        selectModel(mNode);
                    } else {
                        //完成树形控件初始化后,默认加载第一个节点,若不存在任何模型则将右侧详细信息置空                       
                        for (var i = 0; i < $scope.modelTreeList.length; i++) {
                            mNode = getFirstMoelInTheTree($scope.modelTreeList[i]);
                            if (mNode) {
                                selectModel(mNode);
                                break;
                            }
                        }
                        if (!mNode) {
                            $scope.selectNode = {};
                            $(".content-container .right-container > div").hide();
                        }
                    }
                });
            }

        };

        $scope.propOrders = {
            "isCategory": "desc",
            "sort": "asc",
            "modelType": "asc",
            "modelCode": "asc"
        };

        var sortByProps = function (item1, item2) {
            "use strict";
            var props = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                props[_i - 2] = arguments[_i];
            }

            var cps = []; // 存储排序属性比较结果。
            // 如果未指定排序属性，则按照全属性升序排序。    
            var asc = true;
            if (props.length < 1) {
                for (var p in item1) {
                    if (item1[p] > item2[p]) {
                        cps.push(1);
                        break; // 大于时跳出循环。
                    } else if (item1[p] === item2[p]) {
                        cps.push(0);
                    } else {
                        cps.push(-1);
                        break; // 小于时跳出循环。
                    }
                }
            } else {
                for (var i = 0; i < props.length; i++) {
                    var prop = props[i];
                    for (var o in prop) {
                        asc = prop[o] === "asc";
                        if (item1[o] > item2[o]) {
                            cps.push(asc ? 1 : -1);
                            break; // 大于时跳出循环。
                        } else if (item1[o] === item2[o]) {
                            cps.push(0);
                        } else {
                            cps.push(asc ? -1 : 1);
                            break; // 小于时跳出循环。
                        }
                    }
                }
            }

            for (var j = 0; j < cps.length; j++) {
                if (cps[j] === 1 || cps[j] === -1) {
                    return cps[j];
                }
            }
            return 0;
        }

        var listOrder = function (propOrders, list) {
            list.sort(function (a, b) {
                return sortByProps(a, b, propOrders);
            });
        };
        //递归排序
        var recursionOrder = function (propOrders, list) {
            listOrder(propOrders, list);
            list.forEach(function (node) {
                if (node.nodes.length > 0) {
                    recursionOrder(propOrders, node.nodes);
                }
            });
        };

        var countModel = function (list) {
            list.forEach(function (node) {
                if (node.isCategory) {
                    node.quantity = recursionOrderCount(node);
                }
                if (node.nodes.length > 0) {
                    countModel(node.nodes);
                }
            });
        };
        //排序、统计
        var refreshModelTreeList = function () {
            recursionOrder($scope.propOrders, $scope.modelTreeList);
            countModel($scope.modelTreeList);
        };

        var recursionOrderCount = function (node) {
            var m = 0;
            node.nodes.forEach(function (row) {
                if (!row.isCategory) {
                    m++;
                } else {
                    m = m + recursionOrderCount(row);
                }
            });
            return m;
        };

        var updateSortByTree = function (list) {
            var index = 1;
            list.forEach(function (node) {
                if (node.isCategory) {
                    node.sort = index;
                    index++;
                    if (node.nodes.length > 0) {
                        updateSortByTree(node.nodes);
                    }
                }
            });
        };

        var loadModelData = function (orgId, modelId, categoryId) {
            if (!PWC.isNullOrEmpty(modelId) && !PWC.isNullOrEmpty(orgId)) {
                modelConfigurationService.getModelSingle(orgId, modelId, categoryId).success(function (data) {

                    $scope.viewEditModel.basicModel = data;
                    var serviceTypeID = "";
                    if (data.serviceTypeID != null && data.serviceTypeID != undefined) {
                        serviceTypeID = data.serviceTypeID;
                    }
                    $scope.viewEditModel.basicModel.serviceTypeID = $.map($.grep(serviceTypeID.split(","), function (x) {
                        return x.trim() != ""
                    }), function (x) {
                        return x.trim()
                    });

                    $scope.viewEditModel.detailModel.featureId = data.feature;
                    $scope.viewEditModel.detailModel.dataList = data.feature == 1 ? data.indexAnalysisDetaileList : data.entriesCheckDetailList;
                    $scope.viewEditModel.detailModel.formulaCriterion = data;

                    if ($scope.viewEditModel.basicModel.type == $scope.modelTypeEnum.organizationCommon.id) {
                        $scope.buttonShow.defaultSetBtnShow = true;
                    } else {
                        $scope.buttonShow.defaultSetBtnShow = false;
                    }
                });
            }
        };

        var loadKeyValueData = function () {
            if ($scope.queryParams.organizationID) {
                return KeyValueConfigService.getByOrgID($scope.queryParams.organizationID).then(function (res) {
                    $scope.keyValueDataSource.splice(0, $scope.keyValueDataSource.length);
                    res.data.forEach(function (e) {
                        $scope.keyValueDataSource.push(angular.copy(e));
                    });
                    return $q.when($scope.keyValueDataSource);
                }, function (response) {
                    return $q.reject($translate.instant("DataLoadingError"));
                });
            }
        };
        var loadIndustryData = function () {
            if ($scope.queryParams.organizationID) {
                return modelConfigurationService.getIndustry($scope.queryParams.organizationID).success(function (data) {
                    $scope.industryInfo = data;
                });
            }
        };
        var loadServiceList = function (orgId, modelId, isAdd) {
            //初始化加载服务类型列表数据
            modelConfigurationService.getServiceList(orgId, modelId, isAdd).success(function (serviceListData) {
                if (isAdd) {
                    $scope.createModel.serviceList.splice(0, $scope.createModel.serviceList.length);
                    for (var i = 0; i < serviceListData.length; i++) {
                        $scope.createModel.serviceList.push(serviceListData[i]);
                    }
                } else {
                    $scope.viewEditModel.serviceList.splice(0, $scope.viewEditModel.serviceList.length);
                    for (var i = 0; i < serviceListData.length; i++) {
                        $scope.viewEditModel.serviceList.push(serviceListData[i]);
                    }
                }
            });
        }

        $scope.$watch("queryParams.organizationID", function (newValue, oldValue) {
            if (newValue !== oldValue) {
                loadModelConfigurationTree();
                getStdAccountList();
                loadKeyValueData();
                loadIndustryData();
                getProject();
            }
        });

        //选择模型
        var selectModel = function (node) {
            $scope.selectNode = node;

            setButtonDisabled(false, node.modelType);
            setButtonShow(4);
            cancelEditGrid();
            $scope.queryParams.modelID = node.id;
            $scope.queryParams.parentId = node.parentId;
            loadServiceList($scope.queryParams.organizationID, node.id, false);
            loadModelData($scope.queryParams.organizationID, node.id, node.parentId);
            $scope.viewEditModel.editStatus = 1;

            $(".content-container .right-container > div").show();
        };
        var selectTreeNode = function (node) {
            $scope.haveModelTreeNode = false;
            judgeHaveModel(node.nodes);
            $scope.selectNode = node;
            $scope.queryParams.modelID = node.id;
            $scope.queryParams.parentId = node.parentId;
            $scope.selectOldName = node.title;
        };

        var editTreeNodeName = function ($event, node) {
            if (!$scope.hasMaintainModelTreePermission) {
                return;
            }
            $scope.selectNode = node;
            node.isEdit = true;
            $timeout(function () {
                $("#tree-root input:visible").focus();
            }, 100);
        };
        //模型树
        $scope.treeOptions = {
            beforeDrop: function (e) {
                $scope.selectNode = {};
                //$scope.queryParams.modelID = "";
                var nodesource = e.source.nodeScope.$modelValue;
                var oldParentId = nodesource.parentId;
                var newParent = e.dest.nodesScope.$parent.$modelValue;
                //模型移动
                if (!nodesource.isCategory) {
                    if (!newParent || !newParent.isCategory) {
                        return $q.reject();
                    }
                    if (oldParentId != newParent.id) {
                        return modelConfigurationService.updateModelConfig(nodesource.id, oldParentId, newParent.id).success(function (opResult) {
                            //保存失败时撤销移动
                            if (!opResult.result) {
                                $q.reject();
                            } else {
                                //成功后更新结点信息
                                e.source.nodeScope.$modelValue.parentId = newParent.id;
                                e.source.nodeScope.$modelValue.data.categoryID = newParent.id;
                            }
                        });
                    }
                } else {
                    var newParentId = '';
                    if (newParent) {
                        if (!newParent.isCategory) {
                            return $q.reject();
                        }
                        newParentId = newParent.id;
                    } else {
                        newParentId = $scope.queryParams.organizationID;
                    }
                    //节点移动
                    var sortCurrent = e.dest.index + 1;
                    return modelConfigurationService.updateCategoryWhenMove(nodesource.id, oldParentId, newParentId, sortCurrent).success(function (opResult) {
                        //保存失败时撤销移动
                        if (!opResult.result) {
                            $q.reject();
                        } else {
                            //成功后更新结点信息
                            e.source.nodeScope.$modelValue.parentId = newParentId;
                            e.source.nodeScope.$modelValue.data.categoryID = newParentId;
                        }
                    });
                }
            },
            dragStop: function (e) {
                updateSortByTree($scope.modelTreeList);
                refreshModelTreeList();
                loadModelData($scope.queryParams.organizationID, $scope.queryParams.modelID, "");
                $scope.viewEditModel.editStatus = 1;
                $(".content-container .right-container > div").show();

            }
        };

        var setValidation = function (isValid, info, setFocus) {
            $timeout(function () {
                var nameInstance = $(".categoryBox:visible").dxTextBox("instance");
                if (nameInstance) {
                    $(".categoryBox:visible").dxTextBox("instance").option("isValid", isValid);
                    $(".categoryBox:visible").dxTextBox("instance").option("validationError", {
                        message: info
                    })
                    if (setFocus) {
                        nameInstance.focus();
                    }
                }
            }, 100);
        }

        //失去焦点
        var saveEditName = function () {
            if (PWC.isNullOrEmpty($scope.selectNode.title)) {
                setValidation(false, "分类名称不能为空", true);
                return;
            } else {
                setValidation(true, "", false)
            }

            modelConfigurationService.uniqueCategoryName($scope.selectNode.title, $scope.queryParams.organizationID, $scope.selectNode.id).success(function (data) {
                if (data.result) {

                    var mto = {};
                    mto.modelID = $scope.selectNode.id;
                    mto.modelName = $scope.selectNode.title;
                    mto.parentId = $scope.selectNode.parentId;
                    mto.organizationID = $scope.queryParams.organizationID;
                    mto.sort = $scope.selectNode.sort;
                    modelConfigurationService.updateCategory(mto).success(function (opResult) {
                        if (opResult.result) {
                            setValidation(true, "", false);
                            //重新给节点赋值(部分缺失字段)
                            $scope.selectNode.id = opResult.data.modelID;
                            $scope.selectNode.title = opResult.data.modelName;
                            $scope.selectNode.parentId = opResult.data.parentId;
                            $scope.selectNode.organizationID = opResult.data.organizationID;
                            $scope.selectNode.sort = opResult.data.sort;
                            $scope.selectNode.isEdit = false;
                        } else {
                            SweetAlert.warning("保存失败:" + opResult.resultMsg);
                            $timeout(function () {
                                nameInstance.focus();
                            }, 100);
                        }
                    });
                } else {
                    setValidation(false, "分类名称重复,请重新输入...", true)
                }
            });

        };

        //编辑模型
        var editModel = function () {
            setButtonShow(1);
            $scope.copyModelDetail = angular.copy($scope.viewEditModel.basicModel);
            if ($scope.viewEditModel.basicModel.type == $scope.modelTypeEnum.systemCommon.id) {
                $scope.viewEditModel.editStatus = $scope.editStatusEnum.add;
                $scope.viewEditModel.basicModel.type = $scope.modelTypeEnum.organizationCommon.id;
                $scope.viewEditModel.basicModel.modelTypeName = $scope.modelTypeEnum.organizationCommon.name;
                $scope.viewEditModel.basicModel.oldType = $scope.modelTypeEnum.systemCommon.id;
            } else {
                $scope.viewEditModel.editStatus = $scope.editStatusEnum.edit;
            }
            $timeout(function () {
                var nameInstance = $("#nameValidation").dxTextBox("instance");
                if (nameInstance) {
                    nameInstance.option("isValid", true);
                    nameInstance.option("validationError", {
                        message: ''
                    });
                }
            }, 100);
        };

        $scope.validateAndSubmit = function (params) {
            var result = params.validationGroup.validate();
            if (result.isValid) {
                DevExpress.ui.notify({
                    message: "You are registered",
                    position: {
                        my: "left top",
                        at: "left top",
                        offset: 15
                    }
                }, "success", 3000);
            }
        };


        //保存
        var saveModel = function (index, modalInstance) {
            if (index === 1) {
                $scope.isSaveClick = true;
            }
            var viewContainerClass = ".right-container";
            var creatContainerClass = ".model-configuration-manage-create-model-tree-node-modal";
            var showFirstbrokenRuleMessage = function (brokenRules) {
                if (!brokenRules || brokenRules.length < 1)
                    return;

                var firstResult = brokenRules["0"];
                SweetAlert.warning(firstResult.message);
                firstResult.validator.element().focusin();
            }

            // 验证基础信息关键字段(包括 modelName 字段)
            var validationGroupName = index == 1 ? "viewEditGroup" : "createGroup";
            var fieldValidateResult = DevExpress.validationEngine.getGroupConfig(validationGroupName).validate();
            if (!fieldValidateResult.isValid) {
                showFirstbrokenRuleMessage(fieldValidateResult.brokenRules);
                return;
            }

            //等待模型名称远程验证结束,再继续后续的验证以及保存操作
            var waitRemoteValidationComplete = function () {
                if ($scope.remoteValidationCompleted) {
                    var selectorStr = (index == 1 ? viewContainerClass : creatContainerClass) + " [dx-validator='modelNameValidationRules']";
                    var modelNameValidateRst = $(selectorStr).dxValidator("instance").validate();
                    if (!modelNameValidateRst.isValid) {
                        showFirstbrokenRuleMessage([modelNameValidateRst.brokenRule]);
                        return;
                    }

                    //验证明细列表信息
                    selectorStr = (index == 1 ? viewContainerClass : creatContainerClass) + " [dx-data-grid]";
                    var dataGrid = $(selectorStr).dxDataGrid('instance');
                    dataGrid.saveEditData();
                    if (dataGrid.hasEditData()) {
                        return;
                    }
                    var dataGridValidateRst = $scope.validateDataGrid(dataGrid);
                    if (!dataGridValidateRst.isValid) {
                        SweetAlert.warning("模型明细存在不符合要求的数据," + (dataGridValidateRst.message ? dataGridValidateRst.message : "请仔细检查后提交"));
                        return;
                    }

                    $scope.isSaveClick = false;

                    if (index === 1) {
                        var model = angular.copy($scope.viewEditModel.basicModel);
                        model.serviceTypeID = model.serviceTypeID.join(",");
                        if ((model.indexAnalysisDetaileList == null || model.indexAnalysisDetaileList.length === 0) && (model.entriesCheckDetailList == null || model.entriesCheckDetailList.length === 0)) {
                            SweetAlert.warning("请添加模型明细");
                            return;
                        }
                        if ($scope.viewEditModel.editStatus === $scope.editStatusEnum.add) {
                            modelConfigurationService.addModel(model).success(function (opResult) {
                                if (opResult.result) {
                                    SweetAlert.warning("保存成功");
                                    $scope.currentEditOrAddModelCode = model.code;
                                    refreshData(opResult.data, model.type);
                                } else {
                                    SweetAlert.warning("保存失败:" + opResult.resultMsg);
                                }
                            });
                        } else if ($scope.viewEditModel.editStatus === $scope.editStatusEnum.edit) {
                            modelConfigurationService.updateModel(model).success(function (opResult) {
                                if (opResult.result) {
                                    SweetAlert.warning("保存成功");
                                    $scope.currentEditOrAddModelCode = model.code;
                                    setButtonDisabled(false, model.type);
                                    setButtonShow(4);
                                    cancelEditGrid();
                                    $scope.viewEditModel.editStatus = 1;
                                    loadModelData($scope.queryParams.organizationID, $scope.queryParams.modelID, "");
                                } else {
                                    SweetAlert.warning("保存失败:" + opResult.resultMsg);
                                }
                            });
                        }
                    }
                    if (index === 2) {
                        var model = angular.copy($scope.createModel.detailModel);
                        if (model.dataList == null || model.dataList.length === 0) {
                            SweetAlert.warning("请添加模型明细");
                            return;
                        }
                        var newModel = angular.copy($scope.createModel.basicModel);
                        newModel.orgId = $scope.queryParams.organizationID;
                        newModel.ruleType = 2;
                        newModel.isStatus = 1;
                        newModel.feature = model.featureId;
                        if (newModel.feature == 1) {
                            newModel.indexAnalysisDetaileList = model.dataList;
                            newModel.monthJudge = model.formulaCriterion.monthJudge;
                            newModel.monthExplain = model.formulaCriterion.monthExplain;
                            newModel.yearJudge = model.formulaCriterion.yearJudge;
                            newModel.yearExplain = model.formulaCriterion.yearExplain;
                        }
                        if (newModel.feature == 2) {
                            newModel.entriesCheckDetailList = model.dataList;
                        }
                        newModel.serviceTypeID = newModel.serviceTypeID.join(",");
                        modelConfigurationService.addModel(newModel).success(function (opResult) {
                            if (opResult.result) {
                                $scope.currentEditOrAddModelCode = newModel.code;
                                //loadModelConfigurationTree();
                                SweetAlert.warning("保存成功");
                                modalInstance.close();
                                refreshData(opResult.data, newModel.type);
                            } else {
                                SweetAlert.warning("保存失败:" + opResult.resultMsg);
                            }
                        });
                    }

                    return;
                }

                $timeout(waitRemoteValidationComplete, 100);
            };

            waitRemoteValidationComplete();
        };

        //取消
        var cancelEdit = function () {
            setButtonShow(4);
            $scope.viewEditModel.basicModel = angular.copy($scope.copyModelDetail);
            if ($scope.viewEditModel.basicModel.type == $scope.modelTypeEnum.organizationCommon.id) {
                $scope.buttonShow.defaultSetBtnShow = true;
            } else {
                $scope.buttonShow.defaultSetBtnShow = false;
            }
            if ($scope.viewEditModel.detailModel.featureId == $scope.modelFeatureEnum.indexAnalysis.id) {
                $scope.viewEditModel.detailModel.dataList = $scope.viewEditModel.basicModel.indexAnalysisDetaileList;
            }
            if ($scope.viewEditModel.detailModel.featureId == $scope.modelFeatureEnum.entriesCheck.id) {
                $scope.viewEditModel.detailModel.dataList = $scope.viewEditModel.basicModel.entriesCheckDetailList;
            }
            $scope.viewEditModel.detailModel.formulaCriterion = $scope.viewEditModel.basicModel;
            $scope.viewEditModel.editStatus = 1;
        };

        //另存为自定义
        var saveASModel = function () {
            setButtonShow(1);
            $scope.copyModelDetail = angular.copy($scope.viewEditModel.basicModel);
            $scope.viewEditModel.basicModel.name = "";
            $scope.viewEditModel.basicModel.id = "";
            $scope.viewEditModel.editStatus = $scope.editStatusEnum.add;
            $scope.viewEditModel.basicModel.type = $scope.modelTypeEnum.organizationCustomize.id;
            $scope.viewEditModel.basicModel.modelTypeName = $scope.modelTypeEnum.organizationCustomize.name;
            $scope.viewEditModel.basicModel.code = getNewModelCode(2, $scope.viewEditModel.basicModel.feature, $scope.viewEditModel.basicModel.type);
        };

        var cancelEditGrid = function () {
            var dataGrid = $('.model-configuration-manage-detail-info [dx-data-grid]').dxDataGrid('instance');
            if (dataGrid) {
                dataGrid.cancelEditData();
            }
        };

        $scope.selectedStandardList = [];
        $scope.standardAccountList = [];
        var getStdAccountList = function () {
            stdAccountService.getStdAccountByOrgID($scope.queryParams.organizationID).success(function (data) {
                _.each(data, function (account) {
                    account.directionName = $translate.instant(enums.standardAccountDirection[account.direction]);
                });
                $scope.standardAccountList.splice(0, $scope.standardAccountList.length);
                if (data) {
                    data.forEach(function (e) {
                        $scope.standardAccountList.push(angular.copy(e));
                    });
                }
            });
        };

        //#region 左侧导航模块操作按钮 functions 

        // 参数 nodeType 为 1 时,表示期望创建"模型分类根节点"
        // 参数 nodeType 为 2 时,表示期望创建"模型分类"
        // 参数 nodeType 为 3 时,表示期望创建"机构自定义模型"
        $scope.createModelTreeNode = function ($event, nodeType) {
            $scope.createModel = {
                basicModel: {},
                detailModel: {
                    featureId: 1,
                    dataList: [],
                    formulaCriterion: {}
                },
                editStatus: 3,
                showType: 2,
                validationGroupName: "createGroup",
                serviceList: []
            };

            switch (nodeType) {
                case 1:
                    var dic = {};
                    dic.title = ""
                    dic.isCategory = true;
                    dic.isEdit = true;
                    dic.isAdd = true;
                    dic.nodes = [];
                    $scope.modelTreeList.push(dic);
                    $timeout(function () {
                        $("#tree-root input:visible").focus();
                    }, 100);
                    break;
                case 2:
                    if ($scope.selectNode && $scope.selectNode.isCategory) {
                        var dic = {};
                        dic.title = ""
                        dic.parentId = $scope.selectNode.id;
                        dic.parentNode = $scope.selectNode;
                        dic.isCategory = true;
                        dic.isEdit = true;
                        dic.isAdd = true;
                        dic.nodes = [];
                        $scope.selectNode.nodes.push(dic);
                        $timeout(function () {
                            $("#tree-root input:visible").focus();
                        }, 100);
                    }
                    else
                        SweetAlert.warning("请选择模型需要添加分类的模型分类");
                    break;
                case 3:
                    if ($scope.selectNode && $scope.selectNode.isCategory) {
                        $scope.createModel.detailModel.featureId = 1;
                        $scope.createModel.basicModel.type = $scope.modelTypeEnum.organizationCustomize.id;
                        $scope.createModel.basicModel.modelTypeName = $scope.modelTypeEnum.organizationCustomize.name;
                        $scope.createModel.basicModel.industyId = $scope.industryInfo.id;
                        $scope.createModel.basicModel.industryName = $scope.industryInfo.name;
                        $scope.createModel.basicModel.organizationName = $scope.queryParams.organizationName;
                        $scope.createModel.basicModel.organizationID = $scope.queryParams.organizationID;
                        $scope.createModel.basicModel.code = getNewModelCode(2, $scope.createModel.detailModel.featureId, $scope.createModel.basicModel.type);
                        $scope.createModel.basicModel.isStatus = 1;
                        $scope.createModel.basicModel.isFilterShow = true;
                        $scope.createModel.basicModel.categoryID = $scope.selectNode.id;
                        //$scope.createModel.basicModel.categoryString = $scope.modelFeatureEnum.indexAnalysis.name;
                        // editStatus 由 1 变为 3 的目的是:使 dataGrid 变为可编辑状态
                        $scope.createModel.editStatus = 1;
                        modelConfigurationService.getModelCategoryString($scope.queryParams.organizationID, $scope.selectNode.id).success(function (cateString) {
                            $scope.createModel.basicModel.categoryString = cateString;
                        });
                        loadServiceList($scope.queryParams.organizationID, "", true);
                        var modalInstance = $uibModal.open({
                            animation: false,
                            ariaLabelledBy: 'modal-title',
                            ariaDescribedBy: 'modal-body',
                            templateUrl: 'model-config-manage-create-model-tree-node-modal.html',
                            scope: $scope,
                            windowClass: "model-configuration-manage-create-model-tree-node-modal"
                        });
                        $scope.creatModelSave = function () {
                            saveModel(2, modalInstance);
                        };
                        $scope.saveClose = function () {
                            modalInstance.dismiss('cancel');
                        };
                        $timeout(function () {
                            $scope.createModel.editStatus = 3;
                        }, 500);
                    }
                    else
                        SweetAlert.warning("请选择模型需要添加模型的模型分类");
                    break;
                default:
                    SweetAlert.warning("不受支持的 nodeType 枚举值");
                    return;
            }

        };

        var removeNode = function (id, list) {
            var index = 0;
            list.forEach(function (row) {
                if (row.id == id) {
                    list.splice(index, 1);
                }
                if (row.nodes.length > 0) {
                    removeNode(id, row.nodes);
                }
                index++;
            });
        };

        //删除模型分类或模型
        $scope.deleteModelTreeNode = function () {
            if ($scope.selectNode) {
                if ($scope.selectNode.isCategory) {
                    if (!$scope.hasMaintainModelTreePermission) {
                        SweetAlert.warning("你没有权限删除该分类");
                        return;
                    }
                    //判断该分类下是否有模型  
                    $scope.tmpValue = false;
                    judgeHaveModel($scope.selectNode.nodes);
                    if ($scope.tmpValue) {
                        SweetAlert.warning("该分类或子分类下含有模型，无法删除");
                    } else {
                        modelConfigurationService.deleteCategory($scope.selectNode.id, $scope.queryParams.organizationID).success(function (opResult) {
                            if (opResult.result) {
                                removeNode($scope.selectNode.id, $scope.modelTreeList);
                                updateSortByTree($scope.modelTreeList);
                                refreshModelTreeList();
                            }
                        });

                    }
                } else {
                    if (!$scope.hasMaintainCustomPermission) {
                        SweetAlert.warning("你没有权限删除该模型");
                        return;
                    }
                    if ($scope.selectNode.modelType == $scope.modelTypeEnum.organizationCustomize.id) {
                        SweetAlert.swal({
                            title: "你确定要删除该机构自定义模型吗？",
                            type: "warning",
                            showCancelButton: true,
                            confirmButtonColor: "#DD6B55",
                            confirmButtonText: $translate.instant('Confirm'),
                            cancelButtonText: $translate.instant('Cancel'),
                            closeOnConfirm: true,
                            closeOnCancel: true
                        },
                            function (isConfirm) {
                                if (isConfirm) {
                                    modelConfigurationService.deleteModel($scope.selectNode.id).success(function (opResult) {
                                        if (opResult.result) {
                                            removeNode($scope.selectNode.id, $scope.modelTreeList);
                                            refreshModelTreeList();
                                        }
                                    });
                                }
                            });

                    } else {
                        SweetAlert.warning("非机构自定义模型，无法删除");
                    }
                }

            }
        };
        $scope.tmpValue = false;
        var judgeHaveModel = function (nodes) {
            if (_.find(nodes, function (num) {
                return num.isCategory == false
            })) {
                $scope.tmpValue = true;
                $scope.haveModelTreeNode = true;
            } else {
                nodes.forEach(function (mc) {
                    judgeHaveModel(mc.nodes);
                });
            }
        };


        //#endregion

        //#region 右侧基本信息按钮 functions


        var loadModelDefaultData = function (orgId, modelId, categoryId) {
            if (!PWC.isNullOrEmpty(modelId) && !PWC.isNullOrEmpty(orgId)) {
                modelConfigurationService.getModelDefaultSetting(orgId, modelId, categoryId).success(function (data) {

                    $scope.seeDefaultModel.basicModel = data;

                    $scope.seeDefaultModel.detailModel.featureId = data.feature;
                    $scope.seeDefaultModel.detailModel.dataList = data.feature == 1 ? data.indexAnalysisDetaileList : data.entriesCheckDetailList;
                    $scope.seeDefaultModel.detailModel.formulaCriterion = data;
                });
            }
        };
        $scope.seeDefault = function () {
            loadModelDefaultData($scope.queryParams.organizationID, $scope.queryParams.modelID, $scope.queryParams.parentId);

            var modalInstance = $uibModal.open({
                animation: false,
                ariaLabelledBy: 'modal-title',
                ariaDescribedBy: 'modal-body',
                templateUrl: 'model-config-manage-defalut-settings-modal.html',
                scope: $scope,
                windowClass: "model-configuration-manage-defalut-settings-modal"
            });
            $scope.resetModel = function () {
                modelConfigurationService.resetOrgModelSetting($scope.queryParams.modelID, $scope.queryParams.organizationID).success(function (data) {
                    if (data.result) {
                        modalInstance.close();
                        $scope.currentEditOrAddModelCode = $scope.seeDefaultModel.basicModel.code;
                        refreshData(data.data, 1)
                    }
                });
            };
            $scope.resetClose = function () {
                modalInstance.dismiss('cancel');
            };
        };

        var refreshData = function (modelId, modelType) {
            $scope.viewEditModel.editStatus = 1;
            loadModelConfigurationTree();
            $scope.queryParams.modelID = modelId;
            setButtonDisabled(false, modelType);
            setButtonShow(4);
            cancelEditGrid();
            loadModelData($scope.queryParams.organizationID, $scope.queryParams.modelID, "");
        };

        //#endregion
        $scope.textBox = {
            modelName: {
                bindingOptions: {
                    value: "viewEditModel.basicModel.name",
                    visible: "viewEditModel.editStatus != 1",
                    readOnly: "viewEditModel.basicModel.type!=3"
                }
            }
        };
        $scope.loadNodes = function (node) {
            return {
                maxLength: 10,
                onFocusIn: function () {
                    selectTreeNode(node);
                },
                onFocusOut: function (params) {
                    if (!(params.component.option("value") || "").trim()) {
                        var nodeScope = params.element.parents("[ui-tree-node]:first").data().$uiTreeNodeController.scope;
                        var modelValue = nodeScope.$modelValue;
                        //取消添加:将当前节点从树节点中移除,并将当前节点的父节点设置为 选中节点
                        if (modelValue.isAdd) {
                            selectTreeNode(nodeScope.$parentNodeScope.$modelValue);
                            nodeScope.remove();
                            return;
                        }
                        //取消编辑:将测试对显示字段的编辑
                        if (modelValue.isEdit) {
                            modelValue.title = modelValue.data.modelName;
                            modelValue.isEdit = false;
                            return;
                        }
                    }

                    saveEditName();
                },
                bindingOptions: {
                    value: 'node.title',
                    visible: 'node.isEdit'
                }
            };
        };

        $scope.remoteValidationCompleted = false;
        $scope.remoteValidateModelName = function (params, modelName, modelId, organizationId, flag) {
            $scope.remoteValidationCompleted = false;
            var remoteRst = modelConfigurationService.uniqueModelName(modelName, modelId, organizationId, flag);
            remoteRst.success(function (data) {
                params.rule.isValid = data.result;
                params.validator.validate();
                $scope.remoteValidationCompleted = true;
            });
        };

        $scope.modelNameValidationRules = {
            validationRules: [
                {
                    type: "required",
                    message: "模型名称不能为空"
                },
                {
                    type: 'stringLength',
                    max: 50,
                    message: '请输入不超过 50 字的模型名称'
                },
                {
                    type: "custom",
                    validationCallback: function (params) {
                        if ($scope.viewEditModel.editStatus == 1)
                            return true;

                        var modelName = $scope.viewEditModel.basicModel.name;
                        var modelId = $scope.viewEditModel.basicModel.id;
                        var organizationId = $scope.queryParams.organizationID;
                        var flag = $scope.viewEditModel.editStatus === 3;
                        $scope.remoteValidateModelName(params, modelName, modelId, organizationId, flag);
                        return true;
                    },
                    message: "名称重复,请重新输入..."
                }
            ],
            validationGroup: "viewEditGroup"
        };

        $("#saveButton").dxButton({
            text: '保存',
            onClick: register
        });

        $scope.saveButtonOptions = {
            text: '保存',
            type: 'normal',
            validationGroup: "viewEditGroup",
            onClick: function (e) {
                var result = e.validationGroup.validate();
                if (result.isValid) {
                    DevExpress.ui.notify({
                        message: "You are logged in！",
                        position: {
                            my: "left top",
                            at: "left top"
                        }
                    }, "success", 3000);
                }
            }
        };
        var register = function (params) {
            var result = params.validationGroup.validate();
            if (result.isValid) {
                DevExpress.ui.notify({
                    message: "You are logged in！",
                    position: {
                        my: "left top",
                        at: "left top"
                    }
                }, "success", 3000);
            }
        };
        //根据 dataGrid 的 Option 设置验证数据源的有效性,并返回验证结果,isValid:true 验证通过,isValid:false 验证无效,message 验证信息
        //数据源为空则当做验证通过
        $scope.validateDataGrid = function (gridInstance) {
            var data = gridInstance.option().dataSource;
            var columns = gridInstance.option().columns;
            var validateRst = {
                isValid: true,
                message: ""
            };

            if (!data || data.length < 1)
                return validateRst;

            for (var i = 0; i < columns.length; i++) {
                var col = columns[i];
                // isMainValueShow 需要单独验证,因为在 dataGrid 中无法进行验证
                if (col.cellTemplate && col.cellTemplate === "isMainValueTemplate") {
                    //没有主要值的校验
                    var haveMainValue = false;
                    for (var j = 0; !haveMainValue && j < data.length; j++) {
                        haveMainValue = data[j].isMainValueShow;
                    }
                    if (!haveMainValue) {
                        validateRst.isValid = false;
                        validateRst.message = "请设置模型的主要值";
                        var colIndex = _.findIndex(gridInstance.getVisibleColumns(), function (item) {
                            return item.cellTemplate === col.cellTemplate;
                        });
                        for (var j = 0; j < data.length; j++) {
                            var key = gridInstance.keyOf(data[j]);
                            var rowIndex = gridInstance.getRowIndexByKey(key);
                            var cell = gridInstance.getCellElement(rowIndex, colIndex);
                            cell.addClass("dx-validator dx-editor-cell dx-datagrid-invalid dx-focused");
                            cell.find("> div").addClass("dx-highlight-outline dx-texteditor-input");
                        }
                    }
                }

                var rules = col.validationRules;
                if (!rules || rules.length < 1) {
                    continue;
                }

                var haveRepeatIndexName = null;
                for (var j = 0; j < data.length; j++) {
                    var value = data[j][col.dataField];
                    //特殊逻辑验证
                    if ((col.dataField === "acctDirection" || col.dataField === "standardCode" || col.dataField === "entriesComparison" || col.dataField === "thresholdValue") && data[j]["acctJudge"] === "N/A") {
                        if (value) {
                            console.log("代码异常,字段" + col.dataField + " :应该为 null,实际值为: " + value);
                        }
                        continue;
                    }

                    rules = angular.copy(rules);
                    _.each(rules, function (item) {
                        item.value = null;
                        item.isValid = false;
                    });
                    var rst = DevExpress.validationEngine.validate(value, rules, col.dataField);
                    if (!rst.isValid) {
                        validateRst.isValid = false;
                        if (col.dataField === "indexName" && rst.brokenRule.type === "custom" && (!haveRepeatIndexName || !haveRepeatIndexName.isValid)) {
                            haveRepeatIndexName = {
                                isValid: rst.brokenRule.isValid,
                                message: rst.brokenRule.message
                            };
                        }
                        var key = gridInstance.keyOf(data[j]);
                        var rowIndex = gridInstance.getRowIndexByKey(key);
                        var cell = gridInstance.getCellElement(rowIndex, col.dataField);
                        cell.addClass("dx-validator dx-editor-cell dx-datagrid-invalid dx-focused");
                        var divContainer = cell.find("> div");
                        if (divContainer.length < 1) {
                            cell.wrapInner("<div></div>");
                            divContainer = cell.find("> div");
                        }

                        divContainer.addClass("dx-highlight-outline");
                        if (cell.find(".dx-texteditor-input").length < 1) {
                            if (cell.find("input:first") > 1) {
                                cell.find("input:first").addClass("dx-texteditor-input");
                            } else {
                                divContainer.addClass("dx-texteditor-input");
                            }
                        }
                    }
                }
                if (haveRepeatIndexName && !haveRepeatIndexName.isValid) {
                    validateRst.message = haveRepeatIndexName.message;
                }
            }
            return validateRst;
        };

        //模型预览
        $scope.projectYearList = [];
        $scope.previewYear = [];
        $scope.enterpriseList = [];
        $scope.enterpriseSelectShow = false;
        $scope.popoverYearSelect = 520;
        $scope.selectProjectDto = {};
        $scope.selectedYear = {};
        $scope.selectedEnterprise = {};
        var getProject = function () {
            if (!PWC.isNullOrEmpty($scope.queryParams.organizationID)) {
                modelConfigurationService.getProject($scope.queryParams.organizationID).success(function (data) {
                    $scope.projectYearList = data;
                    $scope.projectYearList.forEach(function (row) {
                        if (!_.find($scope.previewYear, function (num) {
                            return num === row.year
                        })) {
                            $scope.previewYear.push(row.year);
                        }
                    })
                    $scope.selectedYear = $scope.previewYear[0];
                    $scope.enterpriseList = $scope.projectYearList.filter(function (element) {
                        return element.year === $scope.selectedYear;
                    });
                    $scope.selectedEnterprise = $scope.enterpriseList[0];
                });
            }
        };

        $scope.resultPreview = function (isAdd) {
            $scope.isAddPreview = isAdd;
            var model = $scope.isAddPreview ? angular.copy($scope.createModel.basicModel) : angular.copy($scope.viewEditModel.basicModel)

            if ((!model.indexAnalysisDetaileList && !model.entriesCheckDetailList) ||
                (model.indexAnalysisDetaileList && model.indexAnalysisDetaileList.length === 0) ||
                (model.entriesCheckDetailList && model.entriesCheckDetailList.length === 0)
            ) {
                SweetAlert.warning("请添加模型明细后再预览");
                return;
            } else {

                if (PWC.isNullOrEmpty($scope.queryParams.modelID) && !isAdd) {
                    return;
                }
                if ($scope.projectYearList.length == 0) {
                    SweetAlert.warning("界面卡片必须显示“进行中”，才能查看预览结果");
                    return;
                }
                getPopoverInstance().show();
            }
        };

        $scope.yearSelectOptions = {
            //target: "#previewModel",
            //showEvent: "dxclick",
            position: "bottom",
            //shading: true,
            //shadingColor: "rgba(0, 0, 0, 0.5)",
            visible: false,
            bindingOptions: {
                width: "popoverYearSelect",
                target: "isAddPreview?'#addPreviewModel':'#previewModel'"
            }
        };
        $scope.projectYearSelect = {
            width: 80,
            bindingOptions: {
                dataSource: "previewYear",
                value: 'selectedYear'
            },
            onSelectionChanged: function (e) {
                $scope.enterpriseList = $scope.projectYearList.filter(function (element) {
                    return element.year === e.selectedItem;
                });
                if ($scope.enterpriseList.length > 1) {
                    $scope.enterpriseSelectShow = true;
                    $scope.popoverYearSelect = 650;
                } else {
                    //$scope.enterpriseSelectShow = false;
                    //$scope.popoverYearSelect = 520;
                    $scope.enterpriseSelectShow = true;
                    $scope.popoverYearSelect = 650;
                }
                $scope.selectProjectDto = e.selectedItem;
                $scope.selectedEnterprise = $scope.enterpriseList[0];
            }
        };
        $scope.enterpriseSetSelect = {
            width: 130,
            displayExpr: 'enterpriseAccountSetName',
            bindingOptions: {
                dataSource: "enterpriseList",
                visible: "enterpriseSelectShow",
                value: 'selectedEnterprise'
            },
            onSelectionChanged: function (e) {
                $scope.selectProjectDto = e.selectedItem;
            }
        };
        var getPopoverInstance = function () {
            var popover = $("#popoverYearSelect").dxPopover('instance');
            return popover;
        };
        $scope.yearSelectClose = function () {
            getPopoverInstance().hide();
        };
        $scope.indexGridColumns = [];
        $scope.indexResultDataList = [];
        $scope.indexResultData = {};
        $scope.enterCheckData = {};
        $scope.enterCheckDetailDataList = [];
        $scope.startPreview = function () {
            var model = $scope.isAddPreview ? angular.copy($scope.createModel.basicModel) : angular.copy($scope.viewEditModel.basicModel)
            if ($scope.selectProjectDto && model) {
                if ($scope.selectProjectDto.haveMappedAccountByPeriod === false) {
                    SweetAlert.warning("请完成科目对应操作，再查看预览结果。");
                    return;
                }
                ;
                if (model.serviceTypeID) {
                    model.serviceTypeID = model.serviceTypeID.join(",");
                }
                //用Array解决参数传递的问题
                var modelList = [];
                modelList.push(model);
                var projectList = [];
                projectList.push($scope.selectProjectDto);
                modelConfigurationService.getModelPreviewResult(modelList, projectList).success(function (data) {
                    if (data.result) {
                        var modalInstance = null;
                        if (model.feature === 1) {
                            //基础信息
                            $scope.indexResultData.code = data.data.code;
                            $scope.indexResultData.name = data.data.name;
                            $scope.indexResultData.description = data.data.description;
                            $scope.indexResultData.monthExplain = data.data.monthExplain;
                            $scope.indexResultData.yearExplain = data.data.yearExplain;
                            $scope.indexResultData.year = $scope.selectProjectDto.year;
                            $scope.indexResultData.yearException = false;
                            $scope.indexResultData.mainValue = '';
                            $scope.indexGridColumns = [];
                            var isShowMonthJudge = false;
                            if (model.monthJudge && model.monthExplain) {
                                isShowMonthJudge = true;
                            }
                            var maxSqence = _.max(data.data.indexAnalysisDetailResults, function (element) {
                                return element.sequence;
                            });
                            $scope.indexResultDataList = [];
                            var searchData = data.data.indexAnalysisDetailResults;
                            $scope.indexGridColumns.push({
                                dataField: "period",
                                caption: "期间",
                                width: "4em",
                                alignment: "center"
                            });
                            for (var i = 1; i <= 13; i++) {
                                var rowObj = {};
                                if (i === 13) {
                                    rowObj.period = "年度";
                                } else {
                                    rowObj.period = i;
                                }
                                for (var m = 1; m <= maxSqence.sequence; m++) {
                                    var cellData = _.first(_.filter(searchData, function (element) {
                                        return (element.period === i && element.sequence === m);
                                    }));
                                    if (cellData) {
                                        if (cellData.displayType === 1 || cellData.displayType === 2) {
                                            rowObj["colmun" + m] = PWC.isNullOrEmpty(cellData.indexResult) ? 0 : parseFloat(cellData.indexResult);
                                        } else if (cellData.displayType === 3) {
                                            rowObj["colmun" + m] = cellData.indexResult === "1" ? '是' : '否';
                                        } else {
                                            rowObj["colmun" + m] = cellData.indexResult;
                                        }
                                        //是否添加月度判断
                                        if (isShowMonthJudge === true && cellData.isMainValue === 1) {
                                            if (i !== 13)
                                                rowObj["monthJudge"] = cellData.isMonExp === 1 ? true : false;
                                            else
                                                rowObj["monthJudge"] = cellData.isYearExp === 1 ? true : false;
                                        }

                                        //动态添加列
                                        if (i === 1) {
                                            var colmun = {};
                                            colmun.dataField = "colmun" + m;
                                            colmun.caption = cellData.indexName;
                                            colmun.width = colmun.caption.length + "em";
                                            if (cellData.displayType === 1) {
                                                colmun.format = {
                                                    type: "fixedPoint",
                                                    precision: 2
                                                };
                                            }
                                            if (cellData.displayType === 2) {
                                                colmun.format = {
                                                    type: "percent",
                                                    precision: 2
                                                };
                                            }
                                            if (cellData.displayType === 3) {
                                                colmun.dataType = "string";

                                            }
                                            if (cellData.isMainValue === 1)
                                                $scope.indexResultData.mainColmunName = cellData.indexName;
                                            $scope.indexGridColumns.push(colmun)
                                        }
                                        //设置基础信息
                                        if (i === 13) {
                                            if (cellData.isMainValue === 1) {
                                                if (cellData.displayType === 1 || cellData.displayType === 2) {
                                                    if (cellData.displayType === 1) {
                                                        $scope.indexResultData.mainValue = PWC.isNullOrEmpty(cellData.indexResult) ?
                                                            PWC.round(0, 2) : PWC.round(cellData.indexResult, 2);
                                                    } else {
                                                        $scope.indexResultData.mainValue = PWC.isNullOrEmpty(cellData.indexResult) ?
                                                            PWC.round(0, 2) + "%" : PWC.round(cellData.indexResult * 100, 2) + "%";
                                                    }

                                                } else {
                                                    $scope.indexResultData.mainValue = cellData.indexResult;
                                                }
                                                $scope.indexResultData.yearException = cellData.isYearExp === 1 ? true : false;
                                            }
                                        }
                                    }
                                }
                                $scope.indexResultDataList.push(rowObj);
                            }
                            //是否添加月度判断
                            if (isShowMonthJudge === true) {
                                var colmunJudge = {};
                                colmunJudge.dataField = "monthJudge";
                                colmunJudge.caption = "异常判断";
                                colmunJudge.dataType = "boolean";
                                colmunJudge.alignment = 'left';
                                colmunJudge.width = "6em";
                                $scope.indexGridColumns.push(colmunJudge)
                            }

                            var visibleFlag = _.reduce($scope.indexGridColumns, function (sum, item) {
                                return sum + parseInt(item.width);
                            }, 0);

                            $scope.indexGridColumns.push({
                                caption: "TRTest",
                                width: "100%",
                                alignment: "center",
                                visible: visibleFlag > 40,
                                visibleIndex: -1
                            });

                            getPopoverInstance().hide();
                            modalInstance = $uibModal.open({
                                animation: false,
                                ariaLabelledBy: 'modal-title',
                                ariaDescribedBy: 'modal-body',
                                templateUrl: 'model-config-manage-result-index-analysis-preview-modal.html',
                                scope: $scope,
                                windowClass: "model-configuration-manage-result-preview-modal index-analysis"
                            });
                        }

                        if (model.feature === 2) {
                            $scope.enterCheckData.code = data.data.code;
                            $scope.enterCheckData.name = data.data.name;
                            $scope.enterCheckData.description = data.data.description;
                            $scope.enterCheckData.year = $scope.selectProjectDto.year;
                            $scope.enterCheckData.doubts = data.data.voucherDoubts;
                            $scope.enterCheckData.debitRelevantAmt = PWC.round(data.data.debitRelevantAmt, 2);
                            $scope.enterCheckData.creditRelevantAmt = PWC.round(data.data.creditRelevantAmt, 2);
                            $scope.enterCheckData.voucherNum = data.data.voucherNum;
                            $scope.enterCheckData.entriesNum = data.data.entriesNum;
                            $scope.enterCheckDetailDataList = data.data.entriesCheckDetailResults;
                            listOrder({
                                "period": "asc",
                                "date": "asc",
                                "group": "asc",
                                "vid": "asc"
                            }, $scope.enterCheckDetailDataList);
                            var index = 1;
                            $scope.enterCheckDetailDataList.forEach(function (row) {
                                row.indexShow = index;
                                //row.debit = PWC.round(row.debit, 2);
                                //row.credit = PWC.round(row.credit, 2);
                                index++;
                            });
                            getPopoverInstance().hide();
                            modalInstance = $uibModal.open({
                                animation: false,
                                ariaLabelledBy: 'modal-title',
                                ariaDescribedBy: 'modal-body',
                                templateUrl: 'model-config-manage-result-entries-check-preview-modal.html',
                                scope: $scope,
                                windowClass: "model-configuration-manage-result-preview-modal entries-check"
                            });
                        }

                        $scope.resultPreviewClose = function () {
                            if (modalInstance == null)
                                return;

                            modalInstance.dismiss('cancel');
                        };
                    } else {
                        SweetAlert.warning("预览计算失败");
                    }
                });

            }
        };

        var getModelConfigurationPermission = function () {

            var list = [];

            var modelManageTemp = constant.adminPermission.systemConfiguration.modelConfiguration;
            list.push(modelManageTemp.queryCode);
            list.push(modelManageTemp.editCode);
            list.push(modelManageTemp.maintainCustomCode);
            list.push(modelManageTemp.maintainModelTreeCode);

            $scope.$root.checkUserPermissionList(list).success(function (data) {
                $scope.hasQueryPermission = data[modelManageTemp.queryCode];
                $scope.hasEditPermission = data[modelManageTemp.editCode];
                $scope.hasMaintainCustomPermission = data[modelManageTemp.maintainCustomCode];
                $scope.hasMaintainModelTreePermission = data[modelManageTemp.maintainModelTreeCode];
            });
        };
        $scope.showOperateLogPop = function () {
            /**
             * 重新设置日志列宽
             */
            var grid = $(".operate-log-wrapper #operateLogGridOptions").dxDataGrid("instance")
            grid.beginUpdate();
            grid.columnOption("originalState", "visible", false);
            grid.columnOption("updateState", "visible", false);
            grid.columnOption("comment", "visible", false);
            grid.columnOption("id", "width", "5em");
            grid.columnOption("createTime", "width", "12em");
            grid.columnOption("operationObject", "width", "12em");
            grid.columnOption("action", "width", "5em");
            grid.columnOption("operationContent", "width", "100%");
            grid.columnOption("operationUser", "width", "10em");
            grid.endUpdate();

            $scope.isShowLog = true;
        };
        (function initializa() {
            $log.debug("ModelConfigurationController.ctor()...");
            $scope.hasQueryPermission = false;
            $scope.hasEditPermission = false;
            $scope.hasMaintainCustomPermission = false;
            $scope.hasMaintainModelTreePermission = false;

            getModelConfigurationPermission();

            $scope.loadModelConfigurationTree = loadModelConfigurationTree();
            $scope.selectModel = selectModel;
            $scope.selectTreeNode = selectTreeNode;
            $scope.editTreeNodeName = editTreeNodeName;
            $scope.editModel = editModel;
            $scope.saveModel = saveModel;
            $scope.saveEditName = saveEditName;
            $scope.cancelEdit = cancelEdit;
            $scope.saveASModel = saveASModel;
            setButtonDisabled(true);
            orgService.getOrgList(0).success(function (data) {
                if ($scope.selectNoOption) {
                    data.splice(0, 0, NoOptions);
                }
                //关键数据配置跳转设置默认选中机构
                if ($scope.organizationId && _.find(data, function (org) {
                    return org.id == $scope.organizationId
                })) {
                    var orgSelected = _.find(data, function (org) {
                        return org.id == $scope.organizationId
                    });
                    $scope.queryParams.organizationName = orgSelected.name;
                    $scope.queryParams.organizationID = orgSelected.id;
                }
                else {
                    $scope.queryParams.organizationName = data[0].name;
                    $scope.queryParams.organizationID = data[0].id;
                }
            });
            //关键数据配置跳转设置默认选中模型
            if ($scope.selectedModelCode) {
                $scope.currentEditOrAddModelCode = $scope.selectedModelCode;
            }
            loadModelConfigurationTree();
            loadIndustryData();
            getStdAccountList();
            //loadStdAccountDatagrid();
            loadKeyValueData();
            getProject();

        })();

        $scope.analysisResultPreviewDataGridOptions = {
            bindingOptions: {
                dataSource: "indexResultDataList",
                columns: "indexGridColumns"
            },
            paging: {
                enabled: false
            },
            selection: {
                mode: "single"
            },
            allowColumnResizing: true,
            sorting: false,
            showBorders: true,
            showRowLines: true,
            onCellPrepared: function (e) {
                if (e.rowType === "data") {
                    if (e.column.dataType === "boolean") {
                        e.cellElement.empty();
                        if (e.value === true)
                            e.cellElement.append($('<span class="horizontal-center"><i class="fa fa-exclamation-circle"></i>异常</span>'));
                    }
                }
            },
            onRowPrepared: function (e) {
                if (e.rowType != 'header' && e.data && e.data.monthJudge && e.data.monthJudge === true)
                    e.rowElement.addClass('entriesHighLight');
            },
            onInitialized: function (e) {
                //var year = "年度";
                //var data = $scope.analysisResultPreviewData;
                //var row = $.grep(data, function (n, i) { return n.month == year });
                //if (row.length > 0)
                //    return;

                //row = {
                //    month: year,
                //    avgValue: data.map(function (x) { return x.avgValue; }).reduce(function (acc, val) { return acc + val; }, 0),
                //    deprecation: data.map(function (x) { return x.deprecation; }).reduce(function (acc, val) { return acc + val; }, 0),
                //    compositePercent: data.map(function (x) { return x.compositePercent; }).reduce(function (acc, val) { return acc + val; }, 0),
                //    exception: data.map(function (x) { return x.exception; }).reduce(function (acc, val) { return acc || val; }, false)
                //}
                //data.push(row);
                //e.component.refresh();
                dxDataGridService.registerAutoSize(e.component);
            }
        };

        $scope.checkResultPreviewDataGridOptions = {
            columns: [
                {
                    dataField: "indexShow",
                    dataType: 'string',
                    caption: "序号",
                    width: 45,
                    alignment: "center"
                },
                {
                    dataField: "period",
                    dataType: 'string',
                    caption: "期间",
                    width: 45,
                    alignment: "center"
                },
                {
                    dataField: "date",
                    dataType: 'date',
                    caption: "凭证日期",
                    width: 90,
                    alignment: "center",
                    format: "yyyy-MM-dd"
                },
                {
                    dataField: "group",
                    dataType: 'string',
                    caption: "凭证类型",
                    width: 90,
                    alignment: "left"
                },
                {
                    dataField: "vid",
                    dataType: 'string',
                    caption: "凭证编号",
                    width: "33%",
                    alignment: "left"
                },
                {
                    dataField: "summary",
                    dataType: 'string',
                    caption: "摘要",
                    width: "34%",
                    alignment: "left"
                },
                {
                    dataField: "acctCode",
                    dataType: 'string',
                    caption: "会计科目",
                    width: "33%",
                    alignment: "left"
                },
                {
                    dataField: "debit",
                    dataType: 'decimal',
                    caption: "借方发生额",
                    format: {
                        type: 'fixedPoint',
                        precision: 2
                    },
                    width: 85,
                    alignment: "right"
                },
                {
                    dataField: "credit",
                    dataType: 'decimal',
                    caption: "贷方发生额",
                    format: {
                        type: 'fixedPoint',
                        precision: 2
                    },
                    width: 85,
                    alignment: "right"
                }
            ],
            bindingOptions: {
                dataSource: "enterCheckDetailDataList"
            },
            paging: {
                enabled: false
            },
            selection: {
                mode: "single"
            },
            scrolling: {
                mode: "virtual",
                preloadEnabled: true
            },
            summary: {
                totalItems: [
                    {
                        column: "debit",
                        displayFormat: "{0}",
                        valueFormat: {
                            type: 'fixedPoint',
                            precision: 2
                        },
                        summaryType: "sum"
                    }, {
                        column: "credit",
                        displayFormat: "{0}",
                        valueFormat: {
                            type: 'fixedPoint',
                            precision: 2
                        },
                        summaryType: "sum"
                    }, {
                        showInColumn: "acctCode",
                        displayFormat: "合计"
                    }
                ]
            },
            allowColumnResizing: true,
            sorting: false,
            showBorders: true,
            showRowLines: true,
            onInitialized: function (e) {
                dxDataGridService.registerAutoSize(e.component);
            },
            onRowPrepared: function (e) {
                if (e.rowType != 'header' && e.data && e.data.isDoubt === 1)
                    e.rowElement.addClass('entriesHighLight');
            }
        };
    }
]);

systemConfigurationModule.directive('modelConfiguration', ['$log',
    function ($log) {
        'use strict';
        $log.debug('modelConfiguration.ctor()...');

        return {
            restrict: 'E',
            templateUrl: '/app/admin/systemConfiguration/modelConfiguration/model-configuration.html' + '?_=' + Math.random(),
            replace: true,
            scope: {
                organizationName: '@',
                organizationId: '@',
                selectedModelCode: '@'
            },
            controller: 'ModelConfigurationController',
            link: function (scope, element) {
                $(".multi-org-container").on("click", function (e) {
                    e.stopPropagation();
                });

                var ClearSelectedTreeNode = function () {
                    ClearSelectedTreeNodeRecursion(scope.OrgMultiData);
                    scope.selectedCopyOrgIDList = [];
                    scope.checkedSet = {};
                };


                //递归清除选中的节点
                var ClearSelectedTreeNodeRecursion = function (tree) {
                    tree.forEach(function (o) {
                        o.selected = false;
                        if (o.children) {
                            ClearSelectedTreeNodeRecursion(o.children);
                        }
                    });
                };

                //显示复制对应关系下的组织机构多选列表
                scope.displayMultiOrg = function () {
                    ClearSelectedTreeNode();
                    if ($('.multi-org-container').is(":hidden")) {
                        $(".multi-org-container").show();
                    } else {
                        $(".multi-org-container").hide();
                    }

                    $(document).one("click", function () {
                        $(".multi-org-container").hide();
                    });
                    event.stopPropagation();
                };

                scope.closeMultiOrg = function () {
                    $(".multi-org-container").hide();
                };

                //企业科目列表和标准科目列表切换
                scope.SwitchAccount = function () {
                    $('.subject-corresponding-container').toggle();
                    $('.std-subject-container').toggle();
                };

                $(".filter-select").on("click", function (e) {
                    e.stopPropagation();
                });

                scope.showFilter = function () {
                    if ($('.filter-select').is(":hidden")) {
                        $(".filter-select").show();
                    } else {
                        $(".filter-select").hide();
                    }

                    $(document).one("click", function () {
                        $(".filter-select").hide();
                    });
                    event.stopPropagation();
                };
            }
        };
    }
]);

systemConfigurationModule.directive('modelConfigurationBasicInfo', [
    '$log', 'modelConfigurationService',
    function ($log, modelConfigurationService) {
        'use strict';
        return {
            restrict: "A",
            templateUrl: "model-config-manage-basic-info.html",
            replace: false,
            scope: {
                basicModel: "=",
                editStatus: "=",
                serviceList: "=",
                showType: "=",
                validationGroupName: "="
            },
            link: function ($scope, element) {

                $scope.serviceTypeOptions = {
                    displayExpr: "name",
                    valueExpr: "id",
                    showSelectionControls: true,
                    searchEnabled: true,
                    noDataText: "当前无服务可选",
                    bindingOptions: {
                        dataSource: "serviceList",
                        value: "basicModel.serviceTypeID"
                    },
                    placeholder: "请选择适用服务",
                    selectAllText: "全选"
                };

                $scope.serviceTypeValidationRules = {
                    validationRules: [
                        {
                            type: "required",
                            message: "请选择适用服务"
                        }
                    ],
                    validationGroup: $scope.validationGroupName
                };

                $scope.modelDescriptionOptions = {
                    maxLength: 500,
                    bindingOptions: {
                        value: "basicModel.description"
                    }
                };

                $scope.modelNameOptions = {
                    onFocusOut: function (e) {
                        e.component.element().dxValidator("instance").validate();
                    },
                    bindingOptions: {
                        value: "basicModel.name"
                    }
                };

                //直接使用父 scope 中对模型名称验证的配置
                $scope.modelNameValidationRules = angular.copy($scope.$parent.modelNameValidationRules);
                var customIndex = _.findIndex($scope.modelNameValidationRules.validationRules, function (item) {
                    return item.type === "custom"
                });
                $scope.modelNameValidationRules.validationRules[customIndex].validationCallback = function (params) {
                    var modelName = $scope.basicModel.name;
                    var modelId = $scope.basicModel.id;
                    var organizationId = $scope.basicModel.organizationID;
                    var flag = $scope.editStatus == 3;
                    $scope.$parent.remoteValidateModelName(params, modelName, modelId, organizationId, flag);
                    return true;
                };
                $scope.modelNameValidationRules.validationGroup = $scope.validationGroupName;
            }
        };
    }
]);

systemConfigurationModule.directive('modelConfigurationDetailInfo', [
    '$log', '$translate', '$uibModal', 'KeyValueConfigService', '$timeout', '$compile', 'SweetAlert', "dxDataGridService",
    function ($log, $translate, $uibModal, KeyValueConfigService, $timeout, $compile, SweetAlert, dxDataGridService) {
        'use strict';
        return {
            restrict: "A",
            templateUrl: "model-config-manage-detail-info.html",
            replace: false,
            scope: {
                detailModel: "=",
                editStatus: "=",
                showType: "=",
                basicModel: "=",
                validationGroupName: "=",
                keyValueDataSource: "=",
                standardAccountList: "="
            },
            link: function ($scope, element) {
                $scope.element = element;
                $scope.featureItems = [
                    {
                        id: 1,
                        name: "指标分析"
                    },
                    {
                        id: 2,
                        name: "分录比对"
                    }
                ];
                $scope.showTypeEnus = [
                    {
                        ID: 1,
                        Name: "金额"
                    },
                    {
                        ID: 2,
                        Name: "百分比"
                    },
                    {
                        ID: 3,
                        Name: "异常判断"
                    }];

                $scope.conditionType = [
                    {
                        ID: 1,
                        Name: "主条件"
                    },
                    {
                        ID: 2,
                        Name: "并列条件"
                    },
                    {
                        ID: 3,
                        Name: "不包含条件"
                    },
                    {
                        ID: 4,
                        Name: "分录条件"
                    }];
                $scope.acctJudgeType = [
                    {
                        ID: 1,
                        Name: "N/A"
                    },
                    {
                        ID: 2,
                        Name: "="
                    },
                    {
                        ID: 3,
                        Name: "<>"
                    }];
                $scope.acctDirectionType = [
                    {
                        ID: 0,
                        Name: "借/贷"
                    },
                    {
                        ID: 1,
                        Name: "借"
                    },
                    {
                        ID: -1,
                        Name: "贷"
                    }];
                $scope.entriesComparisonType = [
                    {
                        ID: 1,
                        Name: "<"
                    },
                    {
                        ID: 2,
                        Name: ">"
                    },
                    {
                        ID: 3,
                        Name: ">="
                    },
                    {
                        ID: 4,
                        Name: "<="
                    },
                    {
                        ID: 5,
                        Name: "<>"
                    },
                    {
                        ID: 6,
                        Name: "="
                    }];
                $scope.selectedFeature = $scope.featureItems[0];

                $scope.formulaBtnType = 4;

                //用于记录当前被设为"主要值"数据行的 sequence
                $scope.isMainValueSequence = -1;
                $scope.isMainValueSequenceRadioName = "isMainValueSequenceRadio_" + (new Date).getTime();
                $scope.$watch("isMainValueSequence", function (newValue, oldValue) {
                    var dataList = $scope.detailModel.dataList;
                    _.each(dataList, function (x) {
                        if (!x.hasOwnProperty("isMainValue"))
                            return;
                        x.isMainValueShow = x.sequence == newValue;
                        x.isMainValue = x.isMainValueShow ? 1 : 0;
                    });
                });

                var dataListChanged = function () {
                    if ($scope.detailModel.dataList.length < 1 || !$scope.detailModel.dataList[0].hasOwnProperty("isMainValue")) {
                        $scope.isMainValueSequence = -1;
                    }
                    if ($scope.detailModel.dataList.length > 0 && $scope.detailModel.dataList[0].hasOwnProperty("isMainValue")) {
                        var firstIndex = _.findIndex($scope.detailModel.dataList, function (x) {
                            return x.isMainValueShow;
                        });

                        //当没有设定"主要值"时(firstIndex === -1),设置第一项为默认"主要值"
                        firstIndex = firstIndex < 0 ? 0 : firstIndex;
                        $scope.isMainValueSequence = $scope.detailModel.dataList[firstIndex].sequence;
                        $scope.detailModel.dataList[firstIndex].isMainValueShow = true;
                        $scope.detailModel.dataList[firstIndex].isMainValue = 1;

                        //当存在多个默认值时,除了第一个"主要值"之外其余全部将 isMainValueShow 设为 false
                        var count = _.countBy($scope.detailModel.dataList, function (x) {
                            return x.isMainValueShow;
                        })["true"];
                        if (count !== (void 0) && count > 1) {
                            for (var i = firstIndex + 1; i < $scope.detailModel.dataList.length; i++) {
                                if ($scope.detailModel.dataList[i] === (void 0))
                                    continue;

                                $scope.detailModel.dataList[i].isMainValueShow = false;
                                $scope.detailModel.dataList[i].isMainValue = 0;
                            }
                        }
                    }
                };

                $scope.$watch("detailModel.dataList", function (newValue, oldValue) {
                    dataListChanged();
                }, false);

                var getDataGrid = function () {
                    var dataGrid = $scope.element.find('[dx-data-grid]').dxDataGrid('instance');
                    return dataGrid;
                };

                var saveDataInRowOptions = function (rowOptions, dataField, value) {
                    rowOptions.data[dataField] = value;
                    if (rowOptions.hasOwnProperty("oldData")) {
                        rowOptions.oldData[dataField] = value;
                    }
                };

                //生成随机字符串
                var randomWord = function (allNumber, stringLength) {
                    var str = "";
                    var arr = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"];
                    var arrNum = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"];
                    for (var i = 0; i < stringLength; i++) {
                        if (allNumber) {
                            var pos = Math.round(Math.random() * (arrNum.length - 1));
                            str += arrNum[pos];
                        } else {
                            var pos = Math.round(Math.random() * (arr.length - 1));
                            str += arr[pos];
                        }
                    }

                    return str;
                };

                //true 验证通过,false 不通过
                var validateBeforeAddRow = function (dataGrid) {
                    if (dataGrid.hasEditData()) {
                        dataGrid.saveEditData();
                    }

                    var dataGridValidateRst = $scope.$parent.validateDataGrid(dataGrid);
                    if (!dataGridValidateRst.isValid) {
                        if (dataGridValidateRst.message) {
                            SweetAlert.warning(dataGridValidateRst.message);
                        } else {
                            SweetAlert.warning("模型明细存在不符合要求的数据,请修改正确后再添加数据!");
                        }

                        return false;
                    }
                    if ($scope.detailModel.dataList.length >= 100) {
                        SweetAlert.warning("添加失败,最多添加100行数据!");
                        return false;
                    }
                    return true;
                };

                $scope.dataGridAddRow = function () {
                    var dataGrid = getDataGrid();
                    if (!validateBeforeAddRow(dataGrid))
                        return;

                    if ($scope.detailModel.featureId === 1) {
                        var indexAnalysisDto = {
                            id: '',
                            modelID: '',
                            indexName: '',
                            indexFormula: '',
                            formulaNamae: "",
                            formulaDescribe: "",
                            sequence: '',
                            displayType: '',
                            isVisible: '',
                            isMainValue: '',
                            isYearValue: '',
                            createTime: '',
                            updateTime: '',
                            isVisibleShow: '',
                            isMainValueShow: '',
                            isYearValueShow: ''
                        };
                        indexAnalysisDto.sequence = $scope.detailModel.dataList.length + 1;
                        indexAnalysisDto.isVisibleShow = true;
                        indexAnalysisDto.isMainValueShow = $scope.detailModel.dataList.length < 1;
                        indexAnalysisDto.isYearValueShow = true;
                        $scope.detailModel.dataList.push(indexAnalysisDto);
                    }
                    if ($scope.detailModel.featureId === 2) {
                        var entriesCheckDto = {
                            id: '',
                            modelID: '',
                            GuideLine: '',
                            acctJudge: '',
                            standardCode: '',
                            stdCodeAndName: '',
                            acctDirection: '',
                            entriesComparison: '',
                            thresholdValue: '',
                            summary: '',
                            isRelevantAmt: null,
                            createTime: '',
                            updateTime: '',
                            indexShow: '',
                            isRelevantAmtShow: '',
                            summaryShow: ''
                        };
                        entriesCheckDto.indexShow = $scope.detailModel.dataList.length + 1;
                        entriesCheckDto.isRelevantAmtShow = false;
                        entriesCheckDto.thresholdValue = 0;
                        $scope.detailModel.dataList.push(entriesCheckDto);
                    }
                    dataListChanged();
                };

                $scope.dataGridDeleteRow = function () {
                    var dataGrid = getDataGrid();
                    var selectedRows = dataGrid.getSelectedRowKeys();
                    if (selectedRows.length == 1) {
                        var index = -1;
                        if ($scope.detailModel.featureId == 1) {
                            index = selectedRows[0].sequence;
                        }
                        if ($scope.detailModel.featureId == 2) {
                            index = selectedRows[0].indexShow;
                        }
                        $scope.detailModel.dataList.splice(index - 1, 1);
                        var i = 1;
                        $scope.detailModel.dataList.forEach(function (row) {
                            if ($scope.detailModel.featureId == 1) {
                                row.sequence = i;
                            }
                            if ($scope.detailModel.featureId == 2) {
                                row.indexShow = i;
                            }
                            i++;
                        });
                        dataListChanged();
                    }
                };

                $scope.dataGridRowUp = function () {
                    var dataGrid = getDataGrid();
                    var selectedRows = dataGrid.getSelectedRowKeys();
                    if (selectedRows.length == 1) {
                        var index = selectedRows[0].sequence;
                        if (index > 1) {
                            $scope.detailModel.dataList[index - 1].sequence = index - 1;
                            $scope.detailModel.dataList[index - 2].sequence = index;
                            $scope.detailModel.dataList.sort(function (a, b) {
                                return a.sequence - b.sequence;
                            });
                        }
                    }
                };

                $scope.dataGridRowDown = function () {
                    var dataGrid = getDataGrid();
                    var selectedRows = dataGrid.getSelectedRowKeys();
                    if (selectedRows.length == 1) {
                        var index = selectedRows[0].sequence;
                        if (index < $scope.detailModel.dataList.length) {
                            $scope.detailModel.dataList[index - 1].sequence = index + 1;
                            $scope.detailModel.dataList[index].sequence = index;
                            $scope.detailModel.dataList.sort(function (a, b) {
                                return a.sequence - b.sequence;
                            });
                        }
                    }
                };

                $scope.dataGridRowCopy = function () {
                    var dataGrid = getDataGrid();
                    if (!validateBeforeAddRow(dataGrid))
                        return;

                    var selectedRows = dataGrid.getSelectedRowKeys();
                    if (selectedRows.length == 1) {
                        var copyMto = angular.copy(selectedRows[0]);
                        copyMto.id = randomWord(false, 10);
                        copyMto.indexShow = $scope.detailModel.dataList.length + 1;
                        if (copyMto.hasOwnProperty('isMainValueShow')) {
                            copyMto.isMainValueShow = $scope.detailModel.dataList.length < 1;
                        }
                        $scope.detailModel.dataList.push(copyMto);
                    }
                    dataListChanged();
                };

                $scope.indexAnalysisGridColumns = [
                    {
                        dataField: "sequence",
                        caption: "计算顺序",
                        width: 70,
                        allowEditing: false,
                        alignment: "center",
                        sortIndex: 0,
                        sortOrder: 'asc'
                    },
                    {
                        dataField: "indexName",
                        caption: "指标名称",
                        allowEditing: true,
                        width: "40%",
                        editorOptions: {
                            placeholder: "请输入指标名称..."
                        },
                        validationRules: [
                            {
                                type: 'required',
                                message: '请输入指标名称'
                            },
                            {
                                type: 'stringLength',
                                max: 50,
                                message: '请输入不超过 50 字的指标名称'
                            },
                            {
                                type: 'custom',
                                message: '请保证指标名称的唯一性',
                                validationCallback: function (params) {
                                    var count = _.countBy($scope.detailModel.dataList, function (item) {
                                        return (item.indexName || "").trim() === (params.value || "").trim();
                                    })["true"];
                                    return count === (void 0) || count <= 1;
                                }
                            }
                        ]
                    },
                    {
                        dataField: "indexFormula",
                        caption: "指标公式",
                        allowEditing: true,
                        width: "60%",
                        editorOptions: {
                            placeholder: "请编辑指标公式...",
                            onValueChanged: function (e) {
                                var rowOptions = $(e.element).parents("tr:first").data().options;
                                saveDataInRowOptions(rowOptions, "indexFormula", e.value);
                                saveDataInRowOptions(rowOptions, "formulaNamae", e.value);
                                saveDataInRowOptions(rowOptions, "formulaDescribe", e.value);
                                rowOptions.component.cancelEditData();
                            }
                        },
                        validationRules: [
                            {
                                type: 'required',
                                message: '请输入指标公式'
                            },
                            {
                                type: 'stringLength',
                                max: 500,
                                message: '请输入不超过 500 字的指标公式'
                            }
                        ]
                    },
                    {
                        dataField: "displayType",
                        caption: "显示类型",
                        allowEditing: true,
                        editorOptions: {
                            placeholder: "请选择",
                            searchEnabled: false
                        },
                        width: 95,
                        lookup: {
                            dataSource: $scope.showTypeEnus,
                            displayExpr: "Name",
                            valueExpr: "ID"
                        },
                        validationRules: [
                            {
                                type: 'required',
                                message: '请选择显示类型'
                            }
                        ]
                    },
                    {
                        dataField: "isVisibleShow",
                        allowEditing: true,
                        caption: "是否显示",
                        width: "6em",
                        dataType: 'boolean'
                    },
                    {
                        alignment: "center",
                        caption: "主要值",
                        allowEditing: true,
                        width: 60,
                        cellTemplate: "isMainValueTemplate",
                        editCellTemplate: "isMainValueTemplate"
                    },
                    {
                        dataField: "isYearValueShow",
                        caption: "年度计算值",
                        allowEditing: true,
                        width: "7em",
                        dataType: 'boolean'
                    }
                ];

                $scope.entriesCheckGridColumns = [
                    {
                        dataField: "indexShow",
                        caption: "序号",
                        width: 80,
                        allowEditing: false,
                        alignment: "center",
                        sortIndex: 0,
                        sortOrder: 'asc'
                    },
                    {
                        dataField: "guideLine",
                        caption: "条件类型",
                        allowEditing: true,
                        editorOptions: {
                            placeholder: "请选择...",
                            searchEnabled: false
                        },
                        lookup: {
                            dataSource: $scope.conditionType,
                            displayExpr: "Name",
                            valueExpr: "ID"
                        },
                        validationRules: [
                            {
                                type: 'required',
                                message: '请选择条件类型'
                            }
                        ]
                    },
                    {
                        dataField: "acctJudge",
                        caption: "科目判断公式",
                        allowEditing: true,
                        width: "8em",
                        editorOptions: {
                            placeholder: "请选择...",
                            searchEnabled: false,
                            onValueChanged: function (e) {
                                var rowOptions = $(e.element).parents("tr:first").data().options;
                                saveDataInRowOptions(rowOptions, "acctJudge", e.value);
                                if (e.value === "N/A") {
                                    saveDataInRowOptions(rowOptions, "acctDirection", null);
                                    saveDataInRowOptions(rowOptions, "standardCode", null);
                                    saveDataInRowOptions(rowOptions, "stdCodeAndName", null);
                                    saveDataInRowOptions(rowOptions, "entriesComparison", null);
                                    saveDataInRowOptions(rowOptions, "thresholdValue", null);
                                }
                                rowOptions.component.cancelEditData();
                                if (e.value !== "N/A") {
                                    $scope.$parent.validateDataGrid(rowOptions.component);
                                }
                            }
                        },
                        lookup: {
                            dataSource: $scope.acctJudgeType,
                            displayExpr: "Name",
                            valueExpr: "Name"
                        },
                        validationRules: [
                            {
                                type: 'required',
                                message: '请选择科目判断公式'
                            }
                        ]
                    },
                    {
                        dataField: "standardCode",
                        caption: "标准科目",
                        allowEditing: true,
                        editCellTemplate: "standardCodeEditTemplate",
                        validationRules: [
                            {
                                type: 'required',
                                message: '请选择标准科目'
                            }
                        ]
                    },
                    {
                        dataField: "acctDirection",
                        caption: "科目方向",
                        allowEditing: true,
                        editorOptions: {
                            placeholder: "请选择...",
                            searchEnabled: false
                        },
                        lookup: {
                            dataSource: $scope.acctDirectionType,
                            displayExpr: "Name",
                            valueExpr: "ID"
                        },
                        validationRules: [
                            {
                                type: 'required',
                                message: '请选择科目方向'
                            }
                        ]
                    },
                    {
                        dataField: "entriesComparison",
                        caption: "分录比较公式",
                        allowEditing: true,
                        editorOptions: {
                            placeholder: "请选择...",
                            searchEnabled: false
                        },
                        width: "8em",
                        lookup: {
                            dataSource: $scope.entriesComparisonType,
                            displayExpr: "Name",
                            valueExpr: "Name"
                        },
                        validationRules: [
                            {
                                type: 'required',
                                message: '请选择分录比对公式'
                            }
                        ]
                    },
                    {
                        dataField: "thresholdValue",
                        caption: "阈值",
                        allowEditing: true,
                        editorOptions: {
                            placeholder: "请输入"
                        },
                        validationRules: [
                            {
                                type: 'required',
                                message: '请输入阈值'
                            },
                            {
                                type: "pattern",
                                pattern: "^-?\\d+$",
                                message: '请输入整数值'
                            }
                        ]
                    },
                    {
                        dataField: "summaryShow",
                        caption: "摘要",
                        allowEditing: true,
                        editorOptions: {
                            placeholder: "请编辑摘要..."
                        }
                    },
                    {
                        dataField: "isRelevantAmtShow",
                        caption: "借贷方相关金额",
                        allowEditing: true,
                        width: "9em",
                        dataType: 'boolean'
                    }
                ];
                $scope.gridColumns = angular.copy($scope.detailModel.featureId == 1 ? $scope.indexAnalysisGridColumns : $scope.entriesCheckGridColumns);

                //在编辑时切换 featureId 时需要清除 dataList 中的数据
                //在视图模式时不要清空,因为此时 dataList 存放的是当前的有效数据
                $scope.$watch("detailModel.featureId", function (newValue, oldValue) {
                    if (newValue !== oldValue) {
                        $scope.gridColumns.splice(0, $scope.gridColumns.length);

                        if ($scope.editStatus != 1) {
                            $scope.detailModel.dataList.splice(0, $scope.detailModel.dataList.length);
                        }

                        if (newValue == 1) {
                            for (var i = 0; i < $scope.indexAnalysisGridColumns.length; i++) {
                                $scope.gridColumns.push($scope.indexAnalysisGridColumns[i]);
                            }
                        }
                        if (newValue == 2) {
                            for (var i = 0; i < $scope.entriesCheckGridColumns.length; i++) {
                                $scope.gridColumns.push($scope.entriesCheckGridColumns[i]);
                            }
                        }
                    }
                });

                $scope.$watch("editStatus", function (newValue, oldValue) {

                    if ($scope.detailModel.featureId === 1) {
                        $scope.gridColumns.forEach(function (row) {
                            if ($scope.basicModel.type === 2) {
                                if (row.dataField === "indexFormula" || row.dataField === "isVisibleShow") {
                                    row.allowEditing = true;
                                }
                                else {
                                    row.allowEditing = false;
                                }
                            }
                            else {
                                if (row.dataField === "sequence") {
                                    row.allowEditing = false;
                                }
                                else {
                                    row.allowEditing = true;
                                }
                            }
                        });
                    }
                    var dataGrid = getDataGrid();
                    dataGrid.refresh();
                    dataGrid.option({
                        editing: {
                            allowUpdating: newValue !== 1
                        }
                    });
                });

                $scope.dataGridOptions = {
                    paging: {
                        enabled: false
                    },
                    editing: {
                        mode: "cell"
                    },
                    bindingOptions: {
                        dataSource: "detailModel.dataList",
                        columns: "gridColumns"
                    },
                    selection: {
                        mode: "single"
                    },
                    height: "auto",
                    allowColumnResizing: true,
                    loadPanel: {
                        enabled: false
                    },
                    noDataText: "没有数据",
                    sorting: false,
                    showBorders: true,
                    showRowLines: true,
                    onContentReady: function (e) {
                        console.log(e);
                        dxDataGridService.resize(e.component);
                    },
                    onEditorPreparing: function (e) {
                        if (e.parentType === "dataRow" && e.dataField !== "indexFormula" && e.dataField !== "acctJudge") {
                            e.editorOptions.onValueChanged = function (args) {
                                var rowOptions = $(args.element).parents("tr:first").data().options;
                                saveDataInRowOptions(rowOptions, e.dataField, args.value);
                                rowOptions.component.cancelEditData();
                            }
                        }
                    },
                    onCellPrepared: function (e) {
                        if (e.rowType === "data" && e.column.command === "edit") {
                            var isEditing = e.row.isEditing,
                                $links = e.cellElement.find(".dx-link");

                            $links.text("");

                            if (isEditing) {
                                $links.filter(".dx-link-save").addClass("dx-icon-save");
                                $links.filter(".dx-link-cancel").addClass("dx-icon-revert");
                            } else {
                                $links.filter(".dx-link-edit").addClass("dx-icon-edit");
                                $links.filter(".dx-link-delete").addClass("dx-icon-trash");
                            }
                        }

                        if (e.rowType === "data") {
                            var editorBtn = null;
                            if (e.column.dataField === "indexFormula") {
                                editorBtn = $("<button class='fa fa-cogs'></button>");
                                editorBtn.attr("ng-click", "openFormula(1,$event)");
                            }
                            if (e.column.dataField === "standardCode") {
                                editorBtn = $("<button class='fa fa-columns'></button>");
                                editorBtn.attr("ng-click", "stdCodeButtonClicked($event)");
                            }
                            if (e.column.dataField === "summaryShow") {
                                editorBtn = $("<button class='fa fa-edit'></button>");
                                editorBtn.attr("ng-click", "summaryButtonClicked($event)");
                            }
                            if (editorBtn != null) {
                                editorBtn.attr("ng-disabled", "editStatus === 1");
                                var input = $("<input readonly='readonly' />");
                                var div = $("<div class='button-container'/>");
                                // 拥有 dx-texteditor-buttons-container class 的元素,直接将 editorBtn 放到该容器中(一般编辑模式下才会有该 class)
                                //没有该容器的则自己包装一个容器用来盛放 editorBtn
                                if (e.cellElement.find(".dx-texteditor-buttons-container").length > 0) {
                                    if (e.column.dataField === "summaryShow") {
                                        input.attr("title", e.data.summary);
                                        input.val(e.data.summaryShow);
                                        div.append(input);
                                        div.append(editorBtn);
                                        e.cellElement.empty();
                                        e.cellElement.append($compile(div)($scope));
                                    }
                                    e.cellElement.find(".dx-texteditor-buttons-container").replaceWith($compile(editorBtn)($scope));
                                } else {
                                    if (e.column.dataField === "indexFormula") {
                                        input.attr("title", e.data.formulaDescribe);
                                        //无论为标准 keyValue 值还是自定义公式,显示样式内容都有后台处理,显示 formulaNamae 字段
                                        input.val(e.data.formulaNamae);
                                    }
                                    if (e.column.dataField === "standardCode") {
                                        input.attr("title", e.data.stdCodeAndName);
                                        input.val(e.data.stdCodeAndName);
                                    }
                                    if (e.column.dataField === "summaryShow") {
                                        input.attr("title", e.data.summary);
                                        input.val(e.data.summaryShow);
                                    }
                                    div.append(input);
                                    div.append(editorBtn);

                                    e.cellElement.empty();
                                    e.cellElement.append($compile(div)($scope));
                                }
                            }

                            //当 acctJudge 字段为 "N/A" 时,将关联的 acctDirection,standardCode,entriesComparison,thresholdValue 置空,并不显示任何内容
                            if (e.data.hasOwnProperty("acctJudge") && e.data.acctJudge === "N/A") {
                                if (e.column.dataField === "acctDirection" || e.column.dataField === "entriesComparison" || e.column.dataField === "standardCode" || e.column.dataField === "thresholdValue") {
                                    e.cellElement.empty();
                                    if (e.data[e.column.dataField])
                                        e.data[e.column.dataField] = "";
                                    if (e.column.dataField === "standardCode" && e.data.stdCodeAndName)
                                        e.data.stdCodeAndName = "";
                                }
                            }
                        }
                    },
                    onCellClick: function (e) {
                        if (e.rowType !== "data")
                            return;

                        var dataGrid = getDataGrid();
                        var allowUpdating = dataGrid.option("editing.allowUpdating");
                        var dataField = e.column.dataField;
                        var allowEditingColumn = e.column.allowEditing;
                        if (allowUpdating) {
                            //在编辑模式下,将单元格设为编辑状态
                            if (!e.isEditing &&
                                dataField !== "sequence" &&
                                dataField !== "indexShow" &&
                                !((dataField === "acctDirection" || dataField === "entriesComparison" || dataField === "thresholdValue") && e.data.acctJudge === "N/A") && allowEditingColumn) {
                                dataGrid.editCell(e.rowIndex, dataField);
                            }
                        }
                    }
                };

                $scope.standardCodeAutocompleteOptions = {
                    dataSource: $scope.standardAccountList,
                    minSearchLength: 1,
                    maxItemCount: 5,
                    searchTimeout: 50,
                    maxlength: 12,
                    placeholder: '请选择科目...',
                    valueExpr: 'code',
                    searchExpr: 'title',
                    searchMode: 'contains',
                    focusStateEnabled: true,
                    itemTemplate: 'autocompleteStandardCodeItemTemplate',
                    onInitialized: function (e) {
                        e.component.option({
                            value: e.model.item.data.standardCode
                        });
                    },
                    onItemClick: function (e) {
                        var data = e.model.item.data;
                        data.standardCode = e.itemData.code;
                        data.stdCodeAndName = e.itemData.code + "/" + e.itemData.fullName;
                    }
                };

                $scope.openFormula = function (type, $event) {

                    // 以下注释代码主要用于 keyValue 选择编辑,该代码主要是自定义实现
                    var rowOptions = $($event.target).parents("tr:first").data().options;
                    $scope.formulaBtnType = type;

                    var modalInstance = $uibModal.open({
                        animation: false,
                        ariaLabelledBy: 'modal-title',
                        ariaDescribedBy: 'modal-body',
                        templateUrl: 'model-config-manage-formula-edit-prop.html',
                        scope: $scope,
                        windowClass: "model-configuration-manage-formula-edit-prop"
                    });

                    $scope.keyValueConfirm = function (event) {
                        var dataGrid = $(event.target).parents(".model-configuration-manage-formula-edit-prop").find(".keyValueGridContainer").dxDataGrid('instance');
                        var selectedRows = dataGrid.getSelectedRowsData();
                        var selection = selectedRows.length ? selectedRows[0] : null;
                        if (!_.isEmpty(selection)) {
                            if ($scope.formulaBtnType == 1) {
                                saveDataInRowOptions(rowOptions, "indexFormula", "@" + selection.keyCode);
                                saveDataInRowOptions(rowOptions, "formulaNamae", selection.name);
                                saveDataInRowOptions(rowOptions, "formulaDescribe", selection.description);
                                rowOptions.component.cancelEditData();
                            }
                            //monthJudge 和 yearJudge,需要显示 formulaNamae,但是存储的值为 keyCode?
                            if ($scope.formulaBtnType == 2) {
                                $scope.detailModel.formulaCriterion.monthJudge = "@" + selection.keyCode;
                            }
                            if ($scope.formulaBtnType == 3) {
                                $scope.detailModel.formulaCriterion.yearJudge = "@" + selection.keyCode;
                            }
                        }

                        modalInstance.close();
                    };

                    $scope.selectorClose = function () {
                        modalInstance.dismiss('cancel');
                    };

                    //使用统一的 keyValue 编辑器,目前该控件尚未完成
                    //TODO:修改成 Edmond Meng 的统一控件
                }

                $scope.stdCodeButtonClicked = function ($event) {
                    var rowOptions = $($event.target).parents("tr:first").data().options;
                    var modalInstance = $uibModal.open({
                        animation: false,
                        ariaLabelledBy: 'modal-title',
                        ariaDescribedBy: 'modal-body',
                        templateUrl: 'model-config-manage-standard-edit-modal.html',
                        scope: $scope,
                        size: 600,
                        windowClass: "model-configuration-manage-standard-edit-modal"
                    });

                    $scope.saveStdCode = function (event) {
                        var treeList = $(event.target).parents(".model-configuration-manage-standard-edit-modal").find(".dx-treelist").dxTreeList('instance');
                        var selectedObjects = treeList.getSelectedRowsData();
                        if (selectedObjects.length < 1)
                            return;

                        var selectedNode = selectedObjects[0];
                        saveDataInRowOptions(rowOptions, "standardCode", selectedNode.code);
                        saveDataInRowOptions(rowOptions, "stdCodeAndName", selectedNode.code + "/" + selectedNode.fullName);
                        rowOptions.component.cancelEditData();
                        modalInstance.close();
                    };

                    $scope.closeStandardEditModal = function () {
                        modalInstance.dismiss('cancel');
                    };
                };

                $scope.summaryDataList = [];

                $scope.summaryButtonClicked = function ($event) {
                    var rowOptions = $($event.target).parents("tr:first").data().options;
                    $scope.summaryDataList = [];

                    var data = rowOptions.data;
                    var summary = data.summaryShow;
                    var strs = summary.split("或");
                    for (var i = 0; i < strs.length; i++) {
                        if (!PWC.isNullOrEmpty(strs[i].trim())) {
                            var mto = {};
                            mto.summaryString = strs[i].trim();
                            $scope.summaryDataList.push(mto);
                        }
                    }

                    var modalInstance = $uibModal.open({
                        animation: false,
                        ariaLabelledBy: 'modal-title',
                        ariaDescribedBy: 'modal-body',
                        templateUrl: 'model-config-manage-summary-edit-prop.html',
                        scope: $scope,
                        size: 400,
                        windowClass: "model-configuration-manage-summary-edit-prop"
                    });

                    $scope.deleteSummaryModule = function (model) {
                        if (model) {
                            $scope.summaryDataList.splice(jQuery.inArray(model, $scope.summaryDataList), 1);
                        }
                        ;
                    };
                    $scope.newSummaryModuleRow = function () {
                        if ($scope.summaryDataList && $scope.summaryDataList.length >= 20) {
                            SweetAlert.warning("添加失败,最多添加20行数据!");
                            return;
                        }

                        var row = {
                            summaryString: null
                        };
                        $scope.summaryDataList.push(row);
                    };

                    $scope.saveSummary = function () {
                        var summarys = '';
                        var summaryDb = '';
                        $scope.summaryDataList.forEach(function (summary) {
                            if (summary.summaryString) {
                                if (summarys) {
                                    summarys += " 或 " + summary.summaryString.trim();
                                } else {
                                    summarys = summary.summaryString.trim();
                                }
                                if (summaryDb) {
                                    summaryDb += " or ([" + summary.summaryString.trim() + "])";
                                } else {
                                    summaryDb = "([" + summary.summaryString.trim() + "])";
                                }
                            }
                        });

                        saveDataInRowOptions(rowOptions, "summaryShow", summarys);
                        saveDataInRowOptions(rowOptions, "summary", summaryDb);
                        rowOptions.component.cancelEditData();
                        modalInstance.close();
                    };

                    $scope.closeSummaryEditProp = function () {
                        modalInstance.dismiss('cancel');
                    };
                };

                $scope.gridOptionsKeyValue = {
                    dataSource: $scope.keyValueDataSource,
                    bindingOptions: {
                        height: 'selectorOptions.gridHeight'
                    },
                    paging: {
                        pageSize: 7
                    },
                    columns: [
                        {
                            dataField: 'id',
                            visible: false
                        }, {
                            caption: $translate.instant('KeyCode'),
                            dataField: 'keyCode',
                            visible: false,
                            fixed: true
                        }, {
                            caption: $translate.instant('KeyName'),
                            dataField: 'name'
                        }, {
                            caption: $translate.instant('Formula'),
                            dataField: 'formula'
                        }, {
                            caption: $translate.instant('Description'),
                            dataField: 'description'
                        }, {
                            caption: $translate.instant('ApplicationScope'),
                            calculateCellValue: function (rowData) {
                                return $translate.instant(rowData.scopeSummary);
                            },
                            calculateFilterExpression: function (filterValue, selectedFilterOperation) {
                                return [this.calculateCellValue, selectedFilterOperation || 'contains', filterValue];
                            }
                        }, {
                            caption: $translate.instant('ApplyService'),
                            dataField: 'serviceTypes'
                        }, {
                            caption: $translate.instant('IndustryID'),
                            dataField: 'industrys'
                        }
                    ],
                    loadPanel: {
                        enabled: false
                    },
                    //paging: {
                    //    pageSize: $scope.selectorOptions.pageSize
                    //},
                    scrolling: {
                        mode: "standard"
                    },
                    searchPanel: {
                        visible: true,
                        width: 240
                    },
                    selection: {
                        mode: 'single'
                    },
                    sorting: {
                        mode: 'single'
                    },
                    filterRow: {
                        visible: true
                    },
                    allowColumnResizing: true,
                    hoverStateEnabled: true,
                    showBorders: true,
                    showRowLines: true,
                    showColumnLines: true
                };

                //必须使用 validationRules 的拷贝数据,不然 validationRules 会被当做 validator 的 option 对象的一部分进行数据结构上的扩展,导致需要二次使用时数据结构已发生改变无法使用
                $scope.dxControlValidationRules = {
                    monthJudge: {
                        required: {
                            type: 'required',
                            message: '请输入月度异常判断公式'
                        },
                        stringLength: {
                            type: 'stringLength',
                            max: 200,
                            message: '请输入不大于 200 字的月度异常判断公式'
                        }
                    },
                    monthExplain: {
                        required: {
                            type: 'required',
                            message: '请输入月度异常判断公式的说明'
                        },
                        stringLength: {
                            type: 'stringLength',
                            max: 200,
                            message: '请输入不大于 200 字的月度异常判断公式的说明'
                        }
                    },
                    yearJudge: {
                        required: {
                            type: 'required',
                            message: '请输入年度异常判断公式'
                        },
                        stringLength: {
                            type: 'stringLength',
                            max: 200,
                            message: '请输入不大于 200 字的年度异常判断公式'
                        }
                    },
                    yearExplain: {
                        required: {
                            type: 'required',
                            message: '请输入年度异常判断公式的说明'
                        },
                        stringLength: {
                            type: 'stringLength',
                            max: 200,
                            message: '请输入不大于 200 字的年度异常判断公式的说明'
                        }
                    }
                };

                //name 是绑定字段名称,value 为当前控件的值用于进行判断对应控件是否需要添加 required 校验
                var fillFormulaCriterion = function (name, value) {
                    var rules = [angular.copy($scope.dxControlValidationRules[name].stringLength)];
                    if (value)
                        rules.push(angular.copy($scope.dxControlValidationRules[name].required));
                    $scope.dxControlsOptions[name].validatorInstance.option("validationRules", rules);
                };

                //公式和说明应该同时录入或不录入
                $scope.dxControlsOptions = {
                    monthJudge: {
                        ControlOption: {
                            bindingOptions: {
                                readOnly: "editStatus == 1",
                                value: "detailModel.formulaCriterion.monthJudge"
                            },
                            onContentReady: function (e) {
                                var btn = $("<button class='fa fa-cogs'></button>");
                                btn.attr("ng-click", "openFormula(2)");
                                btn.attr("ng-disabled", "editStatus == 1");
                                e.element.find(".dx-texteditor-buttons-container").replaceWith($compile(btn)($scope));
                            },
                            onValueChanged: function (e) {
                                fillFormulaCriterion("monthExplain", e.value);
                            }
                        },
                        validatorOption: {
                            validationRules: [
                                {
                                    type: 'stringLength',
                                    max: 200,
                                    message: '请输入不大于 200 字的月度异常判断公式'
                                }
                            ],
                            validationGroup: $scope.validationGroupName,
                            onInitialized: function (e) {
                                $scope.dxControlsOptions.monthJudge.validatorInstance = e.component;
                            }
                        },
                        validatorInstance: null
                    },
                    yearJudge: {
                        ControlOption: {
                            bindingOptions: {
                                readOnly: "editStatus == 1",
                                value: "detailModel.formulaCriterion.yearJudge"
                            },
                            onContentReady: function (e) {
                                var btn = $("<button class='fa fa-cogs'></button>");
                                btn.attr("ng-click", "openFormula(3)");
                                btn.attr("ng-disabled", "editStatus == 1");
                                e.element.find(".dx-texteditor-buttons-container").replaceWith($compile(btn)($scope));
                            },
                            onValueChanged: function (e) {
                                fillFormulaCriterion("yearExplain", e.value);
                            }
                        },
                        validatorOption: {
                            validationRules: [
                                {
                                    type: 'stringLength',
                                    max: 200,
                                    message: '请输入不大于 200 字的年度异常判断公式'
                                }
                            ],
                            validationGroup: $scope.validationGroupName,
                            onInitialized: function (e) {
                                $scope.dxControlsOptions.yearJudge.validatorInstance = e.component;
                            }
                        },
                        validatorInstance: null
                    },
                    monthExplain: {
                        ControlOption: {
                            bindingOptions: {
                                readOnly: "editStatus == 1",
                                value: "detailModel.formulaCriterion.monthExplain"
                            },
                            onValueChanged: function (e) {
                                fillFormulaCriterion("monthJudge", e.value);
                            }
                        },
                        validatorOption: {
                            validationRules: [
                                {
                                    type: 'stringLength',
                                    max: 200,
                                    message: '请输入不大于 200 字的月度异常判断公式的说明'
                                }
                            ],
                            validationGroup: $scope.validationGroupName,
                            onInitialized: function (e) {
                                $scope.dxControlsOptions.monthExplain.validatorInstance = e.component;
                            }
                        },
                        validatorInstance: null
                    },
                    yearExplain: {
                        ControlOption: {
                            bindingOptions: {
                                readOnly: "editStatus == 1",
                                value: "detailModel.formulaCriterion.yearExplain"
                            },
                            onValueChanged: function (e) {
                                fillFormulaCriterion("yearJudge", e.value);
                            }
                        },
                        validatorOption: {
                            validationRules: [
                                {
                                    type: 'stringLength',
                                    max: 200,
                                    message: '请输入不大于 200 字的年度异常判断公式的说明'
                                }
                            ],
                            validationGroup: $scope.validationGroupName,
                            onInitialized: function (e) {
                                $scope.dxControlsOptions.yearExplain.validatorInstance = e.component;
                            }
                        },
                        validatorInstance: null
                    },
                    feature: {
                        ControlOption: {
                            dataSource: $scope.featureItems,
                            displayExpr: "name",
                            valueExpr: "id",
                            bindingOptions: {
                                value: "detailModel.featureId"
                            }
                        },
                        validatorOption: {
                            validationRules: []
                        }
                    },
                    isFilter: {
                        ControlOption: {
                            text: "可筛选模型",
                            activeStateEnabled: false,
                            bindingOptions: {
                                value: "basicModel.isFilterShow",
                                disabled: "editStatus == 1",
                                readOnly: "editStatus == 1"
                            }
                        },
                        validatorOption: {
                            validationRules: []
                        }
                    }
                };

                $scope.standardEditDataGridOptions = {
                    dataSource: $scope.standardAccountList,
                    columns: [
                        {
                            dataField: "code",
                            caption: "科目代码",
                            width: 150
                        },
                        {
                            dataField: "fullName",
                            caption: "科目全称"
                        },
                        {
                            dataField: "directionName",
                            caption: "借贷方向",
                            width: 100
                        }
                    ],

                    loadPanel: {
                        enabled: true
                    },
                    scrolling: {
                        mode: "virtual"
                    },
                    keyExpr: "code",
                    parentIdExpr: "parentCode",
                    selection: {
                        mode: "single"
                    },
                    allowColumnResizing: true,
                    columnAutoWidth: true,
                    showRowLines: true,
                    showColumnLines: true,
                    rowAlternationEnabled: true, //单双行颜色
                    height: 400,
                    showBorders: true,
                    searchPanel: {
                        placeholder: $translate.instant('Search'),
                        visible: true
                    }
                };
            }
        };
    }]);

systemConfigurationModule.controller('fixedAssetsRuleEngineeController', ['$scope', '$translate', '$log', 'ruleEngineService', 'KeyValueConfigService', 'orgService', 'templateGroupService', 'ruleEngineOp', 'ruleEngineComparer','$timeout',
    function ($scope, $translate, $log, ruleEngineService, KeyValueConfigService, orgService, templateGroupService, ruleEngineOp, ruleEngineComparer, $timeout) {
        $scope.mappingData = [];
        $scope.mainData = [
            {
                id: 1, code: "1", name: "房屋、建筑物", year: "20年",
                mapData: [
                    { id: 1, name: "1 房屋、建筑物", keyValue: "土地、房屋及构建物" },
                    { id: 2, name: "1 房屋、建筑物", keyValue: "土地、海域及无居民海岛" },
                    { id: 3, name: "1 房屋、建筑物", keyValue: "房屋" },
                    { id: 4, name: "1 房屋、建筑物", keyValue: "构建物" }
                ]
            },
            {
                id: 2, code: "2", name: "飞机、火车、轮船、机器、机械和其它生成设备", year: "10年",
                mapData: [
                   { id: 1, name: "2 飞机、火车、轮船、机器、机械和其它生成设备", keyValue: "通用设备" },
                   { id: 2, name: "2 飞机、火车、轮船、机器、机械和其它生成设备", keyValue: "计算机设备及软件" },
                   { id: 3, name: "2 飞机、火车、轮船、机器、机械和其它生成设备", keyValue: "办公设备" },
                   { id: 4, name: "2 飞机、火车、轮船、机器、机械和其它生成设备", keyValue: "车辆" }
                ]
            },
            {
                id: 3, code: "3", name: "与生存经营活动有关的器具、工具、家具等", year: "5年",
                mapData: [
                   { id: 1, name: "3 与生存经营活动有关的器具、工具、家具等", keyValue: "专用设备" },
                   { id: 2, name: "3 与生存经营活动有关的器具、工具、家具等", keyValue: "探矿、采矿、选矿和造块设备" },
                   { id: 3, name: "3 与生存经营活动有关的器具、工具、家具等", keyValue: "石油天然气开采专用设备" },
                   { id: 4, name: "3 与生存经营活动有关的器具、工具、家具等", keyValue: "石油和化学工业专用设备" }
                ]
            },
            {
                id: 4, code: "4", name: "飞机、火车、轮船以外的运输工具", year: "4年",
                mapData: [
                       { id: 1, name: "4 飞机、火车、轮船以外的运输工具", keyValue: "文物" },
                       { id: 2, name: "4 飞机、火车、轮船以外的运输工具", keyValue: "陈列品" },
                ]
            },
            {
                id: 5, code: "5", name: "电子设备", year: "3年",
                mapData: [
                      { id: 1, name: "5 电子设备", keyValue: "图书、档案" }
                ]
            },
        ];
        var editModalSelector = ".fixed-assets-rule-enginee #edit-modal";
        var mappingModalSelector = ".fixed-assets-rule-enginee #map-modal"
        //清空
        $scope.resetEditingObject = function () {
            $scope.editingObject = { code: "", name: "", year: "" };
        }
        //创建
        $scope.createNewData = function () {
            $scope.isEdit = false;

            $scope.resetEditingObject();

            $(editModalSelector).modal('show');
        };

        //保存当前正在编辑数据行,经过界面初步校验后才会进入本功能
        $scope.saveEditRow = function () {
            if ($scope.editingObject.year)
                $scope.editingObject.year = $scope.editingObject.year + "年";
            $scope.mainData.push($scope.editingObject);
            $(editModalSelector).modal('hide');
        };

        //取消保存当前编辑行
        $scope.cancelEditOS = function () {
            $(editModalSelector).modal('hide');
        };
        //删除
        $scope.delete = function (data) {
            var index = $scope.mainData.indexOf(data);
            if (index !== -1) {
                $scope.mainData.splice(index, 1);
            }
        };
        $scope.editRow = function (data) {
            $scope.isEdit = true;
            $scope.editingObject = angular.copy(data)
            if (data.year) {
                var m = data.year.substring(0, data.year.length - 1);
                $scope.editingObject.year = m;
            }

            $(editModalSelector).modal('show');
        };
        $scope.editMapping = function (data) {
            $scope.mappingData = data.mapData;
            $(mappingModalSelector).modal('show');
            $timeout(function () {
                loadDetailMap();
            }, 100);          
        };
        $scope.deleteMap = function (data) {
            var index = $scope.mappingData.indexOf(data);
            if (index !== -1) {
                $scope.mappingData.splice(index, 1);
            }
        };
        $scope.saveMapRow = function () {
            $(mappingModalSelector).modal('hide');
        };

        //初始化invoice datagrid 树
        var InitInvoiceGrid = function () {

            var allowFilter = false;
            $scope.fixedAssetsGridOptions = {
                bindingOptions: {
                    dataSource: 'mainData',
                },
                hoverStateEnabled: true,
                headerFilter: {
                    visible: true,
                    height: 300,
                },
                selection: {
                    mode: "single"
                },
                paging: {
                    enabled: false,
                },
                allowColumnResizing: true,
                columnAutoWidth: true,
                showRowLines: true,
                showColumnLines: true,
                //rowAlternationEnabled: true,
                showBorders: true,
                noDataText: $translate.instant('NoDataText'),
                selectAllText: $translate.instant('SelectAll'),
                onContentReady: function (e) {
                    $scope.invoiceGridInstance = e.component;
                },
                columns: [
                            { dataField: 'code', caption: '标准固定资产代码', allowHeaderFiltering: allowFilter, alignment: 'left', width: 150 },
                    {
                        dataField: 'name', caption: '标准固定资产名称', allowHeaderFiltering: allowFilter, width: 450
                    },
                    { dataField: 'year', caption: '折旧年限', width: 100, allowHeaderFiltering: allowFilter },
                    {
                        dataField: 'id', caption: $translate.instant('Operation'), allowHeaderFiltering: allowFilter, alignment: 'center',
                        cellTemplate: function (container, options) {
                            try {
                                var data = options.data;

                                $("<span class='span-btn' />")
                                       .text("编辑")
                                       .on('click', function () {
                                           $scope.editRow(data);
                                       })
                                   .appendTo(container);

                                $("<span class='span-btn' />")
                                    .text("删除")
                                    .on('click', function () {
                                        $scope.delete(data);
                                    })
                                .appendTo(container);

                                $("<span class='span-btn' />")
                                    .text("对应关系维护")
                                    .on('click', function () {
                                        $scope.editMapping(data);
                                    })
                                .appendTo(container);



                            }
                            catch (e) {
                                $log.error(e);
                            }
                        }
                    },
                ],
            };
        };

        var loadDetailMap = function () {
            var allowFilter = false;
            $scope.fixedAssetsMapGridOptions = {
                bindingOptions: {
                    dataSource: 'mappingData',
                },
                //dataSource:$scope.mappingData,
                hoverStateEnabled: true,
                headerFilter: {
                    visible: true,
                    height: 300,
                },
                selection: {
                    mode: "single"
                },
                paging: {
                    enabled: false,
                },
                allowColumnResizing: true,
                columnAutoWidth: true,
                showRowLines: true,
                showColumnLines: true,
                //rowAlternationEnabled: true,
                showBorders: true,
                noDataText: $translate.instant('NoDataText'),
                selectAllText: $translate.instant('SelectAll'),
                onContentReady: function (e) {
                    $scope.invoiceGridInstance = e.component;
                },
                columns: [
                    { dataField: 'name', caption: '标准固定资产分类', allowHeaderFiltering: allowFilter, alignment: 'left', width: 300 },
                    {
                        dataField: 'keyValue', caption: '企业固定资产名称关键字', allowHeaderFiltering: allowFilter, width: 300
                    },

                    {
                        dataField: 'id', caption: $translate.instant('Operation'), allowHeaderFiltering: allowFilter, alignment: 'center',
                        cellTemplate: function (container, options) {
                            try {
                                var data = options.data;
                                $("<span class='span-btn' />")
                                    .text("删除")
                                    .on('click', function () {
                                        $scope.deleteMap(data);
                                    })
                                .appendTo(container);

                            }
                            catch (e) {
                                $log.error(e);
                            }
                        }
                    },
                ],
            };
        };

        //-----------custom table---end--------------------------
        (function initalization() {
            InitInvoiceGrid();
            loadDetailMap();
        })();
    }]);
systemConfigurationModule.directive('fixedAssetsRuleEnginee', ['$log',
    function ($log) {
        return {
            restrict: 'E',
            templateUrl: '/app/admin/systemConfiguration/ruleEngineeConfiguration/fixedAssetsRuleEnginee/fixed-assets-rule-enginee.html' + '?_=' + Math.random(),
            replace: true,
            scope: {},
            controller: 'fixedAssetsRuleEngineeController',
            link: function (scope, element, attr) {

            }
        }
    }]);
systemConfigurationModule
    .controller('ruleEngineeInfrastructureController', ['$scope', '$log', 'SweetAlert', 'organizationStructureService', '$translate', 'businessUnitService', 'customerService',
        function ($scope, $log, SweetAlert, organizationStructureService, $translate, businessUnitService, customerService) {
            'use strict';

            var successAddMessage = $translate.instant('OrganizationStructureAddSuccess');
            var successEditMessage = $translate.instant('OrganizationStructureEditSuccess');
            var successDeleteMessage = $translate.instant('OrganizationStructureDeleteSuccess');
            var operationDeleteMessage = $translate.instant('OrganizationStructureOperationSuccess');
            var notAllowDisableMessage = $translate.instant('notAllowDisableMessage');


            $scope.add = function (newrow) {

                organizationStructureService.addOrganizationStructure(newrow).success(function (guid) {
                    SweetAlert.success(successAddMessage);
                });
            };

            $scope.edit = function (newcontent) {
                organizationStructureService.updateOrganizationStructure(newcontent).success(function (guid) {
                    SweetAlert.success(successEditMessage);
                });
            };

            $scope.remove = function (id) {
                organizationStructureService.deleteOrganizationStructure(id).success(function (guid) {
                    SweetAlert.success(successDeleteMessage);
                });
            };

            $scope.stop = function (id) {
                organizationStructureService.updateOrganizationStructure(id).success(function (successFlag) {
                    $scope.updateflag = successFlag;
                    $scope.$broadcast('to-child', successFlag);
                    if (successFlag) {
                        SweetAlert.success(operationDeleteMessage);
                    } else {
                        SweetAlert.warning(notAllowDisableMessage);
                    }
                });
            };

            //BusinessUnit
            $scope.addbs = function (newrow) {

                businessUnitService.addBusinessUnit(newrow).success(function (guid) {
                    SweetAlert.success(successAddMessage);
                });
            };

            $scope.editbs = function (newcontent) {
                businessUnitService.updateBusinessUnit(newcontent).success(function (guid) {
                    SweetAlert.success(successEditMessage);
                });
            };

            $scope.removebs = function (id) {
                businessUnitService.deleteBusinessUnit(id).success(function (guid) {
                    SweetAlert.success(successDeleteMessage);
                });
            };

            $scope.stopbs = function (id) {
                businessUnitService.updateBusinessUnit(id).success(function (successFlag) {
                    $scope.updateflag = successFlag;
                    $scope.$broadcast('to-child', successFlag);
                    if (successFlag) {
                        SweetAlert.success(operationDeleteMessage);
                    } else {
                        SweetAlert.warning(notAllowDisableMessage);
                    }
                });
            };

            $scope.showOperateLogPop = function () {
                $scope.isShowLog = true;
                // $('#showOperatePop').modal('show');
            };

            var getUserPermission = function () {

                var list = [];

                var basicData = constant.adminPermission.basicData;
                list.push(basicData.orangizationStructure.queryCode);
                list.push(basicData.businessUnit.queryCode);
                list.push(basicData.areaManage.queryCode);
                list.push(basicData.enterpriseAccountSet.queryCode);
                list.push(basicData.customerList.queryCode);

                $scope.$root.checkUserPermissionList(list).success(function (data) {

                    $scope.orangizationStructureShow = data[basicData.orangizationStructure.queryCode];
                    $scope.businessUnitShow = data[basicData.businessUnit.queryCode];
                    $scope.areaManageShow = data[basicData.areaManage.queryCode];
                    $scope.enterpriseAccountSetShow = data[basicData.enterpriseAccountSet.queryCode];
                    $scope.customerListShow = data[basicData.customerList.queryCode];

                });
            };

            (function initialize() {
                $log.debug('ruleEngineInfrastructureController.ctor()...');

                //getUserPermission();

            })();
        }
    ]);
systemConfigurationModule.directive('ruleEngineInfrastructure', ['$log',
    function ($log) {
        return {
            restrict: 'E',
            templateUrl: '/app/admin/systemConfiguration/ruleEngineeConfiguration/ruleEngineeInfrastructure/rule-enginee-infrastructure.html' + '?_=' + Math.random(),
            replace: true,
            scope: {
                state:'='
            },
            controller: 'ruleEngineeInfrastructureController',
            link: function (scope, element, attr) {

            }
        }
    }]);
systemConfigurationModule.factory('ruleEngineComparer', ['ruleEngineService', function (ruleEngineService) {
    var itemState = {
        Removed: 1,
        Modified: 2,
        Added: 3,
        NoChanged: 4
    };
    function compareTaxPayerReportRule(originalItems, updatedItems) {
        var taxPayerReportRuleDtoOperations = [];
        var orginalMaps = {};
        var updatedMaps = {};

        originalItems.forEach(function (item) {
            if (item.hasOwnProperty('id')) {
                orginalMaps[item.id] = item;
            }
        });

        updatedItems.forEach(function (item) {
            if (item.hasOwnProperty('id')) {
                updatedMaps[item.id] = item;
            }
        });

        for (itemKey in orginalMaps) {
            var state = itemState.NoChanged;
            if (orginalMaps.hasOwnProperty(itemKey) === true) {
                if (updatedMaps.hasOwnProperty(itemKey) === true) {

                    var originalItem = orginalMaps[itemKey];
                    var updatedItem = updatedMaps[itemKey];

                    if (originalItem.taxPayerType !== updatedItem.taxPayerType) {
                        state = itemState.Modified;
                    }

                    if (originalItem.templateGroupID !== updatedItem.templateGroupID) {
                        state = itemState.Modified;
                    }

                    if (_.difference(originalItem.orgs, updatedItem.orgs).length > 0 || _.difference(updatedItem.orgs,originalItem.orgs).length > 0) {
                        state = itemState.Modified;
                    }
                }
                else {
                    state = itemState.Removed;
                }

                if (state === itemState.Removed) {
                    taxPayerReportRuleDtoOperations.push({action:'Delete',taxPayerReportRule:originalItem});
                } else if (state === itemState.Modified) {
                    taxPayerReportRuleDtoOperations.push({action:'Update',taxPayerReportRule:updatedItem});
                }
            }
        }

        updatedItems.forEach(function(item){
            if (item.isNotPersistent === true) {
                item.isNotPersistent = false;
                taxPayerReportRuleDtoOperations.push({action:'Add',taxPayerReportRule:item})
            }
        });

        return taxPayerReportRuleDtoOperations;
    }

    function compareTaxRuleSetting(originalItems, updatedItems) {
        var taxRuleSettingsOperations = [];
        var orginalMaps = {};
        var updatedMaps = {};

        originalItems.forEach(function (item) {
            if (item.hasOwnProperty('id')) {
                orginalMaps[item.id] = item;
            }
        });

        updatedItems.forEach(function (item) {
            if (item.hasOwnProperty('id')) {
                updatedMaps[item.id] = item;
            }
        });

        for (itemKey in orginalMaps) {
            var state = itemState.NoChanged;
            if (orginalMaps.hasOwnProperty(itemKey) === true) {
                if (updatedMaps.hasOwnProperty(itemKey) === true) {

                    var originalItem = orginalMaps[itemKey];
                    var updatedItem = updatedMaps[itemKey];
                    updatedItem.isNotAdded = true;

                    if (originalItem.groupName !== updatedItem.groupName) {
                        state = itemState.Modified;
                    }

                    if (originalItem.taxBase !== updatedItem.taxBase) {
                        state = itemState.Modified;
                    }

                    if (originalItem.taxRate !== updatedItem.taxRate) {
                        state = itemState.Modified;
                    }
                    
                    if (_.difference(originalItem.orgs, updatedItem.orgs).length > 0 || _.difference(updatedItem.orgs,originalItem.orgs).length > 0) {
                        state = itemState.Modified;
                    }
                }
                else {
                    state = itemState.Removed;
                }

                if (state === itemState.Removed) {
                    taxRuleSettingsOperations.push({ action: 'Delete', taxRuleSetting: originalItem });
                } else if (state === itemState.Modified) {
                    taxRuleSettingsOperations.push({ action: 'Update', taxRuleSetting: updatedItem });
                }
            }
        }

        updatedItems.forEach(function (item) {
            if (item.isNotPersistent === true) {
                item.isNotPersistent = false;
                taxRuleSettingsOperations.push({ action: 'Add', taxRuleSetting: item })
            }
        });

        return taxRuleSettingsOperations;
    }

    return {
        compareTaxRuleSetting: compareTaxRuleSetting,
        compareTaxPayerReportRule: compareTaxPayerReportRule
    }
}]);
systemConfigurationModule.factory('ruleEngineOp', ['ruleEngineService', function (ruleEngineService) {
    var operations = {};
    operations.taxRuleSettingOperations = [];
    operations.taxPayerReportRuleOperations = [];

    function add(obj, prop, op) {
        var addOperation = {};
        addOperation.action = 'Add';
        addOperation[prop] = op;

        operations[obj].push(addOperation);
        console.log(operations[obj]);
    }

    function update(obj, prop,op) {
        var ret = operations[obj].filter(function (operation) {
            return operation[prop].editKey === op.editKey;
        });
        if (ret.length !== 0) {
            return;
        }
        updateOperation = {};
        updateOperation.action = 'Update';
        updateOperation[prop] = op;
        operations[obj].push(updateOperation);
        console.log(operations[obj]);
    }

    function del(obj, prop, op) {
        var isAlreadyInList = operations[obj].filter(function (operation) {
            return operation[prop].editKey === op.editKey;
        }).length !== 0;

        if (isAlreadyInList === true) {
            operations[obj] = operations[obj].filter(function (operation) {
                return operation[prop].editKey !== op.editKey;
            });
        }

        if (op.isNotPersistent !== true) {
            var deleteOperation = {};
            deleteOperation.action = 'Delete';
            deleteOperation[prop] = op;
            operations[obj].push(deleteOperation);
        }
        console.log(operations[obj]);
    };

    function validationBeforCommit() {
        //start to validation
        //first find the blank row
        //second check validation count
       // console.log(operations.taxRuleSettingOperations);
        //console.log(operations.taxPayerReportRuleOperations);
        var isValid = true;
        operations.taxRuleSettingOperations.every(function (taxRuleSettingOperation) {
            if (taxRuleSettingOperation.action === 'Add' || taxRuleSettingOperation.action === 'Update') {
                var taxRuleSetting = taxRuleSettingOperation.taxRuleSetting;
                if (!(angular.isArray(taxRuleSetting.orgs) && taxRuleSetting.orgs.length > 0) && taxRuleSetting.isDefault === 0 ) {
                    isValid = false;
                } else if (!(angular.isString(taxRuleSetting.taxBase)  || taxRuleSetting.taxBase.trim() === '')) {
                    isValid = false;
                } else if (!angular.isNumber(taxRuleSetting.taxRate)) {
                    isValid = false;
                }

                if (isValid === false) {
                    return false;
                }
                return true;
            }
        });

        operations.taxPayerReportRuleOperations.every(function (taxPayerReportRuleOperation) {
            if (taxPayerReportRuleOperation.action === 'Add' || taxPayerReportRuleOperation.action === 'Update') {
                var taxPayerReportRule = taxPayerReportRuleOperation.taxPayerReportRule;
                if (!(angular.isArray(taxPayerReportRule.orgs) && taxPayerReportRule.orgs.length > 0) && taxPayerReportRule.isDefault === 0) {
                    isValid = false;
                } else if (!angular.isNumber(taxPayerReportRule.taxPayerType)) {
                    isValid = false;
                } else if (!angular.isString(taxPayerReportRule.templateGroupID)) {
                    isValid = false;
                }

                if (isValid === false) {
                    return false;
                }
                return true;
            }
        });

        return isValid;
    };


    return {
        addTaxRuleSettingOperation: function (taxRuleSetting) {
            add('taxRuleSettingOperations', 'taxRuleSetting', taxRuleSetting);
        },
        updateTaxRuleSettingOperation: function (taxRuleSetting) {
            update('taxRuleSettingOperations', 'taxRuleSetting', taxRuleSetting);
        },
        deleteTaxRuleSettingOperation: function (taxRuleSetting) {
            del('taxRuleSettingOperations', 'taxRuleSetting', taxRuleSetting);
        },
        addTaxPayerReportRuleOperation: function (taxPayerReportRule) {
            add('taxPayerReportRuleOperations', 'taxPayerReportRule', taxPayerReportRule);
        },
        updateTaxPayerReportRuleOperation: function (taxPayerReportRule) {
            update('taxPayerReportRuleOperations', 'taxPayerReportRule', taxPayerReportRule);
        },
        deleteTaxPayerReportRuleOperation: function (taxPayerReportRule) {
            del('taxPayerReportRuleOperations', 'taxPayerReportRule', taxPayerReportRule);
        },
        clearOperations:function() {
            operations.taxRuleSettingOperations = [];
            operations.taxPayerReportRuleOperations = [];
        },
        commit: function (successCallback) {
            if (validationBeforCommit()) {
                var batchUpdateTaxRuleDto = {
                    taxRuleSettingOperations: operations.taxRuleSettingOperations,
                    taxPayerReportRuleOperations: operations.taxPayerReportRuleOperations
                };

                //console.log(batchUpdateTaxRuleDto);
                ruleEngineService.saveTaxRuleSettings(batchUpdateTaxRuleDto).success(function () {
                    operations.taxRuleSettingOperations = [];
                    operations.taxPayerReportRuleOperations = [];
                    successCallback();
                });
            } else
            {
                //sweet alert
                swal('有未填写项');
            }
           
        }
    }
}]);
systemConfigurationModule.controller('ruleEngineeConfigController', ['$scope', '$translate', '$log', 'ruleEngineService', 'KeyValueConfigService', 'orgService', 'templateGroupService', 'ruleEngineOp','ruleEngineComparer',
    function ($scope, $translate, $log, ruleEngineService, KeyValueConfigService, orgService, templateGroupService, ruleEngineOp, ruleEngineComparer) {
        var taxRuleSetingGroupsBk, taxPayerReportMappingExceptionsBk, defaultTaxPayerReportMappingBk,taxRuleSettingOrginal,taxReportRuleOrginal
        function init() {
            $scope.state = 0;
            $scope.taxPayerReportState = 0;
            $scope.taxPayerReportMappingExceptions = [];
            $scope.taxRuleSettingsExceptionGroups = {}
            $scope.reportTypes = [];
            $scope.orgs = [];
            $scope.operation = {};
            $scope.currentSelectedOrgs = {value:[]};
            //load data.
            ruleEngineService.getTaxPayerReportMapping().success(function (data) {
                taxReportRuleOrginal = JSON.parse(JSON.stringify(data));
                var taxPayerReportMappings = data;

                $scope.defaultTaxPayerReportMapping = (taxPayerReportMappings.filter(function (item) {
                    return item.isDefault === 1;
                }))[0];

                $scope.taxPayerReportMappingExceptions = taxPayerReportMappings.filter(function (item) {
                    return item.isDefault !== 1;
                });
            });


            //to refactor when i have time 
            ruleEngineService.getTaxRuleSetting().success(function (data) {
                taxRuleSettingOrginal = JSON.parse(JSON.stringify(data));
                var taxRuleSettingsGroups = _.groupBy(data, 'groupName');

                var taxRuleSetingsFormatted = [];
                for (var groupName in taxRuleSettingsGroups) {
                    if (taxRuleSettingsGroups.hasOwnProperty(groupName)) {
                        var taxRuleSettings = taxRuleSettingsGroups[groupName];
                        var taxRuleSettingGroup = [];
                        taxRuleSettingGroup.state = 0;
                        var subTaxRuleSettingGroups = _.groupBy(taxRuleSettings, 'name');
                        for (var subGroupName in subTaxRuleSettingGroups) {
                            if (subTaxRuleSettingGroups.hasOwnProperty(subGroupName)) {
                                var subTaxRuleSettings = subTaxRuleSettingGroups[subGroupName];
                                var taxRuleSettingItem = {};
                                taxRuleSettingItem.groupName = groupName;
                                taxRuleSettingItem.groupNameDesc = setRuleDisplayVal(groupName);
                                taxRuleSettingItem.name = subGroupName;
                                taxRuleSettingItem.nameDesc = setRuleDisplayVal(subGroupName);

                                taxRuleSettingItem.defaultTaxRuleSetting = (subTaxRuleSettings.filter(function (item) {
                                    return item.isDefault === 1;
                                }))[0];

                                taxRuleSettingItem.taxRuleSettingsExceptions = subTaxRuleSettings.filter(function (item) {
                                    return item.isDefault !== 1;
                                });
                                //authority
                                mappingAuthority(subGroupName);
                                taxRuleSettingItem.TempView = $scope.TempView;
                                taxRuleSettingItem.TempDefaultAction = $scope.TempDefaultAction;
                                taxRuleSettingItem.TempAction = $scope.TempAction;
                                taxRuleSettingGroup.push(taxRuleSettingItem);
                            }
                            taxRuleSettingGroup.original = JSON.parse(JSON.stringify(taxRuleSettingGroup));
                        }
                        taxRuleSetingsFormatted.push(taxRuleSettingGroup);
                    }
                }
                $scope.taxRuleSetingGroups = taxRuleSetingsFormatted;
                taxRuleSetingGroupsBk = angular.copy($scope.taxRuleSetingGroups);
            });

            templateGroupService.getall().success(function (data) {
                $scope.reportTypes = data;
            });

            $scope.taxPayerTypes = [{
                id: 0,
                name: '一般纳税人',
            }, {
                id: 1,
                name: '小规模税人',
            }];

            orgService.getOrgListLevel().success(function (data) {
                $scope.orgs = data;
            });
        }

        $scope.defaultTaxPayerTypeOption = {
            bindingOptions: {
                value: 'defaultTaxPayerReportMapping.taxPayerType',
                dataSource: 'taxPayerTypes',
                disabled: '!defaultRatePayerAction||state === 0',
                onItemClick: 'defaultTaxPayerReportMappingChanged',
            },
            valueExpr: 'id',
            displayExpr: 'name',
        };

        $scope.defaultReportTypeOption = {
            bindingOptions: {
                value: 'defaultTaxPayerReportMapping.templateGroupID',
                dataSource: {
                    deep: true,
                    dataPath: 'reportTypes'
                },
                disabled: '!defaultRatePayerAction||state === 0',
                onItemClick: 'defaultTaxPayerReportMappingChanged',
            },
            width: '300px',
            valueExpr: 'id',
            displayExpr: 'name',
        }

        $scope.defaultTaxPayerReportMappingChanged = function () {
            if ($scope.defaultTaxPayerReportMapping.editKey == null) {
                $scope.defaultTaxPayerReportMapping.editKey = +new Date();
            }
            ruleEngineOp.updateTaxPayerReportRuleOperation($scope.defaultTaxPayerReportMapping);
        };

        $scope.showOperateLogPop = function () {
            $scope.isShowLog = true;
        };

        $scope.editTaxPayerReportMapping = function () {
            $scope.taxPayerReportState = 1;
        };

        $scope.saveTaxPayerReportMapping = function () {
            var taxPayerReportMapping = [];
            $scope.taxPayerReportState = 1;
            taxPayerReportMapping.push($scope.defaultTaxPayerReportMapping);
            taxPayerReportMapping = taxPayerReportMapping.concat($scope.taxPayerReportMappingExceptions);
            var batchUpdateTaxRuleDto = {
                taxRuleSettingOperations: [],
                taxPayerReportRuleOperations: ruleEngineComparer.compareTaxPayerReportRule(taxReportRuleOrginal, taxPayerReportMapping)
            };
            console.log(batchUpdateTaxRuleDto);

            ruleEngineService.saveTaxRuleSettings(batchUpdateTaxRuleDto).success(function () {
                var temp = JSON.parse(JSON.stringify(taxPayerReportMapping))
                taxPayerReportMapping = JSON.parse(JSON.stringify(taxReportRuleOrginal));
                taxReportRuleOrginal = temp;
                $scope.taxPayerReportState = 0;
            });
        };

        $scope.cancelTaxPayerReportMapping = function () {

            taxReportRuleOrginalTemp = JSON.parse(JSON.stringify(taxReportRuleOrginal));
            $scope.defaultTaxPayerReportMapping = (taxReportRuleOrginalTemp.filter(function (item) {
                return item.isDefault === 1;
            }))[0];

            $scope.taxPayerReportMappingExceptions = taxReportRuleOrginalTemp.filter(function (item) {
                return item.isDefault !== 1;
            });
            $scope.taxPayerReportState = 0;
        };


        $scope.editTaxRuleSeting = function (taxRuleSetingGroup) {
            taxRuleSetingGroup.state = 1;
        };

        $scope.saveTaxRuleSeting = function (taxRuleSetingGroup) {
            taxRuleSetingGroup.state = 1;
            var taxRuleSettings = [];
            var taxRuleSettingsOrginal = [];
            taxRuleSetingGroup.forEach(function (taxRuleSetting) {
                taxRuleSettings.push(taxRuleSetting.defaultTaxRuleSetting);
                taxRuleSettings = taxRuleSettings.concat(taxRuleSetting.taxRuleSettingsExceptions);
            });

            taxRuleSetingGroup.original.forEach(function (taxRuleSetting) {
                taxRuleSettingsOrginal.push(taxRuleSetting.defaultTaxRuleSetting);
                taxRuleSettingsOrginal = taxRuleSettingsOrginal.concat(taxRuleSetting.taxRuleSettingsExceptions);
            });

            var batchUpdateTaxRuleDto = {
                taxRuleSettingOperations: ruleEngineComparer.compareTaxRuleSetting(taxRuleSettingsOrginal, taxRuleSettings),
                taxPayerReportRuleOperations: []
            };
            console.log(batchUpdateTaxRuleDto);
            ruleEngineService.saveTaxRuleSettings(batchUpdateTaxRuleDto).success(function () {
                taxRuleSetingGroup.original = JSON.parse(JSON.stringify(taxRuleSetingGroup));
                taxRuleSetingGroup.state = 0;
            });
        };

        $scope.cancelTaxRuleSeting = function (taxRuleSetingGroup) {
            var original = JSON.parse(JSON.stringify(taxRuleSetingGroup.original));
            //taxRuleSetingGroup = JSON.parse(JSON.stringify(taxRuleSetingGroup.original));
            taxRuleSetingGroup.forEach(function (item,index) {
                item.groupName = taxRuleSetingGroup.original[index].groupName;
                item.name = taxRuleSetingGroup.original[index].name;
                item.TempAction = taxRuleSetingGroup.original[index].TempAction;
                item.TempDefaultAction = taxRuleSetingGroup.original[index].TempDefaultAction;
                item.TempView = taxRuleSetingGroup.original[index].TempView;
                item.defaultTaxRuleSetting = taxRuleSetingGroup.original[index].defaultTaxRuleSetting;
                item.taxRuleSettingsExceptions = taxRuleSetingGroup.original[index].taxRuleSettingsExceptions;
            });
            
            taxRuleSetingGroup.original = original;
            taxRuleSetingGroup.state = 0;
        };

        init();

        //----------custom table---start---------------------
        //get all key code
        var getAllKeyCode = function () {
            KeyValueConfigService.getAll().success(function (data) {
                $scope.mentionLists = _.map(data, function (config) { return { code: '@' + config.keyCode, name: config.name, description:config.description } });
            });
        };

        var checkPermissionList = function () {
            var list = [];
            var userManageTemp = constant.adminPermission.systemConfiguration.ruleEnginee;
            list = _.values(userManageTemp);
            $scope.viewRatePayer = false;
            $scope.defaultRatePayerAction = false;
            $scope.specialRatePayerAction = false;
            $scope.LocalTax = false;
            //地方税-城市建设
            $scope.viewCityBuilding = false;
            $scope.defaultCityBuildingAction = false;
            $scope.specialCityBuildingAction = false;
            //地方教育
            $scope.viewLocalEducation = false;
            $scope.defaultLocalEducationAction = false;
            $scope.specialLocalEducationAction = false;
            //教育费附加税 

            $scope.viewEducationAttach = false;
            $scope.defaultEducationAttachAction = false;
            $scope.specialEducationAttachAction = false;
            //印花税
            $scope.viewStampTax = false;
            $scope.defaultStampTaxAction = false;
            $scope.specialStampTaxAction = false;
            //水利基金
            $scope.viewWaterFund = false;
            $scope.defaultWaterFundAction = false;
            $scope.specialWaterFundAction = false;
            $scope.$root.checkUserPermissionList(list).success(function (data) {
                $scope.viewRatePayer = data[userManageTemp.viewRatePayer];
                $scope.defaultRatePayerAction = data[userManageTemp.defaultRatePayerAction];
                $scope.specialRatePayerAction = data[userManageTemp.specialRatePayerAction];
                $scope.LocalTax = data[userManageTemp.LocalTax];
                //地方税-城市建设
                $scope.viewCityBuilding = data[userManageTemp.viewCityBuilding];
                $scope.defaultCityBuildingAction = data[userManageTemp.defaultCityBuildingAction];
                $scope.specialCityBuildingAction = data[userManageTemp.specialCityBuildingAction];
                //地方教育
                $scope.viewLocalEducation = data[userManageTemp.viewLocalEducation];
                $scope.defaultLocalEducationAction = data[userManageTemp.defaultLocalEducationAction];
                $scope.specialLocalEducationAction = data[userManageTemp.specialLocalEducationAction];
                //教育费附加税 
                $scope.viewEducationAttach = data[userManageTemp.viewEducationAttach];
                $scope.defaultEducationAttachAction = data[userManageTemp.defaultEducationAttachAction];
                $scope.specialEducationAttachAction = data[userManageTemp.specialEducationExtactAction];
                //印花税
                $scope.viewStampTax = data[userManageTemp.viewStampTax];
                $scope.defaultStampTaxAction = data[userManageTemp.defaultStampTaxAction];
                $scope.specialStampTaxAction = data[userManageTemp.specialStampTaxAction];
                //水利基金
                $scope.viewWaterFund = data[userManageTemp.viewWaterFund];
                $scope.defaultWaterFundAction = data[userManageTemp.defaultWaterFundAction];
                $scope.specialWaterFundAction = data[userManageTemp.specialWaterFundAction];
            });
        };

        // TODO: 为了保证正常运行，暂时通过hard code将判断标准改为支持中英双语
        var taxs = {
            CityBuildingTax: '城市维护建设税',
            LocalEducationAttach: '地方教育费附加',
            WaterFund: '水利基金',
            StampTax: '印花税',
            EducationAttach: '教育费附加'
        };
        var mappingAuthority = function (name) {
            //"CityBuildingTax":"城市维护建设税",
            //"LocalEducationAttach":"地方教育费附加",
            //"WaterFund":"水利基金",
            //"StampTax":"印花税",
            //"EducationAttach":"教育费附加"
            switch (name) {
                case 'CityBuildingTax':
                case taxs.CityBuildingTax:
                    $scope.TempView = $scope.viewCityBuilding;
                    $scope.TempDefaultAction = $scope.defaultCityBuildingAction;
                    $scope.TempAction = $scope.specialCityBuildingAction;
                    break;
                case 'LocalEducationAttach':
                case taxs.LocalEducationAttach:
                    $scope.TempView = $scope.viewLocalEducation;
                    $scope.TempDefaultAction = $scope.defaultLocalEducationAction;
                    $scope.TempAction = $scope.specialLocalEducationAction;
                    break;
                case 'WaterFund':
                case taxs.WaterFund:
                    $scope.TempView = $scope.viewWaterFund;
                    $scope.TempDefaultAction = $scope.defaultWaterFundAction;
                    $scope.TempAction = $scope.specialWaterFundAction;
                    break;
                case 'StampTax':
                case taxs.StampTax:
                    $scope.TempView = $scope.viewStampTax;
                    $scope.TempDefaultAction = $scope.defaultStampTaxAction;
                    $scope.TempAction = $scope.specialStampTaxAction;
                    break;
                case 'EducationAttach':
                case taxs.EducationAttach:
                    $scope.TempView = $scope.viewEducationExtact;
                    $scope.TempDefaultAction = $scope.defaultEducationAttachAction;
                    $scope.TempAction = $scope.specialEducationAttachAction;
                    break;
                default:
                    $scope.TempView = false;
                    $scope.TempDefaultAction = false;
                    $scope.TempAction = false;
            }

        };

        var setRuleDisplayVal = function (groupName) {
            switch (groupName) {
                case taxs.CityBuildingTax: return $translate.instant('CityBuildingTax');
                case taxs.LocalEducationAttach: return $translate.instant('LocalEducationAttach');
                case taxs.WaterFund: return $translate.instant('WaterFund');
                case taxs.StampTax: return $translate.instant('StampTax');
                case taxs.EducationAttach: return $translate.instant('EducationAttach');
                default: return '';

            }
        };
       
        //-----------custom table---end--------------------------
        (function initalization() {
            $scope.TempView = false;
            $scope.TempDefaultAction = false;
            $scope.TempAction = false;
            checkPermissionList();
            getAllKeyCode();
        })();
    }]);
systemConfigurationModule.directive('ruleEngineConfig', ['$log',
    function ($log) {
        return {
            restrict: 'E',
            templateUrl: '/app/admin/systemConfiguration/ruleEngineeConfiguration/vatRuleEnginee/rule-enginee-config.html' + '?_=' + Math.random(),
            replace: true,
            scope: {},
            controller: 'ruleEngineeConfigController',
            link: function (scope, element, attr) {

            }
        }
    }]);
systemConfigurationModule.controller('ruleEngineeTableController', ['$scope', '$log', '$translate', 
    function ($scope, $log, $translate) {
        $scope.validNumber = function (e) {
            var event = e.jQueryEvent,
                str = event.key || String.fromCharCode(event.which);
            if (/^[\.\,e]$/.test(str))
                event.preventDefault();
        };
        $scope.validRateRequired = function (e) {
            var instance = e.component._options.value;
            if (_.isNull(instance) || _.isUndefined(instance)) {
                var event=e.jQueryEvent;
                $(event.target).parent().parent().css('border-color', 'red');
            }
        };
    }]);
systemConfigurationModule.directive('ruleEngineeTable', ['$log','ruleEngineOp','$rootScope','$timeout',
    function ($log, ruleEngineOp,$rootScope,$timeout) {
        return {
            restrict: 'E',
            templateUrl: '/app/admin/systemConfiguration/ruleEngineeConfiguration/vatRuleEnginee/rule-enginee-custome-table.html' + '?_=' + Math.random(),
            replace: true,
            
            scope: {
                taxBasisId: '@',//公式显示组件的id
                isEdit: '=',//是否是可编辑
                defaultEdit:'=',//默认税基是否可编辑
                orgs:'=',//机构下拉列表
                keyCodes:'=',//all key code
                editModel: '=', //所有改组件需要显示的数据来源,
                selectedOrgs: '=',
                operation:'=',
                // editModel={ defaultTaxBase:'xxx',defulatTaxRate:'yyyy',id:'zzzz',
                // exceptions:[{companies:'aaa',taxBase:'bbbb',taxRate:'ccc'}]}
            },
            controller: 'ruleEngineeTableController',
            link: function (scope, element, attr) {
                scope.keyCodesC = angular.copy(scope.keyCodes);

                var currentException;
                $rootScope.$on('orgsChanged', function (event, orgs) {
                    if (currentException !== undefined) {
                        currentException.orgs = orgs;
                    }
                });

                scope.addException = function () {
                    var newException = { id: PWC.newGuid(), GroupName: scope.editModel.groupName, name: scope.editModel.name, isNotPersistent: true, taxBase: '', editKey: +new Date() };
                    scope.editModel.taxRuleSettingsExceptions.push(newException);
                };

                scope.delete = function (exception) {
                    var index = scope.editModel.taxRuleSettingsExceptions.indexOf(exception);
                    if (index !== -1) {
                        if (exception.editKey == null) {
                            exception.editKey = +new Date();
                        }
                        scope.editModel.taxRuleSettingsExceptions.splice(index, 1);
                    }
                };

                scope.openRuleEngineSelectorModel = function (exception) {
                    if (scope.isEdit === true) {
                        scope.operation = { type: 'open' };
                        currentException = exception;
                        scope.selectedOrgs = exception.orgs;
                    }
                };
            }
        }
    }]);
systemConfigurationModule.directive('taxpayerTypeExceptionsTable', ['$log','ruleEngineOp','$rootScope',
    function ($log, ruleEngineOp, $rootScope) {
        return {
            restrict: 'E',
            templateUrl: '/app/admin/systemConfiguration/ruleEngineeConfiguration/vatRuleEnginee/taxpayer-type-exceptions-table.html' + '?_=' + Math.random(),
            replace: true,
            scope: {
                isEdit: '=',
                taxPayerExceptions: '=',
                orgs: '=',
                taxpayerTypes: '=',
                reportTypes:'=',
                selectedOrgs: '=',
                operation:'='
            },
            link: function (scope, element, attr) {

                var currentException;

                $rootScope.$on('orgsChanged', function (event, orgs) {
                    if (currentException !== undefined) {
                        currentException.orgs = orgs;
                    }
                });

                scope.openRuleEngineSelectorModel = function (exception) {
                    if (scope.isEdit === true) {
                        scope.operation = { type: 'open' };
                        currentException = exception;
                        scope.selectedOrgs = exception.orgs;
                    }
                };

                scope.addException = function () {
                    var newException = { id: PWC.newGuid(),isNotPersistent: true, editKey: +new Date() };
                    ruleEngineOp.addTaxPayerReportRuleOperation(newException);
                    scope.taxPayerExceptions.push(newException);
                };

                scope.delete = function (exception) {
                    var index = scope.taxPayerExceptions.indexOf(exception);
                    if (index !== -1) {
                        if (exception.editKey == null) {
                            exception.editKey = +new Date();
                        }
                        scope.taxPayerExceptions.splice(index,1);
                    }
                };

                scope.taxPayerTypesDropdownOption = {
                    bindingOptions: {
                        value: 'exception.taxPayerType',
                        dataSource: 'taxpayerTypes',
                        disabled:'!isEdit'
                    },
                    valueExpr: 'id',
                    displayExpr: 'name'
                };

                scope.reportTypesDropdownOption = {
                    bindingOptions: {
                        value: 'exception.templateGroupID',
                        dataSource: 'reportTypes',
                        disabled: '!isEdit'
                    },
                    valueExpr: 'id',
                    displayExpr: 'name'
                };

               
             
            }
        }
    }]);
systemConfigurationModule
.controller('EnterpriseSubjectCorrespondingController', ['$scope', '$log', 'uiGridConstants', 'SweetAlert', '$translate', 'stdAccountService', 'enterpriceAccountService', '$interval',
    function ($scope, $log, uiGridConstants, SweetAlert, $translate, stdAccountService, enterpriceAccountService, $interval) {
        'use strict';

        var defaultSelectStandardAccount = {
            id: 0,
            name: '',
            fullName: '',
            code: '',
            parentCode: '',
            acctLevel: 0,
            selectValue: ' '
        };

        var isActiveMap = { false: 'Disable', true: 'Enable' };
        var isActivedMap = { false: 'Disabled', true: 'Enabled' };

        var InitStdSubject = function () {
            stdAccountService.getStdAccountHierarchy().success(function (stdAccountsCategories) {
                $scope.stdAccountsCategories = stdAccountsCategories;
            });
        };

        var InitEnterpriseSubject = function () {
            enterpriceAccountService.getEnterpriceAccountList().success(function (data) {
                data.forEach(function (row) {
                    row.directionStr = _.find($scope.directionList, function (num) {
                        return num.id === row.direction
                    }).name;
                    row.subjectTypeStr = _.find($scope.acctPropList, function (num) {
                        return num.id === row.acctProp
                    }).name;
                });

                $scope.gridOptionsSubjectList.data = data;
            });
        };


        var mapStdAccount = function (code, name) {
            var accountMap = {};
            accountMap.enterpriseAccountCodes = [];
            accountMap.standardAccountCode = code;
            var selectedRows = $scope.gridApi.selection.getSelectedRows();
            if (selectedRows.length === 0) {
                SweetAlert.warning($translate.instant('SelectOneAccountToMap'));
                return;
            }
            selectedRows.forEach(function (v) {
                accountMap.enterpriseAccountCodes.push(v['code']);
            });
            enterpriceAccountService.mapAccount(accountMap).success(function (or) {
                if (or.result === true) {
                    $scope.gridApi.selection.clearSelectedRows();
                    SweetAlert.success($translate.instant('AccountMapSuccess'));
                    selectedRows.forEach(function (v) {
                        v['stdCode'] = code;
                        v['stdName'] = name;
                    });
                }
                else {
                    SweetAlert.warning($translate.instant(or.resultMsg).formatObj({ code: or.data.standardAccountCode }));
                }
            });
        };

        $scope.autoMap = function () {

            enterpriceAccountService.autoMap().success(function () {
                // mapping success
                InitEnterpriseSubject();
            });
        };

        $scope.showLinkModal = function () {
            $scope.isShowLinkModal = true;
            $scope.isShowLinkReport = false;
        };

        $scope.showLinkReport = function () {
            $scope.isShowLinkReport = true;
            $scope.isShowLinkModal = false;

        };

        var loadTemplateFormula = function () {
            // 先清空上次记录
            $scope.reportGridOptions.data = [];
            if ($scope.selectStandardAccount && $scope.selectStandardAccount.code) {
                templateFormulaService.getByStandardCode($scope.selectStandardAccount.code).success(function (data) {
                    $scope.reportGridOptions.data = data;
                });
            }
        };

        var showDetail = function (entity) {
            $scope.dbSelectedModel = entity;
            // 选中
            enterpriceAccountService.get($scope.dbSelectedModel.id).success(function (data) {
                $scope.editModel = data;
                if (data.stdCode && data.stdName) {
                    $scope.editModel.standardAccount = data.stdCode + '-' + data.stdName;
                }
                else {
                    $scope.editModel.standardAccount = $translate.instant('NotMap');
                }

                $scope.isAdd = false;

                $scope.editModel.title = $translate.instant('EnterpriceAccount') + '-' + '(' + data.code + ' ' + data.name + ')';

                $scope.selectStandardAccount = _.find($scope.standardAccountList, function (num) { return num.code === data.stdCode });

                if (!$scope.selectStandardAccount || !data.stdCode || data.stdCode === '') {
                    $scope.selectStandardAccount = defaultSelectStandardAccount;
                }

                $scope.selectDirection = _.find($scope.directionList, function (num) { return num.id === data.direction });
                $scope.selectAcctProp = _.find($scope.acctPropList, function (num) { return num.id === data.acctProp });

                $scope.isShowLinkModal = false;
                $scope.isShowLinkReport = true;
                $scope.iseditDetail = false;

                loadTemplateFormula();

                $('#enterpriseAccountDetailPopup').modal('show');
            });
        };

        $scope.reportGridOptions = {
            rowHeight: 28,
            selectionRowHeaderWidth: 28,
            enableFullRowSelection: false,
            enableRowSelection: false,
            enableSorting: false,
            enableFiltering: false,
            enableColumnMenus: false,
            enableRowHeaderSelection: false,
            enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
            columnDefs: [
            {
                field: 'templateName', name: $translate.instant('TemplateName'), width: '20%', headerCellClass: '',

                cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.templateName}}<span></div>'
            },
              {
                  field: 'serviceTypeName', name: $translate.instant('ServiceTypeName'), width: '20%', headerCellClass: '',

                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.serviceTypeName}}<span></div>'
              },

              {
                  field: 'cell', name: $translate.instant('Cell'), width: '10%', headerCellClass: '',

                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.cell}}</span></div>'
              },
              {
                  field: 'rowName', name: $translate.instant('RowName'), width: '15%', headerCellClass: '',

                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.rowName}}</span></div>'
              },
              {
                  field: 'columnName', name: $translate.instant('ColumnName'), width: '15%', headerCellClass: '',

                  cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.columnName}}</span></div>'
              },
               {
                   field: 'formula', name: $translate.instant('Formula'), width: '20%', headerCellClass: '',

                   cellTemplate: '<div class="ui-grid-cell-contents"><span>{{row.entity.formula}}</span></div>'
               }
            ]
        };

        (function initializa() {
            $log.debug('EnterpriseSubjectCorrespondingController.ctor()...');

            $scope.stdAccountsCategories = [];

            $scope.stdSubjectList = [];

            $scope.directionList = [
            { id: 1, name: $translate.instant('StandardAccountDebit') },
            { id: -1, name: $translate.instant('StandardAccountCredit') },
            ];

            $scope.acctPropList = [
            { id: 1, name: $translate.instant('StandardAccountAcctPropAsset') },
            { id: 2, name: $translate.instant('StandardAccountAcctPropDebt') },
            { id: 3, name: $translate.instant('StandardAccountAcctPropCommon') },
            { id: 4, name: $translate.instant('StandardAccountAcctPropInterest') },
            { id: 5, name: $translate.instant('StandardAccountAcctPropCost') },
            { id: 6, name: $translate.instant('StandardAccountAcctPropProfitAndLoss') },
            ];


            $scope.gridOptionsSubjectList = {
                rowHeight: 55,
                selectionRowHeaderWidth: 55,
                enableSorting: false,
                enableColumnMenus: false,
                enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
                enableRowSelection: true,
                enableRowHeaderSelection: false,
                columnDefs: [
                  {
                      field: 'code', name: $translate.instant('EnterpriseSubjectCodeCol'), width: '20%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>{{row.entity.code}}<span></div>'
                  },
                  {
                      field: 'name', name: $translate.instant('EnterpriseSubjectNameCol'), width: '25%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>{{row.entity.name}}</span></div>'
                  },
                  {
                      field: 'directionStr', name: $translate.instant('SubjectDirectionCol'), width: '15%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>{{row.entity.directionStr}}</span></div>'
                  },
                  {
                      field: 'subjectTypeStr', name: $translate.instant('SubjectTypeCol'), width: '15%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>{{row.entity.subjectTypeStr}}</span></div>'
                  },
                  {
                      field: 'stdCode', name: $translate.instant('StandardSubjectCol'), width: '25%', headerCellClass: 'text-align-center',
                      cellTemplate: '<div class="text-align-center" ng-dblClick="grid.appScope.showDetail(row.entity)"><span><i class="material-icons map-status-flag" ng-if="row.entity.stdCode == null">flag</i>{{row.entity.stdCode === null ? "未对应" : (row.entity.stdCode + "-" + row.entity.stdName)}}</span></div>'
                  }
                ],
                onRegisterApi: function (gridApi) {
                    $scope.gridApi = gridApi;
                    // call resize every 500 ms for 5 s after modal finishes opening - usually only necessary on a bootstrap modal
                    $interval(function () {
                        $scope.gridApi.core.handleWindowResize();
                    }, 500, 60 * 60 * 8);
                }
            };


            InitStdSubject();
            InitEnterpriseSubject();

            $scope.mapStdAccount = mapStdAccount;
            $scope.showDetail = showDetail;

        })();
    }
]);
systemConfigurationModule.directive('enterpriseSubjectCorresponding', ['$log',
    function ($log) {
        'use strict';
        $log.debug('enterpriseSubjectCorresponding.ctor()...');

        return {
            restrict: 'E',
            templateUrl: '/app/systemConfiguration/subjectCorresponding/enterpriseSubjectCorresponding/enterprise-subject-corresponding.html' + '?_=' + Math.random(),
            scope: {
                state: '='
            },
            controller: 'EnterpriseSubjectCorrespondingController',
            link: function (scope, element) {
            }
        };
    }
]);
systemConfigurationModule
.controller('StandardSubjectCorrespondingController', ['$scope', '$log', 'uiGridConstants', 'SweetAlert', '$translate', 'stdAccountService', 'enterpriceAccountService', '$interval',
    function ($scope, $log, uiGridConstants, SweetAlert, $translate, stdAccountService, enterpriceAccountService, $interval) {
        'use strict';

        var InitStdSubjectLinkEtsSubject = function () {
            stdAccountService.getStdAccountLinkEtsAccount().success(function (data) {
                data.forEach(function (row) {
                    row.directionStr = _.find($scope.directionList, function (num) {
                        return num.id === row.direction
                    }).name;
                    row.subjectTypeStr = _.find($scope.acctPropList, function (num) {
                        return num.id === row.acctProp
                    }).name;
                });
                $scope.gridOptionsStdSubjectList.data = data;
            });
        };

        (function initialize() {
            $log.debug('StandardSubjectCorrespondingController.ctor()...');
            $scope.stdSubjectList = [];
            $scope.directionList = [
            { id: 1, name: $translate.instant('StandardAccountDebit') },
            { id: -1, name: $translate.instant('StandardAccountCredit') },
            ];

            $scope.acctPropList = [
            { id: 1, name: $translate.instant('StandardAccountAcctPropAsset') },
            { id: 2, name: $translate.instant('StandardAccountAcctPropDebt') },
            { id: 3, name: $translate.instant('StandardAccountAcctPropCommon') },
            { id: 4, name: $translate.instant('StandardAccountAcctPropInterest') },
            { id: 5, name: $translate.instant('StandardAccountAcctPropCost') },
            { id: 6, name: $translate.instant('StandardAccountAcctPropProfitAndLoss') },
            ];

            $scope.gridOptionsStdSubjectList = {
                rowHeight: 55,
                selectionRowHeaderWidth: 55,
                enableSorting: false,
                enableColumnMenus: false,
                enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
                enableRowSelection: true,
                enableRowHeaderSelection: false,
                columnDefs: [
                  {
                      field: 'code', name: $translate.instant('StdSubjectCodeCol'), width: '20%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding"><span>{{row.entity.code}}<span></div>'
                  },
                  {
                      field: 'name', name: $translate.instant('StdSubjectNameCol'), width: '25%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding"><span>{{row.entity.name}}</span></div>'
                  },
                  {
                      field: 'directionStr', name: $translate.instant('SubjectDirectionCol'), width: '15%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding"><span>{{row.entity.directionStr}}</span></div>'
                  },
                  {
                      field: 'subjectTypeStr', name: $translate.instant('SubjectTypeCol'), width: '15%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding"><span>{{row.entity.subjectTypeStr}}</span></div>'
                  },
                  {
                      field: 'etsCode', name: $translate.instant('EtsSubjectNameCol'), width: '25%', headerCellClass: 'text-align-center',
                      cellTemplate: '<div class="text-align-center"><span><i class="material-icons map-status-flag" ng-if="row.entity.etsCode === null">flag</i>{{row.entity.etsCode === null ? "未对应" : (row.entity.etsCode + "-" + row.entity.etsName)}}</span></div>'
                  }
                ],
                onRegisterApi: function (gridApi) {
                    $scope.gridApi = gridApi;
                    // call resize every 500 ms for 5 s after modal finishes opening - usually only necessary on a bootstrap modal
                    $interval(function () {
                        $scope.gridApi.core.handleWindowResize();
                    }, 500, 60 * 60 * 8);
                }
            };

            InitStdSubjectLinkEtsSubject();
        })();
    }
]);
systemConfigurationModule.directive('standardSubjectCorresponding', ['$log',
    function ($log) {
        'use strict';
        $log.debug('standardSubjectCorresponding.ctor()...');

        return {
            restrict: 'E',
            templateUrl: '/app/systemConfiguration/subjectCorresponding/standardSubjectCorresponding/standard-subject-corresponding.html' + '?_=' + Math.random(),
            scope: {
                state: '='
            },
            controller: 'StandardSubjectCorrespondingController',
            link: function (scope, element) {
            }
        };
    }
]);
systemConfigurationModule.controller('SubjectCorrespondingController', ['$scope', '$log', 'uiGridConstants', 'SweetAlert', '$translate', 'stdAccountService', 'enterpriseAccountService', '$interval', 'orgService', '$timeout', '$q', 'commonWebService', 'loginContext',
    function ($scope, $log, uiGridConstants, SweetAlert, $translate, stdAccountService, enterpriseAccountService, $interval, orgService, $timeout, $q, commonWebService, loginContext) {
        'use strict';

        var defaultSelectStandardAccount = {
            id: 0,
            name: '',
            fullName: '',
            code: '',
            parentCode: '',
            acctLevel: 0,
            selectValue: ' '
        };

        //初始化页面右侧可双击进行对应的标准科目表数据
        var InitStdSubject = function () {
            //stdAccountService.getStdAccountHierarchy($scope.selectedOrgID).success(function (stdAccountsCategories) {
            //    $scope.stdAccountsCategories = stdAccountsCategories;
            //    console.log(stdAccountsCategories);
            //    commonWebService.initFancyTree('treetable', stdAccountsCategories);
            //});
        };

        //初始化企业科目数据列表
        var InitEnterpriseSubject = function (rowIndexArray) {
            //企业账套ID
            var espID = $scope.enterpriseAccountSetSelect && $scope.enterpriseAccountSetSelect.id || '';
            var orgID = $scope.selectedOrgID;

            //加载复制对应关系下的结构
            orgService.getOrgIvhTreeList(0, espID).success(function (treeData) {

                console.log(treeData);
                $scope.OrgMultiData = treeData;
                $scope.dsCheckBoxTreeView = treeData;
            });

            //Jacob
            getEspAccountList(espID, orgID, "-1");
        };

        ///根据标准科目code和name获取最终界面上应该显示的内容
        var getStdResult = function (stdCode, stdName) {
            var stdResult = stdCode === null ? $translate.instant('NotMapped') : (stdCode === "0000" ? "-" : stdCode + "-" + stdName);
            return stdResult;
        }

        var getEspAccountList = function (espID, orgID, filterType) {

            enterpriseAccountService.getEnterpriceAccountList(espID, orgID, filterType).success(function (data) {
                if (data === null) return;
                data.forEach(function (row) {

                    var direction = _.find($scope.directionList, function (num) {
                        return num.id === row.direction;
                    });

                    var acctProp = _.find($scope.acctPropList, function (num) {
                        return num.id === row.acctProp;
                    });

                    if (!direction || !acctProp) {
                        console.log('企业科目数据有问题！借贷方向或者科目类型不完全，请重新导入企业账套！ - ' + row.code);
                    }

                    row.directionStr = direction ? direction.name : '';
                    row.subjectTypeStr = acctProp ? acctProp.name : '';

                    if (row.treeLevel != null) {
                        row.$$treeLevel = parseInt(row.treeLevel);
                    }

                    row.stdResult = getStdResult(row.stdCode, row.stdFullName);

                });

                $scope.gridOptionsSubjectList.data = data;
                $scope.EnterpriceAccountList = data;
            });
        };

        //添加企业科目过滤下拉列表
        var InitFilterAccount = function () {

            var data = constant.espAccountFilter.data;
            data.forEach(function (row) {
                row.value = $translate.instant(row.value);
            });

            $scope.accountFilterList = data;
        };

        $scope.isSelected = function (id) {
            return $scope.filter.selected.indexOf(id) >= 0;
        };

        //筛选的时候，ng-check event will be triggered
        $scope.stateChanged = function ($event, data) {
            $scope.filter.selected = [];
            $scope.filterConditionDesc = '';
            var checkbox = $event.target;
            if (checkbox.checked && $scope.filter.selected.indexOf(data.key) < 0) {
                $scope.filter.selected.push(data.key);
                $scope.filterConditionDesc = data.value;
            }

            accountFilterChanged();
        };


        //根据不同的状态过滤，未对应，已对应，借贷方向不一致等
        //Jacob 
        var accountFilterChanged = function () {
            var selectArr = $scope.filter.selected;
            var data = $scope.EnterpriceAccountList;
            var key = $scope.filter.selected[0] || constant.espAccountFilter.key_0;
            var espID = $scope.enterpriseAccountSetSelect && $scope.enterpriseAccountSetSelect.id || '';
            var orgID = $scope.selectedOrgID;

            switch (key) {
                //所有对应状态,取默认值
                case constant.espAccountFilter.key_0:
                    getEspAccountList(espID, orgID, "-1");
                    break;
                    //未对应
                case constant.espAccountFilter.key_1:
                    getEspAccountList(espID, orgID, "0");
                    break;
                    //已对应
                case constant.espAccountFilter.key_2:
                    getEspAccountList(espID, orgID, "1");
                    break;
                    //借贷方向不一致
                case constant.espAccountFilter.key_3:
                    getEspAccountList(espID, orgID, "2");
                    break;
                    //科目类型不一致
                case constant.espAccountFilter.key_4:
                    getEspAccountList(espID, orgID, "3");
                    break;
            }
        };


        var InitNgData = function () {
            $scope.stdCollapsed = true;
            $scope.accountFilterList = [];
            //初始化选择器为空
            $scope.filter = {};
            $scope.filter.selected = [];
            //初始化企业科目筛选列表
            InitFilterAccount();

            $timeout(function () {
                $scope.isLoadComplete = true;
            }, 500);
        }


        var getUITreeScope = function (uiTreeID) {
            return angular.element(document.getElementById(uiTreeID)).scope();
        };
        //如果没有全部展开，return false.
        var isExpandAll = function (uiTreeID) {
            var childNodes = getUITreeScope(uiTreeID).childNodes();

            var isCollapsed = function (c) {
                if (c.collapsed)
                    return true;

                var childs = c.childNodes();

                for (var i = 0; i < childs.length; i++) {
                    if (isCollapsed(childs[i])) {
                        return true;
                    }
                }
                return false;
            }

            return !_.some(childNodes, isCollapsed);
        };

        //ui tree 折叠或展开所有
        var uiTreeExpandOrCollase = function (uiTreeID, iscollapsed) {
            var childNodes = getUITreeScope(uiTreeID).childNodes();
            try {
                if (iscollapsed) {
                    PWC.uiTree.collapsedAll(childNodes);
                }
                else {
                    PWC.uiTree.expandAll(childNodes);
                }
            }
            catch (ex) {
                if (iscollapsed) {
                    $scope.$broadcast('angular-ui-tree:collapse-all');
                }
                else {
                    $scope.$broadcast('angular-ui-tree:expand-all');
                }
            }
        };


        //标准科目，单独展开或折叠
        $scope.toggle1 = function (scope) {
            scope.toggle();
            if (scope.collapsed) {
                $scope.stdCollapsed = true;
            }
            else {
                //如果全部展开，最顶层也要展开
                if (isExpandAll('tree-root')) {
                    $scope.stdCollapsed = false;
                }
            }
        };

        //标准科目，全部展开或折叠
        $scope.toggleStdSubject = function () {
            $scope.stdCollapsed = !$scope.stdCollapsed;
            commonWebService.expandCollapseAll('treetable', $scope.stdCollapsed);
        }

        var reset = function () {
            //清空企业科目列表grid
            $scope.gridOptionsSubjectList.data = [];
            //清空标准科目对应结果数据列表
            $scope.gridOptionsStdSubjectList.data = [];
            //清空过滤器选择为空
            $scope.filter.selected = [];

        };


        //清空右侧标准科目
        var resetstdAccount = function () {
            $scope.stdAccountsCategories = null;
            $scope.stdCollapsed = true;
        };

        //清空账套列表
        var resetEspAccountSet = function () {
            $scope.enterpriseAccountSetList = null;

        };

        var loadData = function () {
            //企业科目全部折叠
            gridOptionsSubjectListCollapseAllRows();
            //初始化企业科目
            InitEnterpriseSubject();
            //初始化对应结果列表
            //InitStdSubjectLinkEtsSubject();
        };

        //根据选择的账套,构造页面数据
        var LoadEtsSubjectBySelectedAccpSetID = function () {
            reset();
            loadData();

            $('.subject-corresponding-container').css('display', 'block');
            $('.std-subject-container').css('display', 'none');
        };

        //根据选择的组织机构，构造账套列表和标准科目
        var LoadEtsSubjectBySelectedOrg = function () {
            reset();
            resetEspAccountSet();
            resetstdAccount();
            InitStdSubject();
            loadEnterpriseAccountSetList();
        };

        //根据机构 获取企业账套
        var loadEnterpriseAccountSetList = function () {

            enterpriseAccountService.getEnterpriseAccountSetListByOrgID($scope.selectedOrgID).success(function (data) {
                if (data && data.length > 0) {
                    $scope.enterpriseAccountSetList = data;
                    $scope.enterpriseAccountSetSelect = data[0];
                    if ($scope.enterpriseAccountSetSelect != null) {
                        loadData();
                    }
                }
            });
        };

        //初始化标准科目对应结果数据列表
        var InitStdSubjectLinkEtsSubject = function () {
            var espID = $scope.enterpriseAccountSetSelect && $scope.enterpriseAccountSetSelect.id || '';
            stdAccountService.getStdAccountLinkEtsAccount($scope.selectedOrgID, espID).success(function (data) {
                data.forEach(function (row) {
                    row.directionStr = _.find($scope.directionList, function (num) {
                        return num.id === row.direction;
                    }).name;
                    row.subjectTypeStr = _.find($scope.acctPropList, function (num) {
                        return num.id === row.acctProp;
                    }).name;
                    if (row.treeLevel != null) {
                        row.$$treeLevel = parseInt(row.treeLevel);
                    }

                    row.etsResult = row.etsCode === null ? $translate.instant('NotMapped') : (row.etsCode + "-" + row.etsName);
                });

                $scope.gridOptionsStdSubjectList.data = data;
            });
        };

        $scope.toggleRow = function (rowNum) {
            $scope.gridApiSubject.treeBase.toggleRowTreeState($scope.gridApiSubject.grid.renderContainers.body.visibleRowCache[rowNum]);
        };

        //手动对应，双击页面右侧的标准科目列表执行该事件
        var mapStdAccount = function (code, name, acctProp, direction) {
            if ($scope.selectedOrg && !$scope.selectedOrg.isHaveEditPermission) {
                SweetAlert.warning($translate.instant('NoEditPermission'));
                return;
            }

            var accountMap = {};
            var rowIndexArray = [];
            var isMapped = false;
            var isAcctPropMatched = false;
            var isDirectionMatched = false;

            accountMap.orgID = $scope.selectedOrgID;
            accountMap.enterpriseAccountCodes = [];
            accountMap.standardAccountCode = code;
            accountMap.accountSetID = $scope.enterpriseAccountSetSelect && $scope.enterpriseAccountSetSelect.id || '';

            var selectedRows = $scope.gridApiSubject.selection.getSelectedRows();
            if (selectedRows.length === 0) {
                SweetAlert.warning($translate.instant('SelectOneAccountToMap'));
                return;
            }

            var stdAcctProp = acctProp;
            var stdDirection = direction;

            selectedRows.forEach(function (v) {
                accountMap.enterpriseAccountCodes.push(v['code']);
                var rowIndex = findIndexByKeyValue($scope.gridOptionsSubjectList.data, "code", v['code'])
                rowIndexArray.push(rowIndex);

                var stdCode = v['stdCode'];
                var acctProp = v['acctProp'];
                var direction = v['direction'];

                //判断企业科目是否已有对应的标准科目，如果有isMapped=true
                if (stdCode != null) {
                    isMapped = true;
                }

                //判断企业科目和标准科目的类型否一致，如果不一致，isAcctPropMatched=true
                if (acctProp != -1 && acctProp != null && stdAcctProp != acctProp) {
                    isAcctPropMatched = true;
                }

                //判断企业科目和标准科目的借贷方是否一致，如果不一致，isDirectionMatched=true
                if (stdDirection != direction) {
                    isDirectionMatched = true;
                }
            });

            //如果已经map过
            if (isMapped) {
                msgboxOptions.text = $translate.instant('ComfirmToReMap');
                SweetAlert.swal(msgboxOptions, function (isConfirm) {
                    if (isConfirm) {
                        //如果科目类型不一致
                        if (isAcctPropMatched) {
                            msgboxOptions.text = $translate.instant('ConfirmToMapForDiffAcctProp');
                            SweetAlert.swal(msgboxOptions, function (isConfirm) {
                                if (isConfirm) {
                                    //如果科目类型一致，则判断借贷方向是否一致
                                    if (isDirectionMatched) {
                                        msgboxOptions.text = $translate.instant('ConfirmToMapForDiffDirection');
                                        SweetAlert.swal(msgboxOptions, function (isConfirm) {
                                            if (isConfirm) {
                                                mapAccountParentChild(selectedRows, accountMap, code, name);
                                            } else {
                                                swal.close();
                                            }
                                        });
                                    } else {
                                        mapAccountParentChild(selectedRows, accountMap, code, name);
                                    }
                                } else {
                                    swal.close();
                                }
                            });
                        }
                        else {
                            //如果借贷方向不一致
                            if (isDirectionMatched) {
                                msgboxOptions.text = $translate.instant('ConfirmToMapForDiffDirection');
                                SweetAlert.swal(msgboxOptions, function (isConfirm) {
                                    if (isConfirm) {
                                        mapAccountParentChild(selectedRows, accountMap, code, name);
                                    } else {
                                        swal.close();
                                    }
                                });
                            }
                            else {
                                mapAccountParentChild(selectedRows, accountMap, code, name);
                            }
                        }
                    } else {
                        swal.close();
                    }
                });
            }
            else {
                //如果没有map过，则判断科目类型是否一致
                if (isAcctPropMatched) {
                    msgboxOptions.text = $translate.instant('ConfirmToMapForDiffAcctProp');
                    SweetAlert.swal(msgboxOptions, function (isConfirm) {
                        if (isConfirm) {
                            //如果科目类型一致，则判断借贷方向是否一致
                            if (isDirectionMatched) {
                                msgboxOptions.text = $translate.instant('ConfirmToMapForDiffDirection');
                                SweetAlert.swal(msgboxOptions, function (isConfirm) {
                                    if (isConfirm) {
                                        mapAccountParentChild(selectedRows, accountMap, code, name);
                                    } else {
                                        swal.close();
                                    }
                                });
                            } else {
                                mapAccountParentChild(selectedRows, accountMap, code, name);
                            }
                        } else {
                            swal.close();
                        }
                    });
                }
                else {
                    //如果借贷方向不一致
                    if (isDirectionMatched) {
                        msgboxOptions.text = $translate.instant('ConfirmToMapForDiffDirection');
                        SweetAlert.swal(msgboxOptions, function (isConfirm) {
                            if (isConfirm) {
                                mapAccountParentChild(selectedRows, accountMap, code, name);
                            } else {
                                swal.close();
                            }
                        });
                    } else {
                        mapAccountParentChild(selectedRows, accountMap, code, name);
                    }
                }
            }
        };

        //弹出框的默认配置项
        var msgboxOptions = {
            title: $translate.instant('ConfirmToContinue'),
            text: $translate.instant('ComfirmToReMap'),
            type: "warning",
            showCancelButton: true,
            confirmButtonColor: "#DD6B55",
            confirmButtonText: $translate.instant('Confirm'),
            cancelButtonText: $translate.instant('Cancel'),
            closeOnConfirm: false,
            closeOnCancel: false
        };


        var mapAccountParentChild = function (selectedRows, accountMap, code, name) {
            enterpriseAccountService.mapAccount(accountMap).success(function (or) {
                if (or.result === true) {
                    $scope.gridApiSubject.selection.clearSelectedRows();

                    //如果重新从数据库加载，数据会重新绑定然后感觉界面被刷新
                    updateChildrenEnterpriseAccount(selectedRows, code, name);

                    //更新父级科目的状态
                    updateParentEnterpriseAccount(or.data.parentAccountMappingResult);

                    swal.close();
                }
                else {
                    SweetAlert.warning($translate.instant(or.resultMsg).formatObj({ code: or.data.standardAccountCode }));
                }
            });
        };

        var updateParentEnterpriseAccount = function (parentMapingDict) {

            for (var key in parentMapingDict) {
                $scope.gridApiSubject.grid.rows.forEach(function (row) {
                    if (row.entity.code === key) {
                        row.entity['stdCode'] = parentMapingDict[key][0];
                        row.entity['stdName'] = parentMapingDict[key][1];
                        row.entity['stdResult'] = getStdResult(row.entity['stdCode'], row.entity['stdFullName']);
                    }
                });
            }
        };

        //当在取消或者手动对应之后更新界面
        //selectedRows: uigrid中当前选中的行集合
        //codeToUpdate: 需要将旧的代码设置为该code
        //nameToUpdate: 需要将旧的科目名称设置为该name
        var updateChildrenEnterpriseAccount = function (selectedRows, codeToUpdate, nameToUpdate) {
            selectedRows.forEach(function (v) {
                v['stdCode'] = codeToUpdate;
                v['stdName'] = nameToUpdate;
                v['stdResult'] = getStdResult(codeToUpdate, nameToUpdate);

                //找出该选中节点的所有子节点，然后复制，避免再次从数据库获取数据
                $scope.gridApiSubject.grid.rows.forEach(function (row) {
                    if (row.entity.parentCode === v.code) {
                        row.entity['stdCode'] = codeToUpdate;
                        row.entity['stdName'] = nameToUpdate;
                        row.entity['stdResult'] = getStdResult(codeToUpdate, nameToUpdate);

                        //再次更新该row下面的子节点如果存在的话
                        updateChildrenMappedCodeNameRecursive($scope.gridApiSubject.grid.rows, row.entity['code'], codeToUpdate, nameToUpdate);
                    }
                });
            });
        };

        var updateChildrenMappedCodeNameRecursive = function (rows, filterCode, codeToUpdate, nameToUpdate) {
            $scope.gridApiSubject.grid.rows.forEach(function (row) {
                if (row.entity.parentCode === filterCode) {
                    row.entity['stdCode'] = codeToUpdate;
                    row.entity['stdName'] = nameToUpdate;
                    row.entity['stdResult'] = getStdResult(codeToUpdate, nameToUpdate);

                    //再次更新该row下面的子节点如果存在的话
                    updateChildrenMappedCodeNameRecursive($scope.gridApiSubject.grid.rows, row.entity['code'], codeToUpdate, nameToUpdate);
                }
            });
        }

        var findIndexByKeyValue = function (arrayToSearch, key, valueToSearch) {
            for (var i = 0; i < arrayToSearch.length; i++) {
                if (arrayToSearch[i][key] == valueToSearch) {
                    return i;
                }
            }
            return null;
        };


        //自动对应
        $scope.autoMap = function () {
            var espID = $scope.enterpriseAccountSetSelect && $scope.enterpriseAccountSetSelect.id || '';
            enterpriseAccountService.autoMap($scope.selectedOrgID, espID).success(function (or) {
                if (or != null && or != undefined) {
                    if (or.result) {
                        // mapping success
                        SweetAlert.success($translate.instant('AutoMappingSuccess'));
                        InitEnterpriseSubject();
                        //InitStdSubjectLinkEtsSubject();
                    }
                    else {
                        SweetAlert.warning($translate.instant(or.resultMsg));
                    }
                }
                else {
                    SweetAlert.warning($translate.instant('AccountMapFailed'));
                }
            });
        };

        $scope.getGridHeight = function () {
            if ($scope.isLoadComplete) {
                return {
                    height: $(".subject-corresponding-container #enterprizeAccount .subject-list-grid-warp").height() + "px"
                };
            }
        };

        //针对选中的取消对应
        var ClearMap = function () {
            var clearEnterpriseAccountIDs = [];

            var selectedRows = $scope.gridApiSubject.selection.getSelectedRows();
            if (selectedRows.length === 0) {
                SweetAlert.warning($translate.instant('SelectOneAccountToClearMap'));
                return;
            }

            selectedRows.forEach(function (v) {
                clearEnterpriseAccountIDs.push(v['id']);
            });

            enterpriseAccountService.clearMap(clearEnterpriseAccountIDs, $scope.selectedOrgID).success(function (or) {
                if (or != null && or != undefined) {
                    if (or.result) {
                        // clear mapping success
                        SweetAlert.success($translate.instant('ClearMapSuccess'));
                        //如果重新从数据库加载，数据会重新绑定然后感觉界面被刷新
                        updateChildrenEnterpriseAccount(selectedRows, null, '');
                        updateParentEnterpriseAccount(or.data.parentAccountMappingResult);
                        $scope.gridApiSubject.selection.clearSelectedRows();
                        //InitEnterpriseSubject(); 
                        //InitStdSubjectLinkEtsSubject();
                    }
                    else {
                        SweetAlert.warning($translate.instant(or.resultMsg));
                    }
                }
                else {
                    SweetAlert.warning($translate.instant('ClearMapFailed'));
                }
            });
        };

        var copyMappingCheck = function () {

            //判断当前选择的机构是否是同行业的，如果不是同一行业则给出提示
            var IsSameIndustry = true;
            if ($scope.selectedCopyOrgIndustryIDList.length > 0) {
                var fIndustryID = $scope.selectedCopyOrgIndustryIDList[0];

                $scope.selectedCopyOrgIndustryIDList.forEach(function (x) {
                    if (x != fIndustryID) {
                        IsSameIndustry = false;
                        return;
                    }
                });
            }

            //当前选中的账套ID
            var espID = $scope.enterpriseAccountSetSelect && $scope.enterpriseAccountSetSelect.id || '';

            //行业不一致弹出框确认配置
            var industryDifferConfirmOptions = {
                title: $translate.instant('ConfirmToContinue'),
                text: $translate.instant('IndustryConfirmToMapping'),
                type: "warning",
                showCancelButton: true,
                confirmButtonColor: "#DD6B55", confirmButtonText: $translate.instant('Confirm'),
                cancelButtonText: $translate.instant('Cancel'),
                closeOnConfirm: false,
                closeOnCancel: true
            };

            //行业不一致的时候需要弹框确认
            if (IsSameIndustry) {
                checkToCopyMapping(espID);
            } else {
                SweetAlert.swal(industryDifferConfirmOptions, function (isConfirmI) {
                    if (isConfirmI) {
                        checkToCopyMapping(espID);
                    }
                });
            }
        };

        //存在已经对应过的机构的确认配置
        var existedMapOrgOption = function (mappingExistOrg) {

            return {
                title: $translate.instant('ConfirmToContinue'),
                text: $translate.instant('ExistMappedOrg') + mappingExistOrg,
                type: "warning",
                showCancelButton: true,
                confirmButtonColor: "#DD6B55", confirmButtonText: $translate.instant('Confirm'),
                cancelButtonText: $translate.instant('Cancel'),
                closeOnConfirm: false,
                closeOnCancel: true
            }
        };

        //检查是否已有科目对应过的机构，并复制对应关系
        var checkToCopyMapping = function (espID) {

            //需要把当前机构的ID排除
            $scope.selectedCopyOrgIDList = _.filter($scope.selectedCopyOrgIDList, function (item) {
                return item !== $scope.selectedOrgID;
            });

            enterpriseAccountService.checkMapped($scope.selectedCopyOrgIDList, espID).success(function (data) {

                //检查是否可以复制对应关系，后端返回检查信息
                if (data && !data.result) {

                    //存在已对应过的机构，将对应过的机构组成字符串并弹框确认
                    var mappingExistOrg = '';
                    data.data.forEach(function (org) {
                        mappingExistOrg += org + ', ';
                    });
                    mappingExistOrg = '[' + mappingExistOrg + ']';

                    //弹框确认是否覆盖已对应过的机构
                    SweetAlert.swal(existedMapOrgOption(mappingExistOrg), function (isConfirmM) {
                        if (isConfirmM) {
                            copyMapping();
                        }
                    });

                } else if (data && data.result) {
                    copyMapping();
                } else {
                    console.log('data is undefined')
                }
            });
        };

        //复制对应关系到选中的组织机构
        var copyMapping = function () {

            var mappingParam = {};
            var espID = $scope.enterpriseAccountSetSelect && $scope.enterpriseAccountSetSelect.id || '';

            mappingParam.orgIDs = $scope.selectedCopyOrgIDList;
            mappingParam.selectedOrgID = $scope.selectedOrgID;
            mappingParam.selectedOrgSetID = espID;

            enterpriseAccountService.copyMapping(mappingParam).success(function (or) {
                if (or != null && or != undefined) {
                    if (or.result) {
                        SweetAlert.success($translate.instant('CopyMappingsSuccess'));
                    }
                    else {
                        SweetAlert.warning($translate.instant(or.resultMsg));
                    }
                }
                else {
                    SweetAlert.warning($translate.instant('CopyMappingsFailed'));
                }
            });
        };

        //点击明细数据弹出窗中的关联模型tab
        $scope.showLinkModal = function () {
            $scope.isShowLinkModal = true;
            $scope.isShowLinkReport = false;
        };

        //点击明细数据弹出窗中的关联报表tab
        $scope.showLinkReport = function () {
            $scope.isShowLinkReport = true;
            $scope.isShowLinkModal = false;

        };

        //加载明细数据中关联报表tab下的模板中的公式
        var loadTemplateFormula = function () {
            // 先清空上次记录
            $scope.reportGridOptions.data = [];
            if ($scope.selectStandardAccount && $scope.selectStandardAccount.code) {
                templateFormulaService.getByStandardCode($scope.selectStandardAccount.code).success(function (data) {
                    $scope.reportGridOptions.data = data;
                });
            }
        };

        //双击企业科目grid中的单条数据，显示明细数据
        var showDetail = function (entity) {
            $scope.dbSelectedModel = entity;
            //如果该条数据未进行对应，则不弹出明细窗口
            if (entity.stdCode === null) {
                $('.list').css('display', 'none');
            }
            else {
                $('.list').css('display', 'block');
            }
            // 选中
            enterpriseAccountService.get($scope.dbSelectedModel.id).success(function (data) {
                $scope.editModel = data;
                if (data.stdCode && data.stdName) {
                    $scope.editModel.standardAccount = data.stdCode + '-' + data.stdName;
                }
                else {
                    $scope.editModel.standardAccount = $translate.instant('NotMap');
                }

                $scope.isAdd = false;

                $scope.editModel.title = $translate.instant('EnterpriseAccount') + '-' + '(' + data.code + ' ' + data.name + ')';

                $scope.selectStandardAccount = _.find($scope.standardAccountList, function (num) {
                    return num.code === data.stdCode
                });

                if (!$scope.selectStandardAccount || !data.stdCode || data.stdCode === '') {
                    $scope.selectStandardAccount = defaultSelectStandardAccount;
                }

                $scope.selectDirection = _.find($scope.directionList, function (num) {
                    return num.id === data.direction
                });
                $scope.selectAcctProp = _.find($scope.acctPropList, function (num) {
                    return num.id === data.acctProp
                });

                $scope.isShowLinkModal = false;
                $scope.isShowLinkReport = true;
                $scope.iseditDetail = false;

                loadTemplateFormula();

                $('#enterpriseAccountDetailPopup').modal('show');
            });
        };


        //加载复制对应关系下的组织机构数据
        var loadOrgMultiData = function () {
            //orgService.getOrgIvhTreeList(0).success(function (data) {
            //    if (data.length > 0) {

            //        $scope.selectedOrgName = data[0].value.name;
            //        $scope.selectedOrgID = data[0].value.id;
            //    }
            //});
        };

        //勾选组织机构事件
        var checkboxCheckStateChangeHandling = function (node, disabled) {
            if (disabled) return;

            var isSelected = node.selected;
            var org = node.itemData.data.value;

            if (isSelected) {
                $scope.checkedSet[org.id] = org.industryID == constant.organization.EstateIndustryId ? org.industryID : constant.organization.NonEstateIndustryId;
            }
            else {
                delete $scope.checkedSet[org.id];
            }

            $scope.selectedCopyOrgIDList = _.keys($scope.checkedSet);
            $scope.selectedCopyOrgIndustryIDList = _.values($scope.checkedSet);
        };

        var otherAwesomeCallbackSpan = function (node, disabled) {
            if (disabled) return;
            node.selected = !node.selected;
            otherAwesomeCallback(node);
        };


        $scope.showOperateLogPop = function () {
            $scope.isShowLog = true;
            // $('#showOperatePop').modal('show');
        };

        var gridOptionsSubjectListCollapseAllRows = function () {

            $scope.gridApiSubject.treeBase.collapseAllRows();
        };

        $scope.$watch('selectedOrgID', function (newVal, oldVal) {
            if (newVal !== oldVal) {
                //初始化右侧标准科目的树,使用jquery fancytree的方式来显示树，所以需要用到ajax的方式
                var stdAccountUrl = '/stdAccount/stdAccountHierarchy?orgID=' + $scope.selectedOrgID;
                commonWebService.initFancyTreeAjax('treetable', stdAccountUrl, mapStdAccount);

                $scope.LoadEtsSubjectBySelectedOrg();
            }
        });

        //加载复制对应关系 机构选择树形结构
        var loadOrgCheckboxTreeViewSettings = function () {

            $scope.orgCheckboxTreeViewOptions = {

                bindingOptions: {
                    dataSource: 'dsCheckBoxTreeView',
                    searchValue: 'searchValue'
                },
                selection: {
                    mode: "multiple"
                },
                loadPanel: {
                    enabled: true
                },
                scrolling: {
                    mode: "virtual"
                },
                keyExpr: "id",
                showRowLines: true,
                showColumnLines: true,
                expandAllEnabled: true,
                rowAlternationEnabled: true,
                expandedRowKeys: [1],
                showBorders: true,
                noDataText: $translate.instant('NoDataText'),
                selectAllText: $translate.instant('SelectAll'),
                displayExpr: 'text', //显示的属性名称，默认为text
                itemsExpr: 'items', //子层的数组名称，默认为items 
                selectNodesRecursive: false, //级联选择
                showCheckBoxesMode: 'normal', //Accepted Values: 'none' | 'normal' | 'selectAll' 
                onInitialized: function (e) {
                    $scope.gridInstance = e.component;
                },
                onItemSelectionChanged: function (e) {
                    checkboxCheckStateChangeHandling(e.node);
                },
                //点击行，选中
                onItemClick: function (e) {
                    var data = e.itemData;
                },
                onContentReady: function (object, container) {
                    //object.element.find('.dx-checkbox-checked').addClass('dx-state-disabled');
                }
            };
        };

        //初始化企业科目grid的设置
        var loadStdAccountListOptions = function () {
            //标准科目对应结果列表grid
            $scope.gridOptionsStdSubjectList = {
                rowHeight: constant.UIGrid.rowHeight,
                selectionRowHeaderWidth: constant.UIGrid.selectionRowHeaderWidth,
                enableSorting: false,
                enableColumnMenus: false,
                enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
                enableRowSelection: true,
                enableSelectAll: true,
                enableRowHeaderSelection: false,
                enableFiltering: false,
                enableColumnResizing: true,
                showTreeExpandNoChildren: false,
                columnDefs: [
                    {
                        field: 'code', name: $translate.instant('StdSubjectCodeCol'), width: '20%', headerCellClass: '',
                        cellTemplate: '<div class="text-align-left-padding"><span title="{{row.entity.code}}">{{row.entity.code}}<span></div>'
                    },
                    {
                        field: 'name', name: $translate.instant('StdSubjectNameCol'), width: '25%', headerCellClass: '',
                        cellTemplate: '<div class="text-align-left-padding"><span title="{{row.entity.fullName}}">{{row.entity.fullName}}</span></div>'
                    },
                  {
                      field: 'directionStr', name: $translate.instant('SubjectDirectionCol'), width: '15%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding"><span title="{{row.entity.directionStr}}">{{row.entity.directionStr}}</span></div>'
                  },
                  {
                      field: 'subjectTypeStr', name: $translate.instant('SubjectTypeCol'), width: '15%', headerCellClass: '',
                      cellTemplate: '<div class="text-align-left-padding"><span title="{{row.entity.subjectTypeStr}}">{{row.entity.subjectTypeStr}}</span></div>'
                  },
                    {
                        field: 'etsCode', name: $translate.instant('EtsSubjectNameCol'), width: '25%', headerCellClass: 'text-align-center',
                        cellTemplate: '<div class="text-align-left-padding" style="margin-right:40px"><i class="material-icons map-status-flag" ng-if="row.entity.etsCode === null">flag</i><span  title="{{row.entity.etsResult}}">{{row.entity.etsResult}}</span></div>'
                    }
                ],
                onRegisterApi: function (gridApi) {
                    $scope.gridApiStdSubject = gridApi;
                    // call resize every 500 ms for 5 s after modal finishes opening - usually only necessary on a bootstrap modal
                    $interval(function () {
                        $scope.gridApiStdSubject.core.handleWindowResize();
                    }, 500, 60 * 60 * 8);
                }
            };
        };

        //初始化标准科目grid的设置
        var loadEnterpriseAccountListOptions = function () {

            //企业科目列表grid
            $scope.gridOptionsSubjectList = {
                enableColumnResizing: true,
                enableRowSelection: false,
                enableSelectAll: true,
                rowHeight: constant.UIGrid.rowHeight,
                enableRowHeaderSelection: true,
                modifierKeysToMultiSelect: false,
                enableHorizontalScrollbar: uiGridConstants.scrollbars.enableHorizontalScrollbar,
                enableVerticalScrollbar: uiGridConstants.scrollbars.enableVerticalScrollbar,
                selectionRowHeaderWidth: constant.UIGrid.selectionRowHeaderWidth,
                enableSorting: false,
                enableColumnMenus: false,
                enableFiltering: false,
                showTreeExpandNoChildren: false,
                enableFullRowSelection: true, //是否点击行任意位置后选中,默认为false
                multiSelect: true,// 是否可以选择多个,默认为true;
                columnDefs: [
                    {
                        field: 'code',
                        width: '17%',
                        name: $translate.instant('EnterpriseSubjectCodeCol'),
                        headerCellClass: '',
                        cellTemplate: '<div ng-if="row.entity.acctLevel==1" class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>{{row.entity.code}}<span></div>' +
                                      '<div ng-if="row.entity.acctLevel==2" class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>&nbsp;{{row.entity.code}}<span></div>' +
                                      '<div ng-if="row.entity.acctLevel==3" class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>&nbsp;&nbsp{{row.entity.code}}<span></div>' +
                                      '<div ng-if="row.entity.acctLevel==4" class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>&nbsp;&nbsp;&nbsp{{row.entity.code}}<span></div>' +
                                      '<div ng-if="row.entity.acctLevel==5" class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>&nbsp;&nbsp;&nbsp;&nbsp;{{row.entity.code}}<span></div>' +
                                      '<div ng-if="row.entity.acctLevel==6" class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;{{row.entity.code}}<span></div>' +
                                      '<div ng-if="row.entity.acctLevel==7" class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp{{row.entity.code}}<span></div>'
                    },
                        {
                            field: 'name',
                            name: $translate.instant('EnterpriseSubjectNameCol'),
                            headerCellClass: '',
                            width: '20%',
                            cellTemplate: '<div class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span title="{{row.entity.name}}">{{row.entity.name}}</span></div>'
                        },
                        {
                            field: 'directionStr',
                            name: $translate.instant('SubjectDirectionCol'),
                            headerCellClass: '',
                            cellTemplate: '<div class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span title="{{row.entity.directionStr}}">{{row.entity.directionStr}}</span></div>'
                        },
                        {
                            field: 'subjectTypeStr',
                            name: $translate.instant('SubjectTypeCol'),
                            headerCellClass: '',
                            cellTemplate: '<div class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span title="{{row.entity.subjectTypeStr}}">{{row.entity.subjectTypeStr}}</span></div>'
                        },
                        {
                            field: 'stdCode',
                            name: $translate.instant('StandardSubjectCol'),
                            headerCellClass: '',
                            width: '25%',
                            cellTemplate: '<div class="text-align-left-padding" ng-dblClick="grid.appScope.showDetail(row.entity)"><span title={{row.entity.stdResult}}><i class="material-icons map-status-flag" ng-if="row.entity.stdCode == null">flag</i>{{row.entity.stdResult}}</span></div>'
                        }
                ],
                onRegisterApi: function (gridApi) {
                    $scope.gridApiSubject = gridApi;
                    $scope.gridSelectCount = $scope.gridApiSubject.selection.getSelectedRows().length;
                    var onRowsRenderedOff = gridApi.core.on.rowsRendered(null, function () {
                        onRowsRenderedOff(); // run once
                        PWC.triggerRowSelectOnClick('#etsGrid'); // requires '.ui-grid-canvas'
                    });
                    // call resize every 500 ms for 5 s after modal finishes opening - usually only necessary on a bootstrap modal
                    $interval(function () {
                        $scope.gridApiSubject.core.handleWindowResize();
                    }, 500, 60 * 60 * 8);
                }
            };
        };

        var loadReportGridOptions = function () {
            //明细数据弹出框中的关联报表tab下的grid
            $scope.reportGridOptions = {
                rowHeight: 28,
                selectionRowHeaderWidth: 28,
                enableFullRowSelection: false,
                enableRowSelection: false,
                enableSorting: false,
                enableFiltering: false,
                enableColumnMenus: false,
                enableColumnResizing: true,
                enableRowHeaderSelection: false,
                enableHorizontalScrollbar: uiGridConstants.scrollbars.NEVER,
                columnDefs: [
                    {
                        field: 'templateName', name: $translate.instant('TemplateName'), width: '20%', headerCellClass: '',
                        cellTemplate: '<div class="ui-grid-cell-contents"><span title="{{row.entity.templateName}}">{{row.entity.templateName}}<span></div>'
                    },
                    {
                        field: 'serviceTypeName', name: $translate.instant('ServiceTypeName'), width: '20%', headerCellClass: '',
                        cellTemplate: '<div class="ui-grid-cell-contents"><span title="{{row.entity.serviceTypeName}}">{{row.entity.serviceTypeName}}<span></div>'
                    },
                    {
                        field: 'cell', name: $translate.instant('Cell'), width: '10%', headerCellClass: '',
                        cellTemplate: '<div class="ui-grid-cell-contents"><span title="{{row.entity.cell}}">{{row.entity.cell}}</span></div>'
                    },
                    {
                        field: 'rowName', name: $translate.instant('RowName'), width: '15%', headerCellClass: '',
                        cellTemplate: '<div class="ui-grid-cell-contents"><span title="{{row.entity.rowName}}">{{row.entity.rowName}}</span></div>'
                    },
                    {
                        field: 'columnName', name: $translate.instant('ColumnName'), width: '15%', headerCellClass: '',
                        cellTemplate: '<div class="ui-grid-cell-contents"><span title="{{row.entity.columnName}}">{{row.entity.columnName}}</span></div>'
                    },
                    {
                        field: 'formula', name: $translate.instant('Formula'), width: '20%', headerCellClass: '',
                        cellTemplate: '<div class="ui-grid-cell-contents"><span title="{{row.entity.formula}}">{{row.entity.formula}}</span></div>'
                    }
                ],
                onRegisterApi: function (gridApi) {
                    $scope.gridApi = gridApi;
                    // call resize every 500 ms for 5 s after modal finishes opening - usually only necessary on a bootstrap modal
                    $interval(function () {
                        $scope.gridApi.core.handleWindowResize();
                    }, 500, 60 * 60 * 8);
                }
            };
        };

        (function initializa() {

            $log.debug('SubjectCorrespondingController.ctor()...');

            $scope.stdAccountsCategories = [];
            $scope.selectedOrgName = '';
            $scope.selectedOrgID = '';
            $scope.checkedSet = {};
            $scope.selectedCopyOrgIDList = [];
            $scope.selectedOrg = {};

            $scope.directionList = [
                { id: 1, name: $translate.instant('StandardAccountDebit') },
                { id: -1, name: $translate.instant('StandardAccountCredit') },
            ];

            $scope.acctPropList = [
                { id: 1, name: $translate.instant('StandardAccountAcctPropAsset') },
                { id: 2, name: $translate.instant('StandardAccountAcctPropDebt') },
                { id: 3, name: $translate.instant('StandardAccountAcctPropCommon') },
                { id: 4, name: $translate.instant('StandardAccountAcctPropInterest') },
                { id: 5, name: $translate.instant('StandardAccountAcctPropCost') },
                { id: 6, name: $translate.instant('StandardAccountAcctPropProfitAndLoss') },
            ];

            loadEnterpriseAccountListOptions();
            loadStdAccountListOptions();
            loadReportGridOptions();
            loadOrgCheckboxTreeViewSettings();

            loadOrgMultiData();
            InitStdSubject();
            InitNgData();

            $scope.mapStdAccount = mapStdAccount;
            $scope.showDetail = showDetail;
            $scope.copyMappingCheck = copyMappingCheck;
            $scope.InitFilterAccount = InitFilterAccount;
            $scope.LoadEtsSubjectBySelectedOrg = LoadEtsSubjectBySelectedOrg;
            $scope.LoadEtsSubjectBySelectedAccpSetID = LoadEtsSubjectBySelectedAccpSetID;
            $scope.ClearMap = ClearMap;
            $scope.otherAwesomeCallbackSpan = otherAwesomeCallbackSpan;
            $scope.accountFilterChanged = accountFilterChanged;
            $scope.InitStdSubjectLinkEtsSubject = InitStdSubjectLinkEtsSubject;

            orgService.getOrgList(0).success(function (data) {
                if ($scope.selectNoOption) {
                    data.splice(0, 0, NoOptions);
                }
                $scope.selectedOrgName = data[0].name;
                $scope.selectedOrgID = data[0].id;
            });

            //初始化右侧标准科目的树,使用jquery fancytree的方式来显示树，所以需要用到ajax的方式
            //var stdAccountUrl = loginContext.apiHost + constant.webapi.prefix + '/stdAccount/stdAccountHierarchy?orgID=' + $scope.selectedOrgID;
            //commonWebService.initFancyTree('treetable', stdAccountUrl);

        })();
    }]);
systemConfigurationModule.directive('subjectCorresponding', ['$log',
    function ($log) {
        'use strict';
        $log.debug('subjectCorresponding.ctor()...');

        return {
            restrict: 'E',
            templateUrl: '/app/admin/systemConfiguration/subjectCorresponding/subject-corresponding.html' + '?_=' + Math.random(),
            replace: true,
            scope: {},
            controller: 'SubjectCorrespondingController',
            link: function (scope, element) {
                  
                Split(['#left-container', '#right-container'], {
                    //An array of initial sizes of the elements, specified as percentage values. Example: Setting the initial sizes to 25% and 75%.
                    sizes: [78, 22],

                    //An array of minimum sizes of the elements, specified as pixel values. Example: Setting the minimum sizes to 100px and 300px, respectively.
                    minSize: [750, 250],

                    onDragEnd: function () {
                        //$timeout(function () {
                        //    $rootScope.$broadcast(enums.vatEvent.layoutChanged, {});
                        //}, 700);
                    }
                });

                $(".multi-org-container").on("click", function (e) {
                    e.stopPropagation();
                });

                var ClearSelectedTreeNode = function () {
                    ClearSelectedTreeNodeRecursion(scope.OrgMultiData);
                    scope.selectedCopyOrgIDList = [];
                    scope.checkedSet = {};
                };


                //递归清除选中的节点
                var ClearSelectedTreeNodeRecursion = function (tree) {
                    tree.forEach(function (o) {
                        o.selected = false;
                        if (o.children) {
                            ClearSelectedTreeNodeRecursion(o.children);
                        }
                    });
                };

                //显示复制对应关系下的组织机构多选列表
                scope.displayMultiOrg = function () {

                    //ClearSelectedTreeNode();
                    scope.gridInstance.unselectAll();
                    if ($('.multi-org-container').is(":hidden")) {
                        $(".multi-org-container").show();
                    } else {
                        $(".multi-org-container").hide();
                    }

                    $(document).one("click", function () {
                        $(".multi-org-container").hide();
                    });
                    event.stopPropagation();
                };

                scope.closeMultiOrg = function () {
                    $(".multi-org-container").hide();
                };

                var previousOrgID, previousEspID;

                //企业科目列表和标准科目列表切换
                scope.SwitchAccount = function (flag) {
                    $('.subject-corresponding-container').toggle();
                    $('.std-subject-container').toggle();

                    var currentOrgID = scope.enterpriseAccountSetSelect && scope.enterpriseAccountSetSelect.id || '';
                    var currentEspID = scope.selectedOrgID;


                    if (flag == 'detail') {

                        if (previousEspID === currentEspID && currentOrgID === previousOrgID) {
                            return;
                        }

                        scope.InitStdSubjectLinkEtsSubject();

                        previousOrgID = currentOrgID;
                        previousEspID = currentEspID;
                    }
                };

                $(".filter-select").on("click", function (e) {
                    e.stopPropagation();
                });

                scope.showFilter = function () {
                    if ($('.filter-select').is(":hidden")) {
                        $(".filter-select").show();
                    } else {
                        $(".filter-select").hide();
                    }

                    $(document).one("click", function () {
                        $(".filter-select").hide();
                    });
                    event.stopPropagation();
                };
            }
        };
    }
]);
systemConfigurationModule
.controller('SubjectCorrespondingInfrastructureController', ['$scope', '$log',
    function ($scope, $log) {
        'use strict';

        (function initialize() {
            $log.debug('SubjectCorrespondingInfrastructureController.ctor()...');

        })();
    }
]);
systemConfigurationModule.directive('subjectCorrespondingInfrastructure', ['$log',
    function ($log) {
        'use strict';
        $log.debug('subjectCorrespondingInfrastructure.ctor()...');

        return {
            restrict: 'E',
            templateUrl: '/app/systemConfiguration/subjectCorresponding/subjectCorrespondingInfrastructure/subject-corresponding-infrastructure.html' + '?_=' + Math.random(),
            scope: {
                state: '='
            },
            controller: 'SubjectCorrespondingInfrastructureController',
            link: function (scope, element) {
            }
        };
    }
]);